/*
*
* Copyright (c) 2013,2014 Institute of Mathematics and Computer Science, University of Latvia (IMCS UL). 
*
* This file is part of DEEK (diagram editor engine kit). 
*
* You can redistribute it and/or modify it under the terms of the GNU General Public License 
* as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
*
* You should have received a copy of the GNU General Public License along with DEEK If not, see http://www.gnu.org/licenses/.
*
*/

package lv.lumii.deek.data;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * Collection of data properties which can be attached to any diagram element.
 * <p/>
 * Each property has a <strong>tag</strong> and a <strong>value</strong>.
 */
public interface DataProperties {

/**
 * Returns the value of the given property.
 *
 * @param tag property tag
 * @return property value
 */
String getDataString(String tag);

/**
 * Sets the value of a property to the given value.
 * Create property, if needed.
 *
 * @param tag   property tag
 * @param value new value
 */
void setDataString(String tag, String value);

//	Object getDataItem(String tag);
//	void setDataItem(String tag, Object value);

/**
 * Return the count of properties in this collection.
 *
 * @return
 */
int size();

/**
 * Save the properties to an XML element.
 *
 * @param doc org.w3c.dom.Document
 * @return org.w3c.dom.Element
 */
org.w3c.dom.Element saveToXML(Document doc);

}
