/*
*
* Copyright (c) 2013,2014 Institute of Mathematics and Computer Science, University of Latvia (IMCS UL). 
*
* This file is part of DEEK (diagram editor engine kit). 
*
* You can redistribute it and/or modify it under the terms of the GNU General Public License 
* as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
*
* You should have received a copy of the GNU General Public License along with DEEK If not, see http://www.gnu.org/licenses/.
*
*/

package lv.lumii.deek.data;

import lv.lumii.deek.model.DElement;
import lv.lumii.deek.model.XMLHelper;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import java.util.HashMap;
import java.util.Map;

/**
 * Implementation of the data properties collection.
 */
class DataPropertiesImpl implements DataProperties {

final static String KEY_ATTR_NAME = "key";
final static String VALUE_ATTR_NAME = "value";
final static String TYPE_ATTR_NAME = "type";
final static String TYPE_ATTR_VALUE_STRING = "string";
private Map<String, Object> dataMap;

public DataPropertiesImpl(DElement owner) {
    dataMap = new HashMap<>();
}

public DataPropertiesImpl(DElement owner, org.w3c.dom.Element element) {
    this(owner);
    loadFromXML(element);
}

@Override
public String getDataString(String tag) {
    Object o = dataMap.get(tag);

    if (o == null)
        return null;

    if (o instanceof String)
        return (String) o;

    return null;
}

@Override
public void setDataString(String tag, String value) {
    dataMap.put(tag, value);
}

@Override
public int size() {
    return dataMap.size();
}

@Override
public org.w3c.dom.Element saveToXML(Document doc) {

    org.w3c.dom.Element e1 = doc.createElement(XMLHelper.DATA_ELEMENT);
    org.w3c.dom.Element e2;

    // TODO add support for non-String value types
    for (Map.Entry<String, Object> e : dataMap.entrySet()) {
        String k = e.getKey();
        Object v = e.getValue();

        if (v instanceof String) {
            e2 = doc.createElement(XMLHelper.DATA_ITEM);
            e2.setAttribute(KEY_ATTR_NAME, k);
            e2.setAttribute(VALUE_ATTR_NAME, (String) v);
            e2.setAttribute(TYPE_ATTR_NAME, TYPE_ATTR_VALUE_STRING);
            e1.appendChild(e2);
        }
    }

    return e1;
}

private void loadFromXML(org.w3c.dom.Element element) {

    dataMap.clear();

    String key, val, type;

    NodeList entries = element.getChildNodes();
    for (int i = 0; i < entries.getLength(); i++) {
        Node n = entries.item(i);
        if (n instanceof Element) {
            org.w3c.dom.Element entry = (Element) n;
            if (entry.getTagName().equals(XMLHelper.DATA_ITEM)) {

                type = entry.getAttribute(TYPE_ATTR_NAME);

                if (type.equals(TYPE_ATTR_VALUE_STRING)) {
                    key = entry.getAttribute(KEY_ATTR_NAME);
                    val = entry.getAttribute(VALUE_ATTR_NAME);
                    dataMap.put(key, val);
                }

            }
        }
    }
}


}
