/*
*
* Copyright (c) 2013,2014 Institute of Mathematics and Computer Science, University of Latvia (IMCS UL). 
*
* This file is part of DEEK (diagram editor engine kit). 
*
* You can redistribute it and/or modify it under the terms of the GNU General Public License 
* as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
*
* You should have received a copy of the GNU General Public License along with DEEK If not, see http://www.gnu.org/licenses/.
*
*/

package lv.lumii.deek.model;

import lv.lumii.deek.style.Style;
import lv.lumii.deek.util.BlockSize;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import java.awt.*;
import java.awt.geom.Rectangle2D;

/**
 * Abstract layout block.
 */
abstract class Block {
protected static final String ATTR_TEXTBLOCK_TEXT = "text";
Pack owner;

/**
 * Draws this block in the provided rectangular area.
 *
 * @param g graphical context.
 * @param r drawing area.
 */
public void draw(Graphics2D g, Rectangle2D.Double r) {
    draw(g, r.x, r.y, r.width, r.height);
}

/**
 * Draw this block in the given area
 *
 * @param g
 * @param x
 * @param y
 * @param w
 * @param h
 */
protected abstract void draw(Graphics2D g, double x, double y, double w, double h);

/**
 * Measure dimensions of this block
 *
 * @return dimensions
 */
protected abstract BlockSize measure();

/**
 * Returns the style for this block.
 *
 * @return Style
 */
Style getStyle() {
    return owner.getStyle();
}

/**
 * Save this block to an XML element.
 *
 * @param doc org.w3c.dom.Document
 * @return Element describing this block
 */
abstract Element saveToXML(Document doc);

}
