/*
*
* Copyright (c) 2013,2014 Institute of Mathematics and Computer Science, University of Latvia (IMCS UL). 
*
* This file is part of DEEK (diagram editor engine kit). 
*
* You can redistribute it and/or modify it under the terms of the GNU General Public License 
* as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
*
* You should have received a copy of the GNU General Public License along with DEEK If not, see http://www.gnu.org/licenses/.
*
*/

package lv.lumii.deek.model;

import lv.lumii.deek.style.Style;
import lv.lumii.deek.util.BlockSize;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import java.awt.*;
import java.awt.geom.Line2D;
import java.awt.geom.Rectangle2D;

/**
 * Layout block which defines a separator line.
 */
class BlockSeparator extends Block {

private static final double SPACING = 3;
private static final double STEP = 2;
private SeparatorType separatorType = SeparatorType.SINGLE;

public BlockSeparator(Pack owner) {
    this.owner = owner;
}

public BlockSeparator(Pack owner, org.w3c.dom.Element elem) {
    this(owner);
    loadFromXML(elem);
}

@Override
public void draw(Graphics2D g, double x, double y, double w, double h) {

    g.setColor(getStyle().getStrokeColor());
    g.setStroke(getStyle().getStroke());

    switch (separatorType) {
        case DOUBLE:
            g.draw(new Line2D.Double(x, y + SPACING, x + w, y + SPACING));
            g.draw(new Line2D.Double(x, y + SPACING + STEP, x + w, y + SPACING + STEP));
            break;

        case SINGLE:
            g.draw(new Line2D.Double(x, y + SPACING, x + w, y + SPACING));
            break;

    }

}

@Override
public BlockSize measure() {
    return new BlockSize(0, getDimension());
}

@Override
org.w3c.dom.Element saveToXML(Document doc) {
    Element e = doc.createElement(XMLHelper.SEPARATOR_ELEMENT);
    e.setAttribute("type", separatorType.toString());
    return e;
}

private void loadFromXML(org.w3c.dom.Element elem) {
    String typeStr = elem.getAttribute("type");
    BlockSeparator.SeparatorType st = BlockSeparator.SeparatorType.valueOf(typeStr);
    separatorType = st;
}

double getDimension() {

    Style s = getStyle();

    switch (separatorType) {
        case DOUBLE:
            return 2 * SPACING + STEP;

        case SINGLE:

        default:
            return 2 * SPACING;
    }
}

enum SeparatorType {

    /**
     * single line
     */
    SINGLE,

    /**
     * double line
     */
    DOUBLE
}
}
