/*
*
* Copyright (c) 2013,2014 Institute of Mathematics and Computer Science, University of Latvia (IMCS UL). 
*
* This file is part of DEEK (diagram editor engine kit). 
*
* You can redistribute it and/or modify it under the terms of the GNU General Public License 
* as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
*
* You should have received a copy of the GNU General Public License along with DEEK If not, see http://www.gnu.org/licenses/.
*
*/

package lv.lumii.deek.model;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.*;
import java.awt.geom.Rectangle2D.Double;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedHashMap;

import lv.lumii.deek.style.BoxShapeType;
import lv.lumii.deek.style.Style;
import lv.lumii.deek.util.BlockSize;
import lv.lumii.deek.util.DUtils;
import lv.lumii.layoutengine.*;
import lv.lumii.layoutengine.Box;
import lv.lumii.layoutengine.LayoutConstraints.ConstraintType;
import lv.lumii.layoutengine.OutsideLabel.BoxOutsideLabel;
import lv.lumii.deek.util.Globals;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import javax.swing.*;

/**
 * Rectangular elements constituting the diagram.
 */
public class DBox extends DContainer {

private Pack contentPack = null;

public DBox(lv.lumii.layoutengine.Box box) {
    super(box);

    if (!Globals.isDeveloperMode()) {
        startTransaction();

        addText(getName());
        addSeparator();
        if (Globals.randomGenerator.nextBoolean()) {
            addText(DUtils.getRandomWord());
        }
        if (Globals.randomGenerator.nextBoolean()) {
            addText(DUtils.getRandomWord());
        }
        if (Globals.randomGenerator.nextBoolean()) {
            addText(DUtils.getRandomWord());
        }
        if (Globals.randomGenerator.nextBoolean()) {
            addText(DUtils.getRandomWord());
        }

        endTransaction();
    }
}

DBox(org.w3c.dom.Element e, LinkedHashMap<Integer, Object> objMap) {
    super(e, objMap);
}

@Override
protected void setupElementStyle(Style style) {
    // TODO
    style.delayUpdate(true);
    super.setupElementStyle(style);
    style.setFillColor(new Color(0.5f, 0.5f, 1.0f));
    style.setGradientColor(Color.WHITE);
    style.setStrokeColor(Color.DARK_GRAY);
    style.setStrokeWidth(2.0f);
    style.setTextColor(Color.WHITE);
    style.setFontSize(24);
    style.setFontBold(true);
    style.setBoxShapeType(BoxShapeType.ROUNDED_RECTANGLE);
    style.setShadow(true);
    style.setPadding(1);
    setSpacing(4);
    style.delayUpdate(false);
}

@Override
public lv.lumii.layoutengine.Box getLayoutElement() {
    return (lv.lumii.layoutengine.Box) layoutElement;
}

boolean hasChildren() {
    // TODO optimize this
    if (getLayoutElement().getBoxes().size() > 0)
        return true;
    if (getLayoutElement().getPureContainers().size() > 0)
        return true;
    if (getLayoutElement().getInsideLabels().size() > 0)
        return true;

    return false;
}

@Override
public void removeChildren() {
    super.removeChildren();

    for (DLine e : incidentLines())
        e.remove();
    for (DOutsideLabel e : outsideLabels())
        e.remove();
}

// -----------------------------------------
@Override
public Rectangle2D.Double getBounds() {
    return getLayoutElement().getBounds();
}

public void setMinWidth(double minWidth) {
    getLayoutElement().setMinWidth(minWidth);
}

public void setMinHeight(double minHeight) {
    getLayoutElement().setMinHeight(minHeight);
}

public void setMinSize(double width, double height) {
    getLayoutElement().setMinSize(width, height);
}

public void resize(Rectangle2D.Double rectangle) {
    getLayoutElement().resize(rectangle);
}

public void resize(double w, double h) {
    Rectangle2D.Double r = getBounds();
    r.width = w;
    r.height = h;
    getLayoutElement().resize(r);
}

public void move(Point2D.Double newCenter, DContainer newOwner, Point2D.Double growPoint, Integer gridRow, Integer gridColumn) {
    getLayoutElement().move(newCenter, newOwner.getLayoutElement(), growPoint, gridRow, gridColumn);
}

public void move(Point2D.Double newCenter, DContainer newOwner, Point2D.Double growPoint) {
    getLayoutElement().move(newCenter, newOwner.getLayoutElement(), growPoint);
}

public void move(Point2D.Double newCenter) {
    move(newCenter, getOwner(), newCenter);
}

@Override
public void drawSelf(Graphics2D g) {
    // TODO
    Shape thisShape = getShape();
    Rectangle2D.Double shapeBounds = (Double) thisShape.getBounds2D();

    // TODO uztaisīt glītāku ēnu
    if (style.hasShadow()) {
        AffineTransform oldTransform = g.getTransform();
        g.translate(Globals.BOX_SHADOW_SHIFT, -Globals.BOX_SHADOW_SHIFT);
        g.setPaint(Globals.SHADOW_COLOR);
        g.fill(thisShape);
        g.setTransform(oldTransform);
    }

    g.setPaint(style.getPaint(shapeBounds));
    g.fill(thisShape);

    if (isCurrent())
        g.setColor(Globals.CURRENT_COLOR);
    else
        g.setColor(style.getStrokeColor());
    g.setStroke(style.getStroke());
    g.draw(thisShape);

    drawContent(g);
}

@Override
protected void drawChildren(Graphics2D g) {

    super.drawChildren(g);

    for (DBoxLabel lbl : outsideLabels()) {
        lbl.draw(g);
    }
}

@Override
public BlockSize measure() {
    if (contentPack != null) {
        return contentPack.measure().pad(style.getPadding());
    } else {
        BlockSize dim = DUtils.measureText2(style.getFont(), getName());
        return dim.pad(style.getPadding());
    }
}

private void drawContent(Graphics2D g) {
    if (hasChildren())
        return;

    double scale = g.getTransform().getScaleX();
    if (scale < 0.5)
        return;

    if (contentPack != null) {
        Rectangle2D.Double rect = getBounds();
        contentPack.draw(g, rect.x + style.getPadding(), rect.y + style.getPadding(), rect.width - 2 * style.getPadding(), rect.height - 2 * style.getPadding());
    } else {
        g.setFont(style.getFont());
        g.setColor(style.getTextColor());

        BlockSize dim = DUtils.measureText(g, getName());

        Rectangle2D.Double r = getBounds();

        float xPos = (float) (r.getCenterX() - dim.width / 2 + style.getPadding());
        float yPos = (float) (r.getCenterY() + dim.height / 4 + style.getPadding());

        g.drawString(getName(), xPos, yPos);
    }
}

private void drawContent1(Graphics2D g) {
//		if (!hasChildren())
//			return;

    lv.lumii.layoutengine.Box b = getLayoutElement();

    double scale = g.getTransform().getScaleX();
    String label;
    String descr;

    if (scale > 0.6) {
        if (scale > 0.9) {
            label = getName();
            if (label.isEmpty())
                label = "Foo Bar";
            descr = getText1();
            if ((descr == null) || (descr.isEmpty()))
                descr = "some description";
        } else {
            label = "Foo...";
            descr = "...";
        }

        g.setColor(Globals.NORMAL_TEXT_COLOR);
        // TODO - set text color to opposite for fill color

        int fontSize1 = 10;
        int fontSize2 = fontSize1 / 2;
        Font font1, font2;

        font1 = new Font("Dialog", Font.PLAIN, fontSize1);
        g.setFont(font1);
        BlockSize dim1 = DUtils.measureText(g, label);
        font2 = new Font("Dialog", Font.PLAIN, fontSize2);
        g.setFont(font2);
        BlockSize dim2 = DUtils.measureText(g, descr);

        double ratioA = (b.getRight() - b.getLeft()) / Math.max(dim1.width, dim2.width);
        double ratioB = (b.getBottom() - b.getTop()) / (dim1.height + dim2.height);
        double ratio = Math.min(ratioA, ratioB) / 1.1;
        fontSize1 = (int) (fontSize1 * ratio);
        fontSize2 = (int) (fontSize1 / 2);

        font1 = new Font("Dialog", Font.PLAIN, fontSize1);
        g.setFont(font1);
        dim1 = DUtils.measureText(g, label);

        float xPos = (float) (b.getLeft() + (b.getRight() - b.getLeft() - dim1.width) / 2.0f);
        float yPos = (float) (b.getTop() + (b.getBottom() - b.getTop()) / 2.0f);


        g.drawString(label, xPos, yPos);


        font2 = new Font("Dialog", Font.ITALIC, fontSize2);
        g.setFont(font2);
        dim2 = DUtils.measureText(g, descr);

        xPos = (float) (b.getLeft() + (b.getRight() - b.getLeft() - dim2.width) / 2.0f);
        yPos = (float) (b.getBottom() - dim2.height / 2);

        g.drawString(descr, xPos, yPos);

    }
}

public Shape getShape() {
    double dim;
    Path2D.Double path;

    Rectangle2D.Double r = this.getLayoutElement().getBounds();


    switch (style.getBoxShapeType()) {
//            case RECTANGLE:
//                return r;

        case ROUNDED_RECTANGLE:
            return new RoundRectangle2D.Double(r.x, r.y, r.width, r.height,
                    Globals.BOX_ROUNDING, Globals.BOX_ROUNDING);

        case CIRCLE:
            dim = Math.min(r.height, r.width);
            r.setFrameFromCenter(r.getCenterX(), r.getCenterY(), r.getCenterX() - dim / 2, r.getCenterY() - dim / 2);

        case ELLIPSE:
            return new Ellipse2D.Double(r.x, r.y, r.width, r.height);

        case DIAMOND:
            // TODO
            dim = Math.min(r.height, r.width);
            r.setFrameFromCenter(r.getCenterX(), r.getCenterY(), r.getCenterX() - dim / 2, r.getCenterY() - dim / 2);
            path = new Path2D.Double();
            path.moveTo(r.x + dim / 2, r.y);
            path.lineTo(r.x + dim, r.y + dim / 2);
            path.lineTo(r.x + dim / 2, r.y + dim);
            path.lineTo(r.x, r.y + dim / 2);
            path.closePath();

            return path;

        default:
            return r;
    }
}

@Override
public DGroup convertToGroup() {

    dropLines();
    dropInsideLabels();

    lv.lumii.layoutengine.Box oldElement = getLayoutElement();

    lv.lumii.layoutengine.Container newElement = oldElement.convertToContainer(new Point2D.Double(0, 0), oldElement.getOwner());

    DGroup dGroup = new DGroup(newElement);

    dGroup.style = this.style;
    dGroup.data = this.data;

    DElement.removeThis(oldElement);

    return dGroup;
}

@Override
DGroup convertToGroup(Point2D.Double moveVector, DContainer owner) {
    lv.lumii.layoutengine.Container c = getLayoutElement().convertToContainer(moveVector, owner.getLayoutElement());
    return new DGroup(c);
}

@Override
DGroup convertToGroup(Point2D.Double moveVector, DContainer owner, Integer gridRow, Integer gridColumn) {
    lv.lumii.layoutengine.Container c = getLayoutElement().convertToContainer(moveVector, owner.getLayoutElement(), gridRow, gridColumn);
    return new DGroup(c);
}

@Override
public DBox convertToBox() {
    return this;
}

private void dropLines() {
    for (DLine l : lines()) {
        l.remove();
    }

}

private void dropInsideLabels() {
    for (DInsideLabel lbl : insideLabels()) {
        lbl.remove();
    }
}

@Override
protected String getTestInfoString() {

    lv.lumii.layoutengine.Box lb = getLayoutElement();
    Rectangle2D.Double r = (Rectangle2D.Double) lb.getBounds();

    String boxInfo = String.format("(%1.0f, %1.0f) (%1.0f x %1.0f) ",
            r.x, r.y, r.width, r.height);

    return super.getTestInfoString() + boxInfo;
}

@Override
protected void drawTestInfo(Graphics2D g) {
    if (log.isLoggingOn()) {
        lv.lumii.layoutengine.Box lb = getLayoutElement();
        Rectangle2D.Double r = (Rectangle2D.Double) lb.getBounds();

        g.drawString(getTestInfoString(), (float) r.x, (float) r.y);
    }
}

/**
 * Adds a text element after the last content element
 *
 * @param text text
 * @return this box
 */
public DBox addText(String text) {
    if (contentPack == null)
        contentPack = new Pack(this);
    contentPack.addText(text);
    recalculate();
    return this;
}

/**
 * Adds a separator between the previous and the next content element
 *
 * @return this box
 */
public DBox addSeparator() {
    if (contentPack == null)
        contentPack = new Pack(this);
    contentPack.addSeparator();
    recalculate();
    return this;
}

@Override
public boolean isSelfHit(Point2D.Double p) {
    return getLayoutElement().getBounds().contains(p);
}

@Override
public boolean isChildHit(Point2D.Double p) {
    for (DBoxLabel e : outsideLabels()) {
        if (e.isHit(p))
            return true;
    }

    return super.isChildHit(p);
}

@Override
public DElement getChildHit(Point2D.Double p) {

    for (DBoxLabel e : outsideLabels()) {

        if (e.isSelfHit(p))
            return e;
    }

    return super.getChildHit(p);
}

@Override
public boolean isHit(Point2D.Double p) {
    boolean hit = super.isHit(p);

    if (hit)
        return true;

    for (DBoxLabel lbl : outsideLabels()) {
        if (lbl.isHit(p))
            return true;
    }
    return false;
}

@Override
public DBox getHitBox(Point2D.Double p) {
    if (!isSelfHit(p))
        return null;

    DBox b2 = super.getHitBox(p);
    return b2 != null
            ? b2
            : this;
}

public Iterable<DBoxLabel> outsideLabels() {
    return new Iterable<DBoxLabel>() {
        @Override
        public Iterator<DBoxLabel> iterator() {
            return new Iterator<DBoxLabel>() {
                private ArrayList<BoxOutsideLabel> _items = getLayoutElement().getOutsideLabels();
                private int pos = 0;

                @Override
                public boolean hasNext() {
                    return pos < _items.size();
                }

                @Override
                public DBoxLabel next() {
                    return DElement.getThisBoxLabel(_items.get(pos++));
                }

                @Override
                public void remove() {

                }
            };
        }
    };
}

public Iterable<DLine> incidentLines() {
    return new Iterable<DLine>() {
        @Override
        public Iterator<DLine> iterator() {
            return new Iterator<DLine>() {
                private ArrayList<Line> _items = getLayoutElement().getIncidentLines();
                private int pos = 0;

                @Override
                public boolean hasNext() {
                    return pos < _items.size();
                }

                @Override
                public DLine next() {
                    return DLine.getThisLine(_items.get(pos++));
                }

                @Override
                public void remove() {

                }
            };
        }
    };
}

public DBoxLabel createOutsideLabel(double width, double height, Point2D.Double center, ConstraintType constraintType, double currentLineSpacing) {
    return new DBoxLabel(getLayoutElement().createOutsideLabel(width, height, center, constraintType, currentLineSpacing));
}

public DBoxLabel createOutsideLabel(double width, double height, Point2D.Double center) {
    return new DBoxLabel(getLayoutElement().createOutsideLabel(width, height, center, ConstraintType.NONE, Globals.getCurrentBoxSpacing()));
}

public DBoxLabel createOutsideLabel(double width, double height, Box.BoxSide side, double position, ConstraintType constraintType, double currentLineSpacing) {
    return new DBoxLabel(getLayoutElement().createOutsideLabel(width, height, side, position, constraintType, currentLineSpacing));
}

public DBoxLabel createOutsideLabel(double width, double height, Box.BoxSide side, double position) {
    return new DBoxLabel(getLayoutElement().createOutsideLabel(width, height, side, position, ConstraintType.NONE, Globals.getCurrentBoxSpacing()));
}

/**
 * Creates a new line connecting this box and the {@code endBox}.
 *
 * @param endBox             the other box
 * @param lineType           line type
 * @param currentLineSpacing spacing for the new line
 * @param points             points providing the desired trace
 * @param cleanUp            clean up the line trace
 * @return new line
 */
public DLine connectTo(DBox endBox, DLine.DLineType lineType, double currentLineSpacing, ArrayList<Point2D.Double> points, boolean cleanUp) {
    Line l = getLayoutElement().connectTo(endBox.getLayoutElement(), lineType.getInternalType(), currentLineSpacing, points, cleanUp);
    return new DLine(l, lineType);
}

/**
 * Creates a new line connecting this box and the {@code endBox}.
 *
 * @param endBox             the other box
 * @param lineType           line type
 * @param currentLineSpacing spacing for the new line
 * @param points             points providing the desired trace
 * @return the new line
 */
public DLine connectTo(DBox endBox, DLine.DLineType lineType, double currentLineSpacing, ArrayList<Point2D.Double> points) {
    return connectTo(endBox, lineType, currentLineSpacing, points, false);
}

/**
 * Creates a new line connecting this box and the {@code endBox}.
 *
 * @param endBox             the other box
 * @param lineType           line type
 * @param currentLineSpacing spacing for the new line
 * @return the new line
 */
public DLine connectTo(DBox endBox, DLine.DLineType lineType, double currentLineSpacing) {
    Line l = getLayoutElement().connectTo(endBox.getLayoutElement(), lineType.getInternalType(), currentLineSpacing);
    return new DLine(l, lineType);
}

/**
 * Creates a new line connecting this box and the {@code endBox}.
 *
 * @param endBox   the other box
 * @param lineType line type
 * @return the new line
 */
public DLine connectTo(DBox endBox, DLine.DLineType lineType) {
    return connectTo(endBox, lineType, Globals.getCurrentBoxSpacing());
}

/**
 * Creates a new line connecting this box and the {@code endBox}.
 *
 * @param endBox the other box
 * @return the new line
 */
public DLine connectTo(DBox endBox) {
    return connectTo(endBox, DLine.DLineType.STRAIGHT, Globals.getCurrentBoxSpacing());
}

@Override
protected JMenu getStyleSubmenu() {
    return style.getMenu(Style.StyleMenuType.BOX);
}

@Override
public JPopupMenu getElementMenu() {
    JPopupMenu menu = super.getElementMenu();

    menu.addSeparator();

    // line sides
    menu.add(getBoxLineSidesSubmenu(Commands.BOX_INCOMING_LINE_SIDES_MENU, Commands.BOX_INCOMING_LINE_SIDES_PREFIX, new ActionListener() {
        @Override
        public void actionPerformed(ActionEvent e) {
            String actionCommand = e.getActionCommand();
            if (actionCommand.equals(Commands.BOX_RESET_LINE_SIDES_COMMAND)) {
                for (DLine l : incidentLines()) {
                    l.resetEndSides();
                }
            } else {
                for (DLine l : incidentLines()) {
                    l.setEndSides(new Box.BoxSide[]{Box.BoxSide.valueOf(actionCommand)});
                }
            }
            updateAll();
        }
    }));

    menu.add(getBoxLineSidesSubmenu(Commands.BOX_OUTGOING_LINE_SIDES_MENU, Commands.BOX_OUTGOING_LINE_SIDES_PREFIX, new ActionListener() {
        @Override
        public void actionPerformed(ActionEvent e) {
            String actionCommand = e.getActionCommand();
            if (actionCommand.equals(Commands.BOX_RESET_LINE_SIDES_COMMAND)) {
                for (DLine l : incidentLines()) {
                    l.resetStartSides();
                }
            } else {
                for (DLine l : incidentLines()) {
                    l.setStartSides(new Box.BoxSide[]{Box.BoxSide.valueOf(actionCommand)});
                }
            }
            updateAll();
        }
    }));

    menu.addSeparator();

    menu.add(menuCommand(Commands.CONVERT_TO_GROUP, Commands.CONVERT_TO_GROUP_COMMAND, new ActionListener() {
        @Override
        public void actionPerformed(ActionEvent e) {
            convertToGroup();
            updateAll();
        }
    }));

    return menu;
}

@Override
public void recalculate() {
    BlockSize dim = measure();
    ensureSize(dim);
}

protected void ensureSize(BlockSize dim) {
//    resize(dim.width, dim.height);
    setMinSize(dim.width, dim.height);
}

private JMenu getBoxLineSidesSubmenu(String title, String prefix, ActionListener listener) {

    JMenuItem item;
    JMenu boxLineSidesSubmenu = new JMenu(title);

    for (Box.BoxSide bs : Box.BoxSide.values()) {
        item = new JMenuItem(bs.toString());
        item.addActionListener(listener);
        item.setActionCommand(bs.toString());
        boxLineSidesSubmenu.add(item);
    }
    item = new JMenuItem(Commands.BOX_RESET_LINE_SIDES);
    item.addActionListener(listener);
    item.setActionCommand(Commands.BOX_RESET_LINE_SIDES_COMMAND);
    boxLineSidesSubmenu.add(item);

    return boxLineSidesSubmenu;
}

@Override
void saveAttributesToXML(Element e) {
    super.saveAttributesToXML(e);
}

@Override
void loadAttributesFromXML(Element e) {
    super.loadAttributesFromXML(e);
}

@Override
protected void saveChildrenToXML(Element e, Document doc, LinkedHashMap<Object, Integer> idMap) {
    super.saveChildrenToXML(e, doc, idMap);

    Element eBoxLabels = doc.createElement(XMLHelper.BOX_LABELS_ELEMENT);

    for (DBoxLabel boxLabel : outsideLabels()) {
        eBoxLabels.appendChild(boxLabel.saveToXML(doc, idMap));
    }
    e.appendChild(eBoxLabels);
}

private static class Commands {

    static final String CONVERT_TO_GROUP = "Convert to group";
    static final String CONVERT_TO_GROUP_COMMAND = "group-converttogroup";
    static final String BOX_INCOMING_LINE_SIDES_MENU = "Incoming line sides";
    static final String BOX_INCOMING_LINE_SIDES_PREFIX = "boxlines-setincomingsides";
    static final String BOX_OUTGOING_LINE_SIDES_MENU = "Outgoing line sides";
    static final String BOX_OUTGOING_LINE_SIDES_PREFIX = "boxlines-setoutgoingsides";
    static final String BOX_RESET_LINE_SIDES = "Reset sides";
    static final String BOX_RESET_LINE_SIDES_COMMAND = "RESET";

}
}
