/*
*
* Copyright (c) 2013,2014 Institute of Mathematics and Computer Science, University of Latvia (IMCS UL). 
*
* This file is part of DEEK (diagram editor engine kit). 
*
* You can redistribute it and/or modify it under the terms of the GNU General Public License 
* as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
*
* You should have received a copy of the GNU General Public License along with DEEK If not, see http://www.gnu.org/licenses/.
*
*/

package lv.lumii.deek.model;

import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.LinkedHashMap;

import lv.lumii.layoutengine.OutsideLabel;

import javax.swing.*;

/**
 * Box label is a label which is attached to one of the sides of a {@code DBox}, outside of the box.
 * <p/>
 * Box labels have a box side at which they are located, and a position along this box side.
 * <p/>
 * Box labels can be manually <em>flipped</em> (rotated by 90 degrees) and <em>auto-flipped</em> (automatically flipped,
 * when they are moved to a different side of the box).
 * <p/>
 * When diagram is being rearranged, all box labels are placed without overlapping, if possible.
 */
public class DBoxLabel extends DOutsideLabel {

public DBoxLabel(OutsideLabel.BoxOutsideLabel label) {
    super(label);
}

DBoxLabel(org.w3c.dom.Element element, LinkedHashMap<Integer, Object> objMap) {
    super(element, objMap);
}

@Override
public lv.lumii.layoutengine.OutsideLabel.BoxOutsideLabel getLayoutElement() {
    return (lv.lumii.layoutengine.OutsideLabel.BoxOutsideLabel) layoutElement;
}

@Override
protected void drawSelf(Graphics2D g) {

    lv.lumii.layoutengine.OutsideLabel.BoxOutsideLabel lbl = getLayoutElement();
    Rectangle2D.Double r = getBounds();
    AffineTransform oldTransform = g.getTransform();

    if (lbl.isFlipped()) {
        g.translate(r.x, r.y);
        double angle = -Math.PI / 2;
        g.rotate(angle);

        g.translate(-r.height, 0);
    } else {
        g.translate(r.x, r.y);

    }

    drawContent(g);

    g.setTransform(oldTransform);


}

@Override
public DBox getOwner() {
    return getThisBox(getLayoutElement().getOwner());
}

public void move(Point2D.Double point, DBox owner) {
    getLayoutElement().move(point, owner.getLayoutElement());
}

@Override
public JPopupMenu getElementMenu() {
    JPopupMenu menu = super.getElementMenu();

//        menu.addSeparator();

    // ...

    return menu;
}
}
