/*
*
* Copyright (c) 2013,2014 Institute of Mathematics and Computer Science, University of Latvia (IMCS UL). 
*
* This file is part of DEEK (diagram editor engine kit). 
*
* You can redistribute it and/or modify it under the terms of the GNU General Public License 
* as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
*
* You should have received a copy of the GNU General Public License along with DEEK If not, see http://www.gnu.org/licenses/.
*
*/

package lv.lumii.deek.model;

import lv.lumii.deek.style.Style;
import lv.lumii.deek.ui.DDrawing;
import lv.lumii.layoutengine.Container;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.LinkedHashMap;

/**
 * Diagram element which is similar to a {@code DBox} but which does not have a rectangular shape and
 * which does not occupy any exclusive area on the diagram canvas.
 */
public class DGroup extends DContainer {

private static double GROUP_PADDING = 3;
private static Stroke GROUP_STROKE = new BasicStroke(
        1.0f, BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER,
        2.0f, new float[]{5.0f}, 0.0f);
private static Color GROUP_COLOR = Color.RED;


public DGroup(lv.lumii.layoutengine.Container container) {
    super(container);
}

DGroup(org.w3c.dom.Element element, LinkedHashMap<Integer, Object> objMap) {
    super(element, objMap);
}

@Override
protected void setupElementStyle(Style style) {
    // TODO
    style.delayUpdate(true);
    style.setStrokeColor(Color.ORANGE);
    style.setFillColor(new Color(0.0f, 1.0f, 1.0f, 0.1f));
    super.setupElementStyle(style);
    style.delayUpdate(false);
}

@Override
protected void drawSelf(Graphics2D g) {

    Shape r = getLayoutElement().getShape();
    if (r == null) {
        return;
    }


    g.setStroke(GROUP_STROKE);
    g.setColor(style.getStrokeColor());

    g.draw(r);

}

@Override
public boolean isSelfHit(Point2D.Double p) {
/*
    Shape s = getLayoutElement().getShape();
    if (s == null)
        return false;

    if (s instanceof Rectangle2D.Double)
        return (s.contains(p));

    if (s instanceof Path2D.Double)
        return (s.contains(p));
*/
    return false;
}

public Rectangle2D.Double getEnclosure() {

    Rectangle2D.Double r = getLayoutElement().findDescendantEnclosingRectangle();

    r.x -= GROUP_PADDING;
    r.y -= GROUP_PADDING;
    r.width += 2 * GROUP_PADDING;
    r.height += 2 * GROUP_PADDING;

    return r;
}

@Override
public DBox convertToBox() {

    lv.lumii.layoutengine.Container oldElement = (lv.lumii.layoutengine.Container) layoutElement;

    Rectangle2D.Double enclRect = oldElement.findDescendantEnclosingRectangle();

    if (enclRect == null)
        enclRect = new Rectangle2D.Double();

    Point2D.Double center = new Point2D.Double(enclRect.getCenterX(), enclRect.getCenterY());

    lv.lumii.layoutengine.Box newElement = oldElement.convertToBox(center, oldElement.getOwner(), center);

    DBox dBox = new DBox(newElement);
    dBox.style = this.style;
    dBox.data = this.data;

    removeThis(oldElement);

    return dBox;
}

public void ungroup() {
    DContainer parent = getOwner();
    Container thisElement = getLayoutElement();

    Point2D.Double nullVector = new Point2D.Double();
    DDrawing drawing = getDrawing();
    drawing.startTransaction();

    for (DContainer c : containers()) {
        c.move(nullVector, parent);
    }
    for (DInsideLabel lbl : insideLabels()) {
        lbl.move(lbl.getCenter(), parent, lbl.getCenter());
    }

    unlinkFromLayout();
    thisElement.remove();
    drawing.endTransaction();
}

@Override
public DGroup convertToGroup() {
    return this;
}

@Override
DGroup convertToGroup(Point2D.Double moveVector, DContainer owner) {
    return null;
}

@Override
DGroup convertToGroup(Point2D.Double moveVector, DContainer owner, Integer gridRow, Integer gridColumn) {
    return null;
}

@Override
public JPopupMenu getElementMenu() {
    JPopupMenu menu = super.getElementMenu();

    menu.addSeparator();

    menu.add(menuCommand(Commands.CONVERT_TO_BOX, Commands.CONVERT_TO_BOX_COMMAND, new ActionListener() {
        @Override
        public void actionPerformed(ActionEvent e) {
            convertToBox();
            updateAll();
        }
    }));

    menu.add(menuCommand(Commands.GROUP_UNGROUP, Commands.GROUP_UNGROUP_COMMAND, new ActionListener() {
        @Override
        public void actionPerformed(ActionEvent e) {
            DDrawing drawing = getDrawing();
            ungroup();
            drawing.requestUpdate();
        }
    }));

    return menu;
}

@Override
public void recalculate() {

}

protected static class Commands {
    // do not use ':' in commands and prefixes, as it is used as separator here

    static final String GROUP_UNGROUP = "Ungroup";
    static final String GROUP_UNGROUP_COMMAND = "group-ungroup";
    static final String CONVERT_TO_BOX = "Convert to box";
    static final String CONVERT_TO_BOX_COMMAND = "group-converttobox";
}
}
