/*
*
* Copyright (c) 2013,2014 Institute of Mathematics and Computer Science, University of Latvia (IMCS UL). 
*
* This file is part of DEEK (diagram editor engine kit). 
*
* You can redistribute it and/or modify it under the terms of the GNU General Public License 
* as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
*
* You should have received a copy of the GNU General Public License along with DEEK If not, see http://www.gnu.org/licenses/.
*
*/

package lv.lumii.deek.model;

import java.awt.*;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.LinkedHashMap;

import lv.lumii.deek.style.Style;
import lv.lumii.deek.util.BlockSize;
import lv.lumii.deek.util.Globals;
import lv.lumii.deek.util.DUtils;
import lv.lumii.layoutengine.InsideLabel;

import javax.swing.*;

/**
 * Inside labels are labels which can be freely placed inside any container (box, group, or diagram).
 * <p/>
 * When diagram is beeing rearranged, inside labels are placed non-overlapping with other boxes and inside labels.
 */
public class DInsideLabel extends DLabel {

private boolean flipThis = false;

public DInsideLabel(InsideLabel label) {
    super(label);
}

DInsideLabel(org.w3c.dom.Element element, LinkedHashMap<Integer, Object> objMap) {
    super(element, objMap);
}

@Override
protected void setupElementStyle(Style style) {
    // TODO
}

@Override
public lv.lumii.layoutengine.InsideLabel getLayoutElement() {
    return (lv.lumii.layoutengine.InsideLabel) layoutElement;
}

@Override
protected void drawSelf(Graphics2D g) {
    Rectangle2D.Double r = getBounds();
    AffineTransform oldTransform = g.getTransform();

    g.translate(r.x, r.y);

    drawContent(g);

    g.setTransform(oldTransform);

    // for testing
    g.setStroke(Globals.LINE_STROKE_PHANTOM);
    g.setColor(Globals.LABEL_BORDER_COLOR_PHANTOM);
    g.draw(getBounds());
//
//		Rectangle2D.Double r = getBounds();
//
//		g.setPaint(Globals.LABEL_COLOR);
//		g.fill(r);
//
//		g.setPaint(Globals.LABEL_BORDER_COLOR);
//		g.draw(r);
//		g.draw(new Line2D.Double(r.x, r.y, r.x + r.width, r.y + r.height));
//		g.draw(new Line2D.Double(r.x + r.width, r.y, r.x, r.y + r.height));
}

@Override
boolean isFlipped() {
    // TODO
    return flipThis;
}

void flip() {
    // todo
}

@Override
public DContainer getOwner() {
    return DContainer.getThisContainer(getLayoutElement().getOwner());
}

@Override
public void resize(Rectangle2D.Double r) {
    String text = getName();
    BlockSize dim = DUtils.measureText2(style.getFont(), text).pad(style.getPadding());

    r.width = Math.max(r.width, dim.width);
    r.height = Math.max(r.height, dim.height);

    getLayoutElement().resize(r);
}

public void move(Point2D.Double newCenter, DContainer newOwner, Point2D.Double growPoint, Integer gridRow, Integer gridColumn) {
    getLayoutElement().move(newCenter, newOwner.getLayoutElement(), growPoint, gridRow, gridColumn);
}

public void move(Point2D.Double newCenter, DContainer newOwner, Point2D.Double growPoint) {
    getLayoutElement().move(newCenter, newOwner.getLayoutElement(), growPoint);
}

@Override
public JPopupMenu getElementMenu() {
    JPopupMenu menu = super.getElementMenu();

//        menu.addSeparator();

    // ...

    return menu;
}
}
