/*
*
* Copyright (c) 2013,2014 Institute of Mathematics and Computer Science, University of Latvia (IMCS UL). 
*
* This file is part of DEEK (diagram editor engine kit). 
*
* You can redistribute it and/or modify it under the terms of the GNU General Public License 
* as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
*
* You should have received a copy of the GNU General Public License along with DEEK If not, see http://www.gnu.org/licenses/.
*
*/

package lv.lumii.deek.model;

import lv.lumii.layoutengine.OutsideLabel;
import org.w3c.dom.Element;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.Rectangle2D;
import java.util.LinkedHashMap;

/**
 * An abstract class describing common properties of box labels and line labels.
 */
public class DOutsideLabel extends DLabel {

public DOutsideLabel(OutsideLabel label) {
    super(label);
}

public DOutsideLabel(Element element, LinkedHashMap<Integer, Object> objMap) {
    super(element, objMap);
}

@Override
public lv.lumii.layoutengine.OutsideLabel getLayoutElement() {
    return (lv.lumii.layoutengine.OutsideLabel) layoutElement;
}

@Override
public boolean isFlipped() {
    return getLayoutElement().isFlipped();
}

public void flip() {
    getLayoutElement().flip();
}

public boolean isAutoFlipped() {
    return getLayoutElement().isAutoFlipped();
}

public void setAutoFlip(boolean b) {
    getLayoutElement().setAutoFlip(b);
}

@Override
public void resize(Rectangle2D.Double r) {
    getLayoutElement().resize(r);
}

@Override
public JPopupMenu getElementMenu() {
    JPopupMenu menu = super.getElementMenu();

    menu.addSeparator();

    menu.add(menuCommand(Commands.OUTSIDE_LABEL_FLIP, Commands.OUTSIDE_LABEL_FLIP_COMMAND, new ActionListener() {
        @Override
        public void actionPerformed(ActionEvent e) {
            flip();
            updateAll();
        }
    },
            isFlipped()
    ));

    menu.add(menuCommand(Commands.OUTSIDE_LABEL_AUTOFLIP, Commands.OUTSIDE_LABEL_AUTOFLIP_COMMAND, new ActionListener() {
        @Override
        public void actionPerformed(ActionEvent e) {
            setAutoFlip(!isAutoFlipped());
            updateAll();
        }
    },
            isAutoFlipped()
    ));

    // ...

    return menu;
}

protected static class Commands {
    // do not use ':' in commands and prefixes, as it is used as separator here

    static final String OUTSIDE_LABEL_AUTOFLIP = "AutoFlip";
    static final String OUTSIDE_LABEL_AUTOFLIP_COMMAND = "outsidelabel-setautoflip";
    static final String OUTSIDE_LABEL_FLIP = "Flip";
    static final String OUTSIDE_LABEL_FLIP_COMMAND = "outsidelabel-flip";
}
}
