/*
*
* Copyright (c) 2013,2014 Institute of Mathematics and Computer Science, University of Latvia (IMCS UL). 
*
* This file is part of DEEK (diagram editor engine kit). 
*
* You can redistribute it and/or modify it under the terms of the GNU General Public License 
* as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
*
* You should have received a copy of the GNU General Public License along with DEEK If not, see http://www.gnu.org/licenses/.
*
*/

package lv.lumii.deek.model;

import lv.lumii.deek.style.Style;
import lv.lumii.deek.style.StyleFactory;
import lv.lumii.deek.util.BlockSize;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import java.awt.*;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;

/**
 * Layout block.
 */
class Pack extends Block {

private ArrayList<Block> items;
//Style style = null;
boolean drawBorder = false;
boolean fill = false;
DElement ownerElement = null;

public Pack() {
    items = new ArrayList<>();
//    style = StyleFactory.createPackStyle();
}

public Pack(DElement ownerElement) {
    items = new ArrayList<>();
    this.ownerElement = ownerElement;
}

@Override
Style getStyle() {
    if (ownerElement != null)
        return ownerElement.style;
    return owner.getStyle();
}

/**
 * Add a text block
 * @param text text for the new block
 */
public void addText(String text) {
    items.add(new TextBlock(this, text));
}

/**
 * Add a separator line
 */
public void addSeparator() {
    items.add(new BlockSeparator(this));
}

/**
 * Clear all contents in this pack.
 */
public void clear() {
    items.clear();
}

/**
 * Draw this pack
 * @param g graphics context
 * @param x x coordinate
 * @param y y coordinate
 * @param w area width
 * @param h area height
 */
@Override
public void draw(Graphics2D g, double x, double y, double w, double h) {

    BlockSize ownSize = measure();

    double currentX = x;
    double currentY = y;
//    if (ownSize.width < w) {
//        currentX += (w - ownSize.width) / 2;
//    }
//    if (ownSize.height < h) {
//        currentY += (h - ownSize.height) / 2;
//    }
    ownSize.growTo(w, h);

    BlockSize childSize;

    for (Block block : items) {
        childSize = block.measure();
        block.draw(g, currentX, currentY, ownSize.width, childSize.height);
        currentY += childSize.height;
    }

}

/**
 * Measure this block
 * @return
 */
@Override
public BlockSize measure() {

    double totalWidth = 0;
    double totalHeight = 0;
    BlockSize childSize;

    for (Block block : items) {
        childSize = block.measure();
        totalHeight += childSize.height;
        totalWidth = Math.max(totalWidth, childSize.width);
    }

    return new BlockSize(totalWidth, totalHeight);
}


/**
 * Save this block to XML
 * @param doc org.w3c.dom.Document
 * @return element created
 */
@Override
Element saveToXML(org.w3c.dom.Document doc) {
    Element e = doc.createElement(XMLHelper.PACK_ELEMENT);

    for (Block block : items) {
        e.appendChild(block.saveToXML(doc));
    }

    return e;
}

Pack(DElement ownerElement, org.w3c.dom.Element e) {
    this(ownerElement);
    loadFromXML(e);
}

private void loadFromXML(Element e) {
    items.clear();
    NodeList blockList = e.getChildNodes();
    for (int i = 0; i < blockList.getLength(); i++) {
        Node n = blockList.item(i);
        if (n instanceof Element) {
            Element elem = (Element) n;
            String elemName = elem.getTagName();
            switch (elemName) {
                case XMLHelper.PACK_ELEMENT:
                    Pack p = new Pack(ownerElement, elem); // TODO
                    items.add(p);
                    break;

                case XMLHelper.TEXTBLOCK_ELEMENT:
                    TextBlock tb = new TextBlock(this, elem);
                    items.add(tb);
                    break;

                case XMLHelper.SEPARATOR_ELEMENT:
                    BlockSeparator bs = new BlockSeparator(this, elem);
                    items.add(bs);
                    break;
            }
        }
    }
}


}
