/*
*
* Copyright (c) 2013,2014 Institute of Mathematics and Computer Science, University of Latvia (IMCS UL). 
*
* This file is part of DEEK (diagram editor engine kit). 
*
* You can redistribute it and/or modify it under the terms of the GNU General Public License 
* as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
*
* You should have received a copy of the GNU General Public License along with DEEK If not, see http://www.gnu.org/licenses/.
*
*/

package lv.lumii.deek.model;

import lv.lumii.deek.style.Style;
import lv.lumii.deek.util.BlockSize;
import lv.lumii.deek.util.DUtils;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import java.awt.*;

/**
 * Layout block containing simple text
 */
public class TextBlock extends Block {
private String myText;

public TextBlock(Pack owner) {
    this(owner, "");
}

public TextBlock(Pack owner, String text) {
    myText = text;
    this.owner = owner;
}

public TextBlock(Pack owner, Element elem) {
    this(owner);
    loadFromXML(elem);
}

@Override
public void draw(Graphics2D g, double x, double y, double w, double h) {
    Style s = getStyle();
    g.setColor(s.getTextColor());
    g.setFont(s.getFont());
    g.drawString(myText, (float) (x + s.getPadding()), (float) (y + s.getPadding() + h * 0.75));
}

@Override
public BlockSize measure() {
    BlockSize dim = DUtils.measureText2(getStyle().getFont(), myText);
    dim.pad(getStyle().getPadding());
    return dim;
}

void loadFromXML(Element e) {
    myText = e.getAttribute(ATTR_TEXTBLOCK_TEXT);
}

@Override
Element saveToXML(Document doc) {
    Element e = doc.createElement(XMLHelper.TEXTBLOCK_ELEMENT);
    e.setAttribute(ATTR_TEXTBLOCK_TEXT, myText);
    return e;
}
}
