/*
*
* Copyright (c) 2013,2014 Institute of Mathematics and Computer Science, University of Latvia (IMCS UL). 
*
* This file is part of DEEK (diagram editor engine kit). 
*
* You can redistribute it and/or modify it under the terms of the GNU General Public License 
* as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
*
* You should have received a copy of the GNU General Public License along with DEEK If not, see http://www.gnu.org/licenses/.
*
*/

package lv.lumii.deek.test;

import java.awt.geom.Rectangle2D;

import lv.lumii.deek.model.DLine;

/**
 * Diagram content automatic generation API - for testing purposes.
 */
public interface ContentGenerator {

/**
 * Generate N random boxes at the diagram level
 *
 * @param numBoxes Number of random boxes to be generated
 * @param area     Rectangular area where to place the boxes
 */
void generateRandomBoxes(int numBoxes, Rectangle2D.Double area);

/**
 * Generate N random boxes, nested in existing boxes
 *
 * @param numBoxes Number of random nested boxes to be generated
 */
void generateNestedRandomBoxes(int numBoxes);

/**
 * Generate N random lines
 *
 * @param numLines Number of random lines to be generated
 * @param lineType Type of lines
 */
void generateRandomLines(int numLines, DLine.DLineType lineType);

/**
 * Generate N lines from a random box to the nearest neighbor box
 *
 * @param numLines Number of random lines to be generated
 * @param lineType Type of lines
 */
void generateShortRandomLines(int numLines, DLine.DLineType lineType);

/**
 * Generate N random lines, avoiding multi-edges
 *
 * @param numLines Number of random lines to be generated
 * @param lineType Type of lines
 */
void generateSingleRandomLines(int numLines, DLine.DLineType lineType);

/**
 * Generate tree with N nodes and (N-1) edges
 *
 * @param numBoxes Number of nodes in the tree
 * @param area     Rectangular area where to place the boxes
 * @param lineType Type of lines
 */
void generateTree(int numBoxes, Rectangle2D.Double area, DLine.DLineType lineType);

void generateRandomInsideLabels(int numLabels);

void generateRandomOutsideLabels(int numLabels);

void generateRandomLineLabels(int numLabels);

//    void addRandomBox(Rectangle2D.Double area);
//    void addRandomNestedBox();
//    void addRandomLine(LineType lineType);

/**
 * Show/hide the generator options dialog
 */
void showHideOptions();

}
