/*
*
* Copyright (c) 2013,2014 Institute of Mathematics and Computer Science, University of Latvia (IMCS UL). 
*
* This file is part of DEEK (diagram editor engine kit). 
*
* You can redistribute it and/or modify it under the terms of the GNU General Public License 
* as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
*
* You should have received a copy of the GNU General Public License along with DEEK If not, see http://www.gnu.org/licenses/.
*
*/

package lv.lumii.deek.test;

import java.awt.Frame;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import lv.lumii.deek.model.*;
import lv.lumii.deek.ui.DDrawing;
import lv.lumii.deek.util.DLogger;
import lv.lumii.layoutengine.ArrangeData.ArrangeStyle;
import lv.lumii.layoutengine.Box.BoxSide;
import lv.lumii.layoutengine.Element;
import lv.lumii.layoutengine.InsideLabel;
import lv.lumii.layoutengine.LayoutConstraints.ConstraintType;
import lv.lumii.layoutengine.LayoutConstraints.GridLayoutConstraints;
import lv.lumii.layoutengine.OutsideLabel;
import lv.lumii.layoutengine.OutsideLabel.LineLabel;
import lv.lumii.layoutengine.OutsideLabel.LineLabel.Orientation;
import lv.lumii.deek.util.Globals;

public class ContentManipulatorOne
        implements ContentManipulator {

private DDrawing drawing;
private Globals globals = Globals.getInstance();
private DLogger log = new DLogger();

public ContentManipulatorOne(DDrawing drawing) {
    this.drawing = drawing;
}

@Override
public void generateRandomBoxes(int numBoxes, Rectangle2D.Double area) {

    if (numBoxes < 1) return;

    preGeneration(String.format("generate %d random boxes", numBoxes));

    for (int i = 0; i < numBoxes; i++) {
        addRandomBox(area);
    }

    postGeneration();

}

@Override
public void generateNestedRandomBoxes(int numBoxes) {

    if (numBoxes < 1) return;

    SortedMap<Long, DBox> boxMap = new TreeMap<>();
    for (DBox b : drawing.getDiagram().allBoxes()) {
        boxMap.put(b.getUuid(), b);
    }

    List<DBox> allBoxes = new ArrayList<>();

    for (Map.Entry<Long, DBox> entry : boxMap.entrySet()) {
        allBoxes.add(entry.getValue());
    }

    int numOldBoxes = allBoxes.size();
    if (numOldBoxes < 1) return;

    preGeneration(String.format("generate %d nested random boxes", numBoxes));

    int parentIndex;
    DBox parent;
    Rectangle2D.Double parentRect, childRect;

    for (int i = 0; i < numBoxes; i++) {
        parentIndex = Globals.randomGenerator.nextInt(numOldBoxes);
        parent = allBoxes.get(parentIndex);
        parentRect = parent.getBounds();

        childRect = getRandomSubboxRect(parentRect);

        parent.createBox(childRect);
    }

    postGeneration();

}

@Override
public void generateTree(int numBoxes, Rectangle2D.Double area, DLine.DLineType lineType) {

    if (numBoxes < 2) return;

    drawing.clear();

    preGeneration(String.format("generate tree, step 1: generate %d random boxes", numBoxes));

    for (int i = 0; i < numBoxes; i++) {
        addRandomBox(area);
    }
    postGeneration();

    boolean generateTreeFromRoot = Globals.randomGenerator.nextBoolean();

    preGeneration(String.format("generate tree, step 2: generate %d connecting lines", numBoxes - 1));

    int treeIndex, freeIndex;
    DBox b0, b1;

    List<DBox> freeBoxes = new ArrayList<>();
    for (DBox b : drawing.getDiagram().allBoxes()) {
        freeBoxes.add(b);
    }
    List<DBox> treeBoxes = new ArrayList<>();

    freeIndex = Globals.randomGenerator.nextInt(freeBoxes.size());
    b1 = freeBoxes.get(freeIndex);
    freeBoxes.remove(freeIndex);
    treeBoxes.add(b1);

    while (freeBoxes.size() > 0) {
        treeIndex = Globals.randomGenerator.nextInt(treeBoxes.size());
        b0 = treeBoxes.get(treeIndex);

        freeIndex = Globals.randomGenerator.nextInt(freeBoxes.size());
        b1 = freeBoxes.get(freeIndex);
        freeBoxes.remove(freeIndex);
        treeBoxes.add(b1);

        if (generateTreeFromRoot) {

            b0.connectTo(b1, lineType);

        } else {

            // generate multi-root tree (grow up and down)
            if (Globals.randomGenerator.nextBoolean()) {
                b0.connectTo(b1, lineType);
            } else {
                b1.connectTo(b0, lineType);
            }

        }


    }

    postGeneration();

}

@Override
public void generateRandomLines(int numLines, DLine.DLineType lineType) {

    List<DBox> allBoxes = new ArrayList<>();
    for (DBox b : drawing.getDiagram().allBoxes()) {
        allBoxes.add(b);
    }
    int numBoxes = allBoxes.size();

    if (numBoxes < 2) return;

    preGeneration(String.format("generate %d random lines of type %s", numLines, lineType));

    // nesauc N reizes addRandomLine, lai nesauktu daudzas reizes getAllBoxes()

    for (int i = 0; i < numLines; i++) {
        int startBoxIndex = Globals.randomGenerator.nextInt(numBoxes);
        int endBoxIndex = (startBoxIndex + 1 + Globals.randomGenerator.nextInt(numBoxes - 1)) % numBoxes;

        allBoxes.get(startBoxIndex).connectTo(allBoxes.get(endBoxIndex), lineType);

    }

    postGeneration();

}

@Override
public void generateSingleRandomLines(int numLines, DLine.DLineType lineType) {

    List<DBox> allBoxes = getAllBoxes();
    int numBoxes = allBoxes.size();

    if (numBoxes < 2) return;

    int numExistingLines = drawing.getLineCount();

    int maxNumLines = numBoxes * (numBoxes - 1);     // orientets

    int numNewLinesTarget = numLines + numExistingLines <= maxNumLines ? numLines : maxNumLines - numExistingLines;

    preGeneration(String.format("generate %d single random lines of type %s", numLines, lineType));

    // nesauc N reizes addRandomLine, lai nesauktu daudzas reizes getAllBoxes()

    int numNewLines = 0;
    while (numNewLines < numNewLinesTarget) {

        int startBoxIndex = Globals.randomGenerator.nextInt(numBoxes);
        int endBoxIndex = (startBoxIndex + 1 + Globals.randomGenerator.nextInt(numBoxes - 1)) % numBoxes;

        DBox b1 = allBoxes.get(startBoxIndex);
        DBox b2 = allBoxes.get(endBoxIndex);

        boolean lineExists = false;
        for (DLine l : b1.incidentLines()) {
            if ((l.getStart() == b1) && (l.getEnd() == b2)) {
                lineExists = true;
                break;
            }
        }
        if (!lineExists) {
            b1.connectTo(b2, lineType);
            numNewLines++;
        }
    }

    postGeneration();


}

@Override
public void generateShortRandomLines(int numLines, DLine.DLineType lineType) {

    if (numLines < 1) return;

    List<DBox> allBoxes = getAllBoxes();
    int numBoxes = allBoxes.size();
    if (numBoxes < 2) return;

    preGeneration(String.format("generate %d short random lines of type %s", numLines, lineType));

    // nesauc N reizes addRandomLine, lai nesauktu daudzas reizes getAllBoxes()

    DBox b1, b2;
    double c1x, c1y, c2x, c2y;
    double minDist;
    double currDist;
    int startBoxIndex, endBoxIndex;

    for (int i = 0; i < numLines; i++) {

        startBoxIndex = Globals.randomGenerator.nextInt(numBoxes);

        b1 = allBoxes.get(startBoxIndex);
        c1x = b1.getBounds().getCenterX();
        c1y = b1.getBounds().getCenterY();

        minDist = java.lang.Double.MAX_VALUE;
        endBoxIndex = -1;

        for (int i2 = 0; i2 < numBoxes; i2++) {

            // skip the line to itself
            if (i2 != startBoxIndex) {
                b2 = allBoxes.get(i2);

                c2x = b2.getBounds().getCenterX();
                c2y = b2.getBounds().getCenterY();

                currDist = Math.abs(c1x - c2x) + Math.abs(c1y - c2y);
                if (currDist < minDist) {
                    minDist = currDist;
                    endBoxIndex = i2;
                }
            }

        }

        b1.connectTo(allBoxes.get(endBoxIndex), lineType);

    }

    postGeneration();

}

@Override
public void generateRandomInsideLabels(int numLabels) {

    if (numLabels < 1) return;

    List<DBox> allBoxes = getAllBoxes();
    int numBoxes = allBoxes.size();
    if (numBoxes < 1) return;

    preGeneration(String.format("generate %d random inside labels", numLabels));

    for (int i = 0; i < numLabels; i++) {
        DBox b = pickRandomBox();
        addRandomInsideLabel(b);
    }

    postGeneration();

}

@Override
public void generateRandomOutsideLabels(int numLabels) {

    if (numLabels < 1) return;

    List<DBox> allBoxes = getAllBoxes();
    int numBoxes = allBoxes.size();
    if (numBoxes < 1) return;

    preGeneration(String.format("generate %d random outside labels", numLabels));

    for (int i = 0; i < numLabels; i++) {
        DBox b = pickRandomBox();
        addRandomOutsideLabel(b);
    }

    postGeneration();

}


//	private long inTime;
//	private long lapTime;

@Override
public void generateRandomLineLabels(int numLabels) {

    if (numLabels < 1) return;

    List<DLine> allLines = getAllLines();

    int numLines = allLines.size();
    if (numLines < 1) return;

    preGeneration(String.format("generate %d random line labels", numLabels));

    for (int i = 0; i < numLabels; i++) {
        DLine l = pickRandomLine();
        addRandomLineLabel(l);
    }

    postGeneration();

}

private void preGeneration(String stepName) {

//		inTime = System.currentTimeMillis();
//		lapTime = inTime;

    System.out.println(String.format("generating with the seed = %d", Globals.getSeed()));

    drawing.startTransaction(stepName);
}

private void postGeneration() {
    drawing.endTransaction();

}

private Rectangle2D.Double getRandomSubboxRect2(Rectangle2D.Double parentRect) {

    return new Rectangle2D.Double(
            parentRect.getX() + parentRect.getWidth() * Globals.randomGenerator.nextDouble(),
            parentRect.getY() + parentRect.getHeight() * Globals.randomGenerator.nextDouble(),
            globals.getBoxAutosizeWidthMin() + globals.randomGenerator.nextDouble() * globals.getBoxAutosizeWidthDelta(),
            globals.getBoxAutosizeHeightMin() + globals.randomGenerator.nextDouble() * globals.getBoxAutosizeHeightDelta());

}

private Rectangle2D.Double getRandomSubboxRect(Rectangle2D.Double parentRect) {

    double w = globals.getBoxAutosizeWidthMin() +
            Globals.randomGenerator.nextDouble() * globals.getBoxAutosizeWidthDelta();
    double h = globals.getBoxAutosizeHeightMin() +
            Globals.randomGenerator.nextDouble() * globals.getBoxAutosizeHeightDelta();

    return new Rectangle2D.Double(
            parentRect.getX() + (parentRect.getWidth() - w) * Globals.randomGenerator.nextDouble(),
            parentRect.getY() + (parentRect.getHeight() - h) * Globals.randomGenerator.nextDouble(),
            w,
            h);

}

private Rectangle2D.Double getRandomSubrect(Rectangle2D.Double parentRect) {

    double w = Globals.randomGenerator.nextDouble() * parentRect.getWidth();
    double h = Globals.randomGenerator.nextDouble() * parentRect.getHeight();

    return new Rectangle2D.Double(
            parentRect.getX() + (parentRect.getWidth() - w) * Globals.randomGenerator.nextDouble(),
            parentRect.getY() + (parentRect.getHeight() - h) * Globals.randomGenerator.nextDouble(),
            w,
            h);
}

private Rectangle2D.Double getRandomSubrect(Rectangle2D.Double parentRect, double avgWidth, double avgHeight) {

    final double C = 1;

    double minValue = Globals.getCurrentBoxSpacing();

    double w = avgWidth + Globals.randomGenerator.nextGaussian() * C;
    w = Math.min(w, parentRect.getX() + parentRect.getWidth());
    w = Math.max(w, minValue);

    double h = avgHeight + Globals.randomGenerator.nextGaussian() * C;
    h = Math.min(h, parentRect.getY() + parentRect.getHeight());
    h = Math.max(h, minValue);

    return new Rectangle2D.Double(
            parentRect.getX() + (parentRect.getWidth() - w) * Globals.randomGenerator.nextDouble(),
            parentRect.getY() + (parentRect.getHeight() - h) * Globals.randomGenerator.nextDouble(),
            w,
            h);
}

private Point2D.Double getRandomPoint(Rectangle2D.Double parentRect) {
    if (parentRect == null)
        return getRandomPoint(new Rectangle2D.Double(0, 0, 1000, 500));

    return new Point2D.Double(parentRect.getX() + parentRect.getWidth() * Globals.randomGenerator.nextDouble(),
            parentRect.getY() + parentRect.getHeight() * Globals.randomGenerator.nextDouble());
}

private void addRandomBox(Rectangle2D.Double area) {

    Rectangle2D.Double boxRect = getRandomSubboxRect(area);

    log.outputToLog(String.format("random rectangle %s generated inside bounds %s",
            log.dumpRectangle(boxRect),
            log.dumpRectangle(area)
    ));

    DBox b = drawing.addBox(boxRect);


}

private void addRandomNestedBox() {

    List<DBox> allBoxes = getAllBoxes();

    int numBoxes = allBoxes.size();
    if (numBoxes > 0) {

        int boxIndex = Globals.randomGenerator.nextInt(numBoxes);
        DBox box = allBoxes.get(boxIndex);

        Rectangle2D.Double newBoxRect = getRandomSubboxRect(box.getBounds());

        log.outputToLog(String.format("random rectangle %s generated inside bounds %s",
                log.dumpRectangle(newBoxRect),
                log.dumpRectangle(box.getBounds())
        ));

        DBox newBox = drawing.addBox(newBoxRect, box);


    }

}

private void addRandomLine(DLine.DLineType lineType) {

    List<DBox> allBoxes = getAllBoxes();
    int numBoxes = allBoxes.size();

//		if (numBoxes < 2) return;
    if (numBoxes < 1) return;

    int startBoxIndex = Globals.randomGenerator.nextInt(numBoxes);

    int endBoxIndex = Globals.randomGenerator.nextInt(numBoxes);

    DBox startBox = allBoxes.get(startBoxIndex);
    long startBoxId = startBox.getUuid();

    DBox endBox = allBoxes.get(endBoxIndex);
    long endBoxId = endBox.getUuid();

    log.outputToLog(String.format("add line from box #%d (%s) to box #%d (%s)",
            startBoxId, log.dumpContainer(startBox),
            endBoxId, log.dumpContainer(endBox)));

    DLine newLine = drawing.addLine(startBox, endBox, lineType);

    log.appendToLog(String.format("  result: line #%d (%s) added",
            newLine.getUuid(),
            log.dumpLine(newLine)));


}

private void addRandomInsideLabel(DBox box) {
    Rectangle2D.Double r = box.getBounds();
    Rectangle2D.Double r2;
    if (throwCoin())
        r2 = getRandomSubrect(r, 20, 5);
    else
        r2 = getRandomSubrect(r);

    box.createInsideLabel(r2);
}

private void addRandomOutsideLabel(DBox box) {

    double w = 40 + 3 * Globals.randomGenerator.nextGaussian();
    double h = 8 + 3 * Globals.randomGenerator.nextGaussian();
    BoxSide bs = BoxSide.values()[Globals.randomGenerator.nextInt(BoxSide.values().length)];
    double pos = Globals.randomGenerator.nextDouble();

    DBoxLabel ol = box.createOutsideLabel(w, h, bs, pos);

    if (throwCoin()) {
        ol.setAutoFlip(true);
    }
}

private void addRandomLineLabel(DLine line) {
    double w = 40 + 3 * Globals.randomGenerator.nextGaussian();
    double h = 8 + 3 * Globals.randomGenerator.nextGaussian();
    Orientation o = Orientation.values()[Globals.randomGenerator.nextInt(Orientation.values().length)];
    double pos = Globals.randomGenerator.nextDouble();

    DLineLabel ll = line.createLabel(w, h, pos, o);

    if (throwCoin()) {
        ll.setAutoFlip(true);
    }
    if (throwCoin()) {
        ll.setRotation(true);
    }
}

private void resizeBoxMini(DBox box) {
    // simple resize - just width and height

    Rectangle2D.Double boxRect = box.getBounds();

    double newWidth, newHeight;
    double widthDelta, heightDelta;

    newWidth = boxRect.getWidth() + globals.resizeBoxStep * Globals.randomGenerator.nextGaussian();
    newHeight = boxRect.getHeight() + globals.resizeBoxStep * Globals.randomGenerator.nextGaussian();

    if (newWidth < globals.getBoxResizeWidthMin()) newWidth += 2 * (globals.getBoxResizeWidthMin() - newWidth);
    if (newHeight < globals.getBoxResizeHeightMin()) newHeight += 2 * (globals.getBoxResizeHeightMin() - newHeight);


    widthDelta = newWidth - boxRect.getWidth();
    heightDelta = newHeight - boxRect.getHeight();

    log.appendToLog(String.format("  by [%4.2f, %4.2f]",
            widthDelta, heightDelta));

    boxRect.setRect(boxRect.getX(), boxRect.getY(), newWidth, newHeight);
    box.resize(boxRect);

    drawing.setCurrent(box); //##

}

private void resizeBoxMaxi(DBox box) {
    // mega resize - the whole rect

    Rectangle2D.Double boxNewRect;
    if (box.getOwner() instanceof DBox) {
        boxNewRect = getRandomSubboxRect(((DBox) box.getOwner()).getBounds());
    } else {
        boxNewRect = getRandomSubboxRect(drawing.getBounds());
    }


    log.appendToLog(String.format("  to [x:%.3f, y:%.3f, width:%.3f, height:%.3f ]",
            boxNewRect.x, boxNewRect.y, boxNewRect.width, boxNewRect.height));

    box.resize(boxNewRect);

    drawing.setCurrent(box); //##

}

private void resizeRandomBox() {

    List<DBox> allBoxes = getAllBoxes();

    int numBoxes = allBoxes.size();
    if (numBoxes < 1) return;

    int boxIndex = Globals.randomGenerator.nextInt(numBoxes);
    DBox box = allBoxes.get(boxIndex);

    long boxId = box.getUuid();
    log.outputToLog(String.format("resize box #%d (%s)",
            boxId,
            log.dumpContainer(box)));

    if (Globals.randomGenerator.nextDouble() > 0.2) {
        resizeBoxMini(box);
    } else {
        resizeBoxMaxi(box);
    }


    log.appendToLog(String.format("  result: %s",
            log.dumpContainer(box)));


}

private void resizeRandomContent() {

    List<DBox> allBoxes = getAllBoxes();

    int numBoxes = allBoxes.size();
    if (numBoxes < 1) return;

    int boxIndex = Globals.randomGenerator.nextInt(numBoxes);
    DBox box = allBoxes.get(boxIndex);

    long boxId = box.getUuid();

    double newMinW = globals.defaultBoxContentWidthMin + globals.randomGenerator.nextDouble() * globals.defaultBoxContentWidthDelta;
    double newMinH = globals.defaultBoxContentHeightMin + globals.randomGenerator.nextDouble() * globals.defaultBoxContentHeightDelta;

    log.outputToLog(String.format("set minW/minH for box #%d (%s) to %.6f, %.6f",
            boxId,
            log.dumpContainer(box),
            newMinW,
            newMinH));

    box.setMinWidth(newMinW);
    box.setMinHeight(newMinH);

    log.appendToLog(String.format("  result: %s",
            log.dumpContainer(box)));


}

private void moveOuterBoxToInner() {

    List<DBox> allBoxes = getDirectBoxes();
    int numBoxes = allBoxes.size();

    if (numBoxes < 2) return;

    int sourceBoxIndex = Globals.randomGenerator.nextInt(numBoxes);
    int targetBoxIndex = (sourceBoxIndex + 1 + Globals.randomGenerator.nextInt(numBoxes - 1)) % numBoxes;

    DBox sourceBox = allBoxes.get(sourceBoxIndex);
    long sourceBoxId = sourceBox.getUuid();
    Rectangle2D.Double sourceRect = sourceBox.getBounds();

    Point2D.Double sourceAnchorPoint = getRandomPoint(sourceRect);

    DBox targetBox = allBoxes.get(targetBoxIndex);
    long targetBoxId = targetBox.getUuid();
    Rectangle2D.Double targetRect = targetBox.getBounds();

    Point2D.Double targetPoint = new Point2D.Double(targetRect.getX() + Globals.randomGenerator.nextDouble() * targetRect.getWidth(),
            targetRect.getY() + Globals.randomGenerator.nextDouble() * targetRect.getHeight());

    Point2D.Double newCenter = new Point2D.Double(targetPoint.getX() + sourceRect.getCenterX() - sourceAnchorPoint.getX(),
            targetPoint.getY() + sourceRect.getCenterY() - sourceAnchorPoint.getY());

    log.outputToLog(String.format("move box #%d (%s) into #%d (%s) [%s - %s]",
            sourceBoxId, log.dumpContainer(sourceBox),
            targetBoxId, log.dumpContainer(targetBox),
            log.dumpPoint(newCenter), log.dumpPoint(targetPoint)));

    sourceBox.move(newCenter, targetBox, targetPoint);

    drawing.setCurrent(sourceBox); //##

    log.appendToLog(String.format("  result: (%s) in (%s)",
            log.dumpContainer(sourceBox),
            log.dumpContainer(targetBox)));


}

private Rectangle2D.Double getBoundsOrEmpty(DContainer c) {
    Rectangle2D.Double r = c.getBounds();
    if (r != null)
        return r;
    return new Rectangle2D.Double();
}

private void moveOuterContainerToInner() {

    List<DContainer> allBoxes = getDirectContainers();
    int numBoxes = allBoxes.size();

    if (numBoxes < 2) return;

    int sourceBoxIndex = Globals.randomGenerator.nextInt(numBoxes);
    int targetBoxIndex = (sourceBoxIndex + 1 + Globals.randomGenerator.nextInt(numBoxes - 1)) % numBoxes;

    DContainer sourceBox = allBoxes.get(sourceBoxIndex);
    long sourceBoxId = sourceBox.getUuid();
    Rectangle2D.Double sourceRect = getBoundsOrEmpty(sourceBox);

    Point2D.Double sourceAnchorPoint = getRandomPoint(sourceRect);

    DContainer targetBox = allBoxes.get(targetBoxIndex);
    long targetBoxId = targetBox.getUuid();
    Rectangle2D.Double targetRect = getBoundsOrEmpty(targetBox);

    Point2D.Double targetPoint = new Point2D.Double(targetRect.getX() + Globals.randomGenerator.nextDouble() * targetRect.getWidth(),
            targetRect.getY() + Globals.randomGenerator.nextDouble() * targetRect.getHeight());

    Point2D.Double newCenter = new Point2D.Double(targetPoint.getX() + sourceRect.getCenterX() - sourceAnchorPoint.getX(),
            targetPoint.getY() + sourceRect.getCenterY() - sourceAnchorPoint.getY());

    log.outputToLog(String.format("move box #%d (%s) into #%d (%s) [%s - %s]",
            sourceBoxId, log.dumpContainer(sourceBox),
            targetBoxId, log.dumpContainer(targetBox),
            log.dumpPoint(newCenter), log.dumpPoint(targetPoint)));

    if (sourceBox instanceof DBox) {
        ((DBox) sourceBox).move(newCenter, targetBox, targetPoint);

    } else if (sourceBox instanceof DGroup) {
        ((DGroup) sourceBox).move(new Point2D.Double(), targetBox);

    }

    drawing.setCurrent(sourceBox); //##

    log.appendToLog(String.format("  result: (%s) in (%s)",
            log.dumpContainer(sourceBox),
            log.dumpContainer(targetBox)));


}

private void moveInnerBoxToOuter() {

    List<DBox> innerBoxes = getAllBoxes();
    innerBoxes.removeAll(getDirectBoxes());

    int numInner = innerBoxes.size();
    if (numInner < 1) return;

    int boxIndex = Globals.randomGenerator.nextInt(numInner);
    DBox b = innerBoxes.get(boxIndex);
    long boxId = b.getUuid();

    Point2D.Double newPoint;


    if (Globals.randomGenerator.nextDouble() > 0.2) {
        // a) move to outer, width/o geometry change
        newPoint = new Point2D.Double(b.getBounds().getCenterX(), b.getBounds().getCenterY());

        log.outputToLog(String.format("move inner box #%d (%s) to top",
                boxId, log.dumpContainer(b)));

    } else {
        // b) movo to outer, with new rect
        newPoint = getRandomPoint(drawing.getBounds());

        log.outputToLog(String.format("move inner box #%d (%s) to top, new center: [%s]",
                boxId, log.dumpContainer(b),
                log.dumpPoint(newPoint)));

    }

//		b.move(newPoint, drawing.layout, newPoint);
//		drawing.moveBox(b, newPoint);
    b.move(newPoint);

    drawing.setCurrent(b); //##

    log.appendToLog(String.format("  result: (%s)",
            log.dumpContainer(b)));

}

private void moveInnerContainerToOuter() {

    List<DContainer> innerBoxes = getAllContainers();
    innerBoxes.removeAll(getDirectContainers());

    int numInner = innerBoxes.size();
    if (numInner < 1) return;

    int boxIndex = Globals.randomGenerator.nextInt(numInner);
    DContainer b = innerBoxes.get(boxIndex);
    long boxId = b.getUuid();

    Point2D.Double newPoint;


    if (Globals.randomGenerator.nextDouble() > 0.2) {
        // a) move to outer, width/o geometry change
        newPoint = new Point2D.Double(getBoundsOrEmpty(b).getCenterX(), getBoundsOrEmpty(b).getCenterY());

        log.outputToLog(String.format("move inner container #%d (%s) to top",
                boxId, log.dumpContainer(b)));

    } else {
        // b) movo to outer, with new rect
        newPoint = getRandomPoint(drawing.getBounds());

        log.outputToLog(String.format("move inner container #%d (%s) to top, new center: [%s]",
                boxId, log.dumpContainer(b),
                log.dumpPoint(newPoint)));

    }

//		b.move(newPoint, drawing.layout, newPoint);
//		drawing.moveBox(b, newPoint);
    if (b instanceof DBox) {
        ((DBox) b).move(newPoint, drawing.getDiagram(), newPoint);
    } else if (b instanceof DGroup) {
        ((DGroup) b).move(new Point2D.Double(), drawing.getDiagram());
    }

    drawing.setCurrent(b); //##

    log.appendToLog(String.format("  result: (%s)",
            log.dumpContainer(b)));

}

private void moveOuterBoxToOuter() {

    List<DBox> topBoxes = getDirectBoxes();
    int numBoxes = topBoxes.size();

    if (numBoxes < 1) return;

    int boxIndex = Globals.randomGenerator.nextInt(numBoxes);
    DBox box = topBoxes.get(boxIndex);
    Point2D.Double newCenter = getRandomPoint(drawing.getBounds());

    long boxId = box.getUuid();

    log.outputToLog(String.format("move outer box #%d (%s) to point (%s)",
            boxId, log.dumpContainer(box),
            log.dumpPoint(newCenter)));

//		box.move(newCenter, drawing.layout, newCenter);
//		drawing.moveBox(box, newCenter);
    box.move(newCenter);

    drawing.setCurrent(box); //##

    log.appendToLog(String.format("  result: (%s)",
            log.dumpContainer(box)));


}

private void moveOuterContainerToOuter() {
}

private void moveInnerBoxToInner() {

    List<DBox> innerItems = new ArrayList<>(getAllBoxes());
    innerItems.removeAll(getDirectBoxes());

    List<DBox> outerItems = new ArrayList<>(getDirectBoxes());

    int numInnerBoxes = innerItems.size();
    if (numInnerBoxes < 1) return;
    int innerBoxIndex = Globals.randomGenerator.nextInt(numInnerBoxes);

    int numOuterBoxes = outerItems.size();
    int outerBoxIndex = Globals.randomGenerator.nextInt(numOuterBoxes);

    DBox boxToMove = innerItems.get(innerBoxIndex);

    long boxToMoveId = boxToMove.getUuid();

    DBox newParent = outerItems.get(outerBoxIndex);

    long newParentId = newParent.getUuid();

    Point2D.Double newCenter = getRandomPoint(newParent.getBounds());

    log.outputToLog(String.format("move inner box #%d (%s) to new owner #%d (%s) in (%s)",
            boxToMoveId, log.dumpContainer(boxToMove),
            newParentId,
            log.dumpContainer(newParent), log.dumpPoint(newCenter)));

    boxToMove.move(newCenter, newParent, newCenter);
    drawing.setCurrent(boxToMove); //##

    log.appendToLog(String.format("  result: (%s) in (%s)",
            log.dumpContainer(boxToMove),
            log.dumpContainer(newParent)));

}

private void moveInnerContainerToInner() {


    List<DContainer> innerItems = new ArrayList<>(getAllContainers());
    innerItems.removeAll(getDirectContainers());

    List<DContainer> outerItems = new ArrayList<>(getDirectContainers());

    int numInnerBoxes = innerItems.size();
    if (numInnerBoxes < 1) return;
    int innerBoxIndex = Globals.randomGenerator.nextInt(numInnerBoxes);

    int numOuterBoxes = outerItems.size();
    int outerBoxIndex = Globals.randomGenerator.nextInt(numOuterBoxes);

    DContainer boxToMove = innerItems.get(innerBoxIndex);

    long boxToMoveId = boxToMove.getUuid();

    DContainer newParent = outerItems.get(outerBoxIndex);

    long newParentId = newParent.getUuid();

    Point2D.Double newCenter = getRandomPoint(getBoundsOrEmpty(newParent));

    log.outputToLog(String.format("move inner box #%d (%s) to new owner #%d (%s) in (%s)",
            boxToMoveId, log.dumpContainer(boxToMove),
            newParentId,
            log.dumpContainer(newParent), log.dumpPoint(newCenter)));


    boxToMove.move(new Point2D.Double(), newParent);
    drawing.setCurrent(boxToMove); //##

    log.appendToLog(String.format("  result: (%s) in (%s)",
            log.dumpContainer(boxToMove),
            log.dumpContainer(newParent)));

}

private void moveRandomBox() {

    int opt = Globals.randomGenerator.nextInt(4);

    switch (opt) {
        case 0:
            // move inner to outer
            moveInnerContainerToOuter();
            break;

        case 1:
            // move outer to inner
            moveOuterContainerToInner();
            break;

        case 2:
            // move inner to inner
            moveInnerContainerToInner();
            break;

        default:
            // move outer to outer
            moveOuterBoxToOuter();
    }


}

private void deleteRandomBox() {

    ArrayList<DBox> elements = getAllBoxes();

    int numElements = elements.size();
    if (numElements > 0) {

        int elementIndex = Globals.randomGenerator.nextInt(numElements);
        DBox elementToDelete = elements.get(elementIndex);

        long elementId = elementToDelete.getUuid();
        log.outputToLog(String.format("delete box #%d", elementId));

        drawing.removeElement(elementToDelete);

    }
}

private void deleteRandomLine() {

    ArrayList<DLine> lines = getAllLines();

    int numElements = lines.size();
    if (numElements > 0) {

        int elementIndex = Globals.randomGenerator.nextInt(numElements);
        DLine elementToDelete = lines.get(elementIndex);

        long elementId = elementToDelete.getUuid();
        log.outputToLog(String.format("delete line #%d", elementId));

        drawing.removeElement(elementToDelete);

    }
}

private void reshapeRandomLine() {

    ArrayList<DLine> lines = getAllLines();

    int numLines = lines.size();
    if (numLines > 0) {

        int lineIndex = Globals.randomGenerator.nextInt(numLines);
        DLine lineToReshape = lines.get(lineIndex);
        long elementId = lineToReshape.getUuid();

        log.outputToLog(String.format("reshape line #%d", elementId));

        DBox startBox = lineToReshape.getStart();
        DBox endBox = lineToReshape.getEnd();

        Rectangle2D.Double startRect = startBox.getBounds();
        Rectangle2D.Double endRect = endBox.getBounds();

        Point2D.Double startPoint = getRandomPoint(startRect);
        Point2D.Double endPoint = getRandomPoint(endRect);
        if (endPoint.equals(startPoint))
            endPoint.x += 1.0;

        final double PROBEXTERNALLINE = 0.1;
        ArrayList<Point2D.Double> newPoints;

        if (Globals.randomGenerator.nextDouble() < PROBEXTERNALLINE) {
            newPoints = getRandomOrthoPoints();
        } else {
            newPoints = getRandomOrthoPoints(startPoint, endPoint);
        }

        log.appendToLog("points before reshape: " + log.dumpLine(lineToReshape));
        log.appendToLog("reshape to: " + log.dumpLinePoints(newPoints));

        lineToReshape.setPoints(newPoints);

        drawing.setCurrent(lineToReshape); //##

        log.appendToLog("  points after reshape: " + log.dumpLine(lineToReshape));

    }

}

private ArrayList<Point2D.Double> getRandomOrthoPoints() {
    Rectangle2D.Double r = drawing.getBounds();

    Point2D.Double startPoint = getRandomPoint(r);
    Point2D.Double endPoint = getRandomPoint(r);

    if (endPoint.equals(startPoint))
        endPoint.x += 1.0;

    return getRandomOrthoPoints(startPoint, endPoint);
}

private ArrayList<Point2D.Double> getRandomOrthoPoints(Rectangle2D.Double maxArea) {

    Point2D.Double startPoint = getRandomPoint(maxArea);
    Point2D.Double endPoint = getRandomPoint(maxArea);

    return getRandomOrthoPoints(startPoint, endPoint, maxArea);
}

private ArrayList<Point2D.Double> getRandomOrthoPoints(Point2D.Double startPoint, Point2D.Double endPoint) {
    Rectangle2D.Double r = drawing.getBounds();

    return getRandomOrthoPoints(
            startPoint,
            endPoint,
//				r
            new Rectangle2D.Double(
                    Math.min(startPoint.x, endPoint.x),
                    Math.min(startPoint.y, endPoint.y),
                    Math.abs(startPoint.x - endPoint.x),
                    Math.abs(startPoint.y - endPoint.y))
    );
}

private ArrayList<Point2D.Double> getRandomOrthoPoints(Point2D.Double startPoint, Point2D.Double endPoint, Rectangle2D.Double maxArea) {

    ArrayList<Point2D.Double> points = new ArrayList<>();

    if ((startPoint.x == endPoint.x) || (startPoint.y == endPoint.y)) {
        points.add(startPoint);
        points.add(endPoint);
        return points;
    }

    points.add(startPoint);

    Point2D.Double prevPoint, nextPoint;
    prevPoint = startPoint;

    double distanceXToGo, distanceYToGo, distanceXTotal, distanceYTotal, delta;

    distanceXTotal = endPoint.x - prevPoint.x;
    distanceYTotal = endPoint.y - prevPoint.y;

    boolean isHorizontal = Globals.randomGenerator.nextBoolean();

    int numSegmentsTotal = Math.max(2, 5 + (int) (Globals.randomGenerator.nextGaussian() * 4));
    int numXSegmentsTotal = isHorizontal ? (numSegmentsTotal + 1) / 2 : numSegmentsTotal / 2;
    int numYSegmentsTotal = isHorizontal ? numSegmentsTotal / 2 : (numSegmentsTotal + 1) / 2;

    int numXSegmentsToGo, numYSegmentsToGo, numSegmentsToGo;

    int walkStrategy = Globals.randomGenerator.nextInt(4);

    log.appendToLog(String.format("mode: %d, segments: %d", walkStrategy, numSegmentsTotal));


    // N-2 random segments
    for (int i = 0; i < numSegmentsTotal - 2; i++) {

        numSegmentsToGo = numSegmentsTotal - i;
        numXSegmentsToGo = isHorizontal ? (numSegmentsToGo + 1) / 2 : numSegmentsToGo / 2;
        numYSegmentsToGo = isHorizontal ? numSegmentsToGo / 2 : (numSegmentsToGo + 1) / 2;

        distanceXToGo = endPoint.x - prevPoint.x;
        distanceYToGo = endPoint.y - prevPoint.y;

        switch (walkStrategy) {
            case 1: // adaptive non-monotone
                if (isHorizontal) {
                    delta = distanceXToGo / numXSegmentsToGo * (1 + Globals.randomGenerator.nextGaussian());
                    nextPoint = new Point2D.Double(prevPoint.x + delta, prevPoint.y);
                } else {
                    delta = distanceYToGo / numYSegmentsToGo * (1 + Globals.randomGenerator.nextGaussian());
                    nextPoint = new Point2D.Double(prevPoint.x, prevPoint.y + delta);
                }
                break;

            case 2: // random
                if (isHorizontal) {
                    nextPoint = new Point2D.Double(maxArea.x + Globals.randomGenerator.nextDouble() * maxArea.width, prevPoint.y);
                } else {
                    nextPoint = new Point2D.Double(prevPoint.x, maxArea.y + Globals.randomGenerator.nextDouble() * maxArea.height);
                }
                break;

            case 3: //? non-adaptive non-monotone
                if (isHorizontal) {
                    delta = distanceXTotal / numXSegmentsTotal * (1 + Globals.randomGenerator.nextGaussian());
                    nextPoint = new Point2D.Double(prevPoint.x + delta, prevPoint.y);
                } else {
                    delta = distanceYTotal / numYSegmentsTotal * (1 + Globals.randomGenerator.nextGaussian());
                    nextPoint = new Point2D.Double(prevPoint.x, prevPoint.y + delta);
                }
                break;


            case 0: // adaptive linear monotone
            default:
                if (isHorizontal) {
                    delta = distanceXToGo * Globals.randomGenerator.nextDouble();
                    nextPoint = new Point2D.Double(prevPoint.x + delta, prevPoint.y);
                } else {
                    delta = distanceYToGo * Globals.randomGenerator.nextDouble();
                    nextPoint = new Point2D.Double(prevPoint.x, prevPoint.y + delta);
                }
                break;

        }

        points.add(nextPoint);

        prevPoint = nextPoint;
        isHorizontal = !isHorizontal;
    }

    // finish the line
    if (isHorizontal) {
        nextPoint = new Point2D.Double(endPoint.x, prevPoint.y);
    } else {
        nextPoint = new Point2D.Double(prevPoint.x, endPoint.y);
    }
    points.add(nextPoint);
    points.add(endPoint);

    return points;

}

private long getElementId(Element e) {
    return DElement.getThisId(e);
//		return 1;
}

// ------------------------------------------------------------------	
private DLine pickRandomLine() {
    return pickRandomLine(drawing.getDiagram());
}

private DLine pickRandomLine(DContainer fromWhere) {

    ArrayList<DLine> lines = new ArrayList<>();
    for (DLine e : fromWhere.allLines()) {
        lines.add(e);
    }

    return (lines.size() > 0) ?
            lines.get(globals.randomGenerator.nextInt(lines.size())) :
            null;

}

private DContainer pickRandomContainer() {
    return pickRandomContainer(drawing.getDiagram());
}

private DContainer pickRandomContainer(DContainer fromWhere) {

    ArrayList<DContainer> containers = new ArrayList<>();
    for (DContainer e : fromWhere.containers()) {
        containers.add(e);
    }
    return (containers.size() > 0) ?
            containers.get(globals.randomGenerator.nextInt(containers.size())) : null;

}

private DGroup pickRandomGroup() {
    return pickRandomGroup(drawing.getDiagram());
}

private DGroup pickRandomGroup(DContainer fromWhere) {

    ArrayList<DGroup> groups = new ArrayList<>();
    for (DGroup e : fromWhere.groups()) {
        groups.add(e);
    }
    return (groups.size() > 0) ?
            groups.get(globals.randomGenerator.nextInt(groups.size())) : null;

}

private DBox pickRandomTopBox() {
    return pickRandomTopBox(drawing.getDiagram());
}

private DBox pickRandomTopBox(DContainer fromWhere) {

    ArrayList<DBox> boxes = new ArrayList<>();
    for (DBox e : fromWhere.boxes()) {
        boxes.add(e);
    }
    return (boxes.size() > 0) ?
            boxes.get(globals.randomGenerator.nextInt(boxes.size())) : null;

}

private DBox pickRandomBox() {
    return pickRandomBox(drawing.getDiagram());
}

private DBox pickRandomBox(DContainer fromWhere) {

    ArrayList<DBox> boxes = new ArrayList<>();
    for (DBox e : fromWhere.allBoxes()) {
        boxes.add(e);
    }
    return (boxes.size() > 0)
            ? boxes.get(globals.randomGenerator.nextInt(boxes.size()))
            : null;
}

private DContainer pickOtherRandomContainer(DContainer avoidThis) {
    ArrayList<DContainer> all = new ArrayList<>();
    for (DContainer e : drawing.getDiagram().allContainers()) {
        all.add(e);
    }

    ArrayList<DContainer> avoidable = new ArrayList<>();
    for (DContainer e : avoidThis.allContainers()) {
        avoidable.add(e);
    }

    all.remove(avoidThis);
    all.removeAll(avoidable);

    return (all.size() > 0)
            ? all.get(globals.randomGenerator.nextInt(all.size()))
            : null;
}

//  --------------------------------------------------------------------	

private ArrangeStyle pickRandomArrangeStyle() {
    ArrangeStyle as = ArrangeStyle.values()[globals.randomGenerator.nextInt(ArrangeStyle.values().length)];
    if (as != ArrangeStyle.NONE) return as;
    else return ArrangeStyle.INHERITED;
}

private DLine.DLineType pickRandomLineType(DLine.DLineType currentType) {

    // patvaļīgs tips, atšķirīgs no esošā
    int step = globals.randomGenerator.nextInt(DLine.DLineType.values().length - 1) + 1;
    int idx = (currentType.ordinal() + step) % DLine.DLineType.values().length;
    return DLine.DLineType.values()[idx];

}

//    private <E extends Enum<E>> E pickOtherValue(E val) {
//        int step = globals.randomGenerator.nextInt(E.values().length - 1) + 1;
//        int idx = (val.ordinal() + step) % E.values().length;
//        return E.values()[idx];
//
//    }

//  --------------------------------------------------------------------	
private void changeConstraintType(DContainer c) {

    DContainer lc = (c == null) ? drawing.getDiagram() : c;

    if (lc.getLayoutConstraints().getType() == ConstraintType.GRID) {
        lc.setConstraintType(ConstraintType.NONE);
    } else {
        lc.setConstraintType(ConstraintType.GRID);
        gridifyContainer(lc);
    }
}

//  --------------------------------------------------------------------	
private void toggleContainerType(DContainer c1) {

    if (c1 == null) return;
    if (c1 instanceof DDiagram) return; // cannot change

    if (c1 instanceof DBox)
        c1.convertToGroup();
    else
        c1.convertToBox();


}

//  --------------------------------------------------------------------	

@Override
public void gridifyContainer(DContainer c) {

    DContainer gridContainer = (c == null) ? drawing.getDiagram() : c;


    // reset all constraints
    gridContainer.setConstraintType(ConstraintType.GRID);

    GridLayoutConstraints glc = (GridLayoutConstraints) gridContainer.getLayoutConstraints();

    double avgElementsPerCell = 4.0; // subelements per cell
    double probabilityCellAttachment = 1.0;

    double probabilityHorizontal = 0.1;
    double probabilityVertical = 0.1 + probabilityHorizontal;

    int numChildren1 = getDirectContainers(gridContainer).size();
    int numChildren2 = getDirectInnerLabels(gridContainer).size();
    int numChildren = numChildren1 + numChildren2;

    int numCells = (int) Math.round(numChildren / avgElementsPerCell + 0.5);
    int numRows, numCols;

    double coin = Globals.randomGenerator.nextDouble();

    if (coin < probabilityHorizontal) {
        numRows = 1;
        numCols = numCells;
    } else if (coin < probabilityVertical) {
        numRows = numCells;
        numCols = 1;
    } else {
        numRows = numCols = (int) Math.round(Math.sqrt(numCells));
    }

    drawing.startTransaction("set sub-element rows/columns");
    for (DContainer c2 : gridContainer.containers()) {

        if (Globals.randomGenerator.nextDouble() < probabilityCellAttachment) {
            glc.setRow(c2.getLayoutElement(), Globals.randomGenerator.nextInt(numRows) + 1);
        } else {
            glc.setRow(c2.getLayoutElement(), null);
        }

        if (Globals.randomGenerator.nextDouble() < probabilityCellAttachment) {
            glc.setColumn(c2.getLayoutElement(), Globals.randomGenerator.nextInt(numCols) + 1);
        } else {
            glc.setColumn(c2.getLayoutElement(), null);
        }

    }
    for (DInsideLabel il : gridContainer.insideLabels()) {

        if (Globals.randomGenerator.nextDouble() < probabilityCellAttachment) {
            glc.setRow(il.getLayoutElement(), Globals.randomGenerator.nextInt(numRows) + 1);
        } else {
            glc.setRow(il.getLayoutElement(), null);
        }

        if (Globals.randomGenerator.nextDouble() < probabilityCellAttachment) {
            glc.setColumn(il.getLayoutElement(), Globals.randomGenerator.nextInt(numCols) + 1);
        } else {
            glc.setColumn(il.getLayoutElement(), null);
        }

    }
    drawing.endTransaction();

}

private boolean throwCoin() {
    return globals.randomGenerator.nextBoolean();
}

private void resizeGrid(DBox b) {
    if (b == null) return;
    if (b.getLayoutConstraints().getType() != ConstraintType.GRID) return;

    GridLayoutConstraints glc = (GridLayoutConstraints) b.getLayoutConstraints();
    int nRows = glc.getRowCount();
    int nColumns = glc.getColumnCount();

    if (nRows > 1) {


        int borderToMove = globals.randomGenerator.nextInt(nRows - 1) + 1;
        double offset = globals.randomGenerator.nextGaussian() * b.getBounds().getHeight() / 3;

        System.out.println(String.format("shift rowborder %d by %f", borderToMove, offset));

        if (throwCoin()) {
            borderToMove++;
            glc.setRowTop(borderToMove, glc.getRowTop(borderToMove) + offset);
        } else {
            glc.setRowBottom(borderToMove, glc.getRowBottom(borderToMove) + offset);
        }

    }

    if (nColumns > 1) {


        int borderToMove = globals.randomGenerator.nextInt(nColumns - 1) + 1;
        double offset = globals.randomGenerator.nextGaussian() * b.getBounds().getWidth() / 3;

        System.out.println(String.format("shift columnborder %d by %f", borderToMove, offset));

        if (throwCoin()) {
            borderToMove++;
            glc.setColumnLeft(borderToMove, glc.getColumnLeft(borderToMove) + offset);
        } else {
            glc.setColumnRight(borderToMove, glc.getColumnRight(borderToMove) + offset);
        }


    }
}

@Override
public void doOneEditingStep() {

    OperationChooser operation = OperationChooser.throwCoin();
    System.out.println(String.format("Izvēlēta operācija: %s", operation.toString()));

    DContainer c;
    DBox b;
    InsideLabel il;
    OutsideLabel ol;
    LineLabel ll;

    switch (operation) {
        case DELETE_BOX:
            this.deleteRandomBox();
            break;

        case DELETE_LINE:
            this.deleteRandomLine();
            break;

        case MOVE_BOX:
            this.moveRandomBox();
            break;

        case NEW_ORTHO_LINE:
            this.addRandomLine(DLine.DLineType.ORTHO);
            break;

        case NEW_STRAIGHT_LINE:
            this.addRandomLine(DLine.DLineType.STRAIGHT);
            break;

        case NEW_POLY_LINE:
            // TODO
            break;

        case NEW_NESTED_BOX:
            this.addRandomNestedBox();
            break;

        case NEW_TOP_BOX:
            this.addRandomBox(Globals.defaultGeneratorArea);
            break;

        case RESHAPE_LINE:
            this.reshapeRandomLine();
            break;

        case RESIZE_BOX:
            this.resizeRandomBox();
            break;

        case RESIZE_CONTENT:
            this.resizeRandomContent();
            break;

        case CHANGE_ARRANGE_STYLE:
            c = pickRandomContainer();
            if (c == null) break;
            c.setArrangeStyle(pickRandomArrangeStyle());
            break;

        case CHANGE_LINE_TYPE:
            DLine l = pickRandomLine();
            if (l != null) {
                l.setLineType(pickRandomLineType(l.getLineType()));
            }
            break;

        case CHANGE_CONSTRAINT_TYPE:
            c = pickRandomContainer();
            if (c == null) break;
            changeConstraintType(c);
            break;

        case CHANGE_CONTAINER_TYPE:
            c = pickRandomContainer();
            if (c == null) break;
            toggleContainerType(c);
            break;

        case RESIZE_GRID:
            b = pickRandomTopBox();
            if (b == null) break;
            resizeGrid(b);
            break;

        case NEW_INSIDE_LABEL:
            b = pickRandomTopBox();
            if (b == null) break;
            addRandomInsideLabel(b);
            break;

        case NEW_OUTSIDE_LABEL:
            b = pickRandomTopBox();
            if (b == null) break;
            addRandomOutsideLabel(b);
            break;

        case NEW_LINE_LABEL:
            l = pickRandomLine();
            if (l == null) break;
            addRandomLineLabel(l);
            break;

        case MOVE_INSIDE_LABEL:
            break;

        case MOVE_OUTSIDE_LABEL:
            break;

        case MOVE_LINE_LABEL:
            break;

        default:
            System.out.println("Unsupported operation: " + operation.toString());

    }

}

@Override
public void showHideOptions() {
    OperationChooser.showHideDialog();
}

@Override
public void convertChildBoxesToLabels(DContainer container) {

    DContainer gridContainer = (container == null) ? drawing.getDiagram() : container;

    GridLayoutConstraints glc;
    Rectangle2D.Double tempRect;
    Integer tempRow;
    Integer tempColumn;

//		ArrayList<DBox> innerBoxes = new ArrayList<>(gridContainer.getBoxes());
    ArrayList<DBox> innerBoxes = getDirectBoxes(gridContainer);

    drawing.startTransaction("convert inner boxes to labels");

    if (gridContainer.getLayoutConstraints().getType() == ConstraintType.GRID) {
        glc = (GridLayoutConstraints) gridContainer.getLayoutConstraints();

        for (DBox b : innerBoxes) {

            tempRect = b.getBounds();

            tempRow = glc.getRow(b.getLayoutElement());
            tempColumn = glc.getColumn(b.getLayoutElement());

            drawing.removeElement(b);

            gridContainer.createInsideLabel(
                    tempRect,
                    ConstraintType.NONE,
                    drawing.getCurrentBoxSpacing(),
                    tempRow,
                    tempColumn
            );

        }

    } else {

        for (DBox b : innerBoxes) {

            tempRect = b.getBounds();

            drawing.removeElement(b);

            gridContainer.createInsideLabel(
                    tempRect,
                    ConstraintType.NONE,
                    drawing.getCurrentBoxSpacing(),
                    null,
                    null
            );

        }

    }

    drawing.endTransaction();
}

ArrayList<DBox> getAllBoxes(DContainer fromWhere) {
    ArrayList<DBox> all = new ArrayList<>();
    for (DBox e : fromWhere.allBoxes()) {
        all.add(e);
    }
    return all;
}

ArrayList<DBox> getAllBoxes() {
    return getAllBoxes(drawing.getDiagram());
}

ArrayList<DContainer> getAllContainers(DContainer fromWhere) {
    ArrayList<DContainer> all = new ArrayList<>();
    for (DContainer e : fromWhere.allContainers()) {
        all.add(e);
    }
    return all;
}

ArrayList<DContainer> getAllContainers() {
    return getAllContainers(drawing.getDiagram());
}

ArrayList<DBox> getDirectBoxes(DContainer fromWhere) {
    ArrayList<DBox> all = new ArrayList<>();
    for (DBox e : fromWhere.boxes()) {
        all.add(e);
    }
    return all;
}

ArrayList<DBox> getDirectBoxes() {
    return getDirectBoxes(drawing.getDiagram());
}

ArrayList<DGroup> getDirectGroups(DContainer fromWhere) {
    ArrayList<DGroup> all = new ArrayList<>();
    for (DGroup e : fromWhere.groups()) {
        all.add(e);
    }
    return all;
}

public ArrayList<DGroup> getDirectGroups() {
    return getDirectGroups(drawing.getDiagram());
}

ArrayList<DContainer> getDirectContainers(DContainer fromWhere) {
    ArrayList<DContainer> all = new ArrayList<>();
    for (DContainer e : fromWhere.containers()) {
        all.add(e);
    }
    return all;
}

ArrayList<DContainer> getDirectContainers() {
    return getDirectContainers(drawing.getDiagram());
}

ArrayList<DInsideLabel> getDirectInnerLabels(DContainer fromWhere) {
    ArrayList<DInsideLabel> all = new ArrayList<>();
    for (DInsideLabel e : fromWhere.insideLabels()) {
        all.add(e);
    }
    return all;
}

public ArrayList<DInsideLabel> getDirectInnerLabels() {
    return getDirectInnerLabels(drawing.getDiagram());
}

ArrayList<DLine> getAllLines() {
    ArrayList<DLine> all = new ArrayList<>();
    for (DLine e : drawing.getDiagram().allLines()) {
        all.add(e);
    }
    return all;
}

//###
private enum OperationChooser {
    NEW_TOP_BOX(10),
    NEW_NESTED_BOX(20),
    MOVE_BOX(5),
    RESIZE_BOX(5),
    RESIZE_CONTENT(5),
    DELETE_BOX(5),
    NEW_ORTHO_LINE(15),
    NEW_STRAIGHT_LINE(10),
    NEW_POLY_LINE(5),
    DELETE_LINE(5),
    RESHAPE_LINE(20),
    CHANGE_LINE_TYPE(4),
    CHANGE_ARRANGE_STYLE(5),
    CHANGE_CONSTRAINT_TYPE(4),
    RESIZE_GRID(10),
    CHANGE_CONTAINER_TYPE(20),
    NEW_INSIDE_LABEL(5),
    NEW_OUTSIDE_LABEL(5),
    NEW_LINE_LABEL(5),
    MOVE_INSIDE_LABEL(0),
    MOVE_OUTSIDE_LABEL(0),
    MOVE_LINE_LABEL(0);
    private static int totalWeight = 0;
    private static boolean isTotalOK = false;
    private static ConfigurationDialog configurationDialog = null;
    private int operationWeight = 0;

    private OperationChooser(int weight) {
        this.operationWeight = weight > 0 ? weight : 0;
    }

    private static void updateTotal() {
        totalWeight = 0;
        for (OperationChooser o : values()) {
            totalWeight += o.operationWeight;
        }
        isTotalOK = true;
    }

    public static OperationChooser throwCoin() {
        if (!isTotalOK) updateTotal();

        int coin = Globals.randomGenerator.nextInt(totalWeight);
        int currentSum = 0;
        for (OperationChooser o : values()) {
            currentSum += o.operationWeight;
            if (coin < currentSum) return o;
        }
        assert currentSum == totalWeight;
        return values()[0];
    }

    public static void showHideDialog() {
        if (configurationDialog == null) {
            configurationDialog = new ConfigurationDialog(Frame.getFrames()[0]); // TODO uztaisit drosaku veidu
            configurationDialog.setVisible(true);
        } else if (configurationDialog.isVisible()) {
            configurationDialog.setVisible(false);
        } else {
            configurationDialog.setVisible(true);
        }
    }

    public int getWeight() {
        return operationWeight;
    }

    public void setWeight(int newWeight) {
        operationWeight = newWeight > 0 ? newWeight : 0;
        updateTotal();
    }

    @SuppressWarnings("serial")
    private static class ConfigurationDialog
            extends JDialog
            implements ActionListener {

        private Map<OperationChooser, JLabel> valueMap = new HashMap<>();
        private Map<OperationChooser, JLabel> percentMap = new HashMap<>();

        public ConfigurationDialog(Frame owner) {
            super(owner);
            initComponents();
        }

        public void initComponents() {
            setDefaultCloseOperation(JDialog.HIDE_ON_CLOSE);
            setAutoRequestFocus(false);
            setType(Type.UTILITY);

            build();
        }

        public void build() {
            JPanel dialogPanel = new JPanel();
            dialogPanel.setLayout(new BoxLayout(dialogPanel, BoxLayout.Y_AXIS));
            for (OperationChooser o : values()) {
                // create subpanel
                JPanel panel = new JPanel();

                // add o.name
                panel.setBorder(BorderFactory.createTitledBorder(o.name()));

                // add 0
                JButton zeroButton = new JButton("0");
                zeroButton.setName(o.name());
                zeroButton.setActionCommand("0");
                zeroButton.addActionListener(this);
                panel.add(zeroButton);

                // add --
                JButton minusminusButton = new JButton("--");
                minusminusButton.setName(o.name());
                minusminusButton.setActionCommand("--");
                minusminusButton.addActionListener(this);
                panel.add(minusminusButton);

                // add -
                JButton minusButton = new JButton("-");
                minusButton.setName(o.name());
                minusButton.setActionCommand("-");
                minusButton.addActionListener(this);
                panel.add(minusButton);

                // add o.value
//					JLabel valueLabel = new JLabel(Integer.toString(o.getWeight()));
                JLabel valueLabel = new JLabel("?");
                valueMap.put(o, valueLabel);
                panel.add(valueLabel);

                // add +
                JButton plusButton = new JButton("+");
                plusButton.setName(o.name());
                plusButton.setActionCommand("+");
                plusButton.addActionListener(this);
                panel.add(plusButton);

                // add ++
                JButton plusplusButton = new JButton("++");
                plusplusButton.setName(o.name());
                plusplusButton.setActionCommand("++");
                plusplusButton.addActionListener(this);
                panel.add(plusplusButton);

                // add 100
                JButton hundredButton = new JButton("100");
                hundredButton.setName(o.name());
                hundredButton.setActionCommand("100");
                hundredButton.addActionListener(this);
                panel.add(hundredButton);

                // add %
                updateTotal();
//					JLabel percentLabel = new JLabel(String.format("( %d%% )", (o.getWeight() * 100) / totalWeight));
                JLabel percentLabel = new JLabel("%");
                percentMap.put(o, percentLabel);
                panel.add(percentLabel);

//					panel.setPreferredSize(new Dimension(400, 70));
                // add subpanel
                dialogPanel.add(panel);
            }
            updateLabels();
            add(new JScrollPane(dialogPanel));
            setSize(400, Math.min(70 * values().length, Toolkit.getDefaultToolkit().getScreenSize().height));
            setLocation(Toolkit.getDefaultToolkit().getScreenSize().width - getWidth(), 0);
            setTitle("Configure Operation Probabilities");
        }

        public void updateLabels() {
            for (OperationChooser o : values()) {
                JLabel valueLabel = valueMap.get(o);
                valueLabel.setText(Integer.toString(o.getWeight()));

                JLabel percentLabel = percentMap.get(o);
                percentLabel.setText(String.format("(%d%%)", (100 * o.getWeight()) / totalWeight));
            }
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            String actionCommand = e.getActionCommand();

            String actionName = ((JButton) e.getSource()).getName();
            OperationChooser op = valueOf(actionName);
            int oldWeight = op.getWeight();

            if ("-".equals(actionCommand)) {
            }

            switch (e.getActionCommand()) {
                case "-":
//					op.setWeight(oldWeight > 0 ? oldWeight - 1 : 0);
                    op.setWeight(oldWeight - 1);
                    break;

                case "+":
                    op.setWeight(oldWeight + 1);
                    break;

                case "0":
                    if (op.getWeight() < totalWeight) {
                        op.setWeight(0);
                    }
                    break;

                case "--":
                    op.setWeight(oldWeight - 10);
                    break;

                case "++":
                    op.setWeight(oldWeight + 10);
                    break;

                case "100":
                    for (OperationChooser o : values()) {
                        o.setWeight(0);
                    }
                    op.setWeight(100);
                    break;

            }

            updateTotal();
            updateLabels();

        }
    }
}


}
