/*
*
* Copyright (c) 2013,2014 Institute of Mathematics and Computer Science, University of Latvia (IMCS UL). 
*
* This file is part of DEEK (diagram editor engine kit). 
*
* You can redistribute it and/or modify it under the terms of the GNU General Public License 
* as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
*
* You should have received a copy of the GNU General Public License along with DEEK If not, see http://www.gnu.org/licenses/.
*
*/

package lv.lumii.deek.ui;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
//import java.io.InputStream;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Scanner;

import javax.swing.*;
import javax.swing.filechooser.FileNameExtensionFilter;

//import org.slf4j.Logger;
//import org.slf4j.LoggerFactory;

import lv.lumii.deek.model.*;
import lv.lumii.deek.test.ContentManipulator;
import lv.lumii.deek.util.BlockSize;
import lv.lumii.deek.util.DLogger;
import lv.lumii.layoutengine.LayoutConstraints.ConstraintType;
import lv.lumii.layoutengine.ArrangeData.ArrangeStyle;
//import lv.lumii.layoutengine.Box.BoxSide;
import lv.lumii.layoutengine.LayoutConstraints.GridLayoutConstraints;
//import lv.lumii.layoutengine.OutsideLabel.LineLabel;
//import lv.lumii.layoutengine.OutsideLabel.LineLabel.Orientation;
import lv.lumii.test.OpusHost;
import lv.lumii.deek.util.Globals;
//import lv.lumii.deek.model.DrawingInterface.TrackingMode;
import lv.lumii.deek.test.ContentGenerator;


public class DController {

private static final String LOCAL_FOLDER = "samples/";
private static final String FILENAME_PREFIX = "_drawing-";
// names in style "samples/_drawing-john-2011-03-17-18-52-27.xml
private static final String SAVE_PREFIX = LOCAL_FOLDER
        + FILENAME_PREFIX
        + System.getProperty("user.name").toLowerCase()
        + "-";
//	private List<View> views;
private static final String PICTURE_FORMAT_PNG = "png";
private static final String PICTURE_FORMAT_JPEG = "jpg";
private static final String PICTURE_FORMAT_GIF = "gif";

// ------------------------------------------------------
private static final String PICTURE_FORMAT_BMP = "bmp";
private static OpusHost _host;
private DDrawing _drawing;
private DViewport _mainView;
private Globals _globals = Globals.getInstance();

// ------------------------------------------------------
private boolean trackMouse = true;
private DBox _currentBox;
private DLabel _currentLabel;
private Rectangle2D.Double _currentBoxWorldRect;

// ------------------------------------------------------
// ------------------------------------------------------
// ------------------------------------------------------
// ------------------------------------------------------
private DLine _currentLine;
private DElement popupElement = null;
private long lapTime;
private long startTime = 0;
private ViewportMode viewportMode = null;
private boolean isViewportModeSticky = true;
private DActions actions;
private DLogger log = new DLogger();

public DController(OpusHost host, DDrawing model, DViewport view) {
    this._drawing = model;
    this._mainView = view;
    _host = host;

    initComponents();

    _mainView.setController(this);

}

private void initComponents() {

    actions = new DActions();
    setMode(ViewportMode.INSERT_BOX, false);
}

public void mouseMovedWorld(Point2D.Double worldPoint, MouseEvent e) {

    boolean isShift = e.isShiftDown();

    // TODO - jaasataisa tuvaak sagaidaamajam feedback
    if (isShift) {
        _drawing.setTrackingMode(DDrawing.TrackingMode.ALL);
    } else {
        _drawing.setTrackingMode(DDrawing.TrackingMode.NONE);
    }

    if (_drawing.hoverTest(worldPoint)) {
        _mainView.updateView();
    }

}

public void mouseLeftPressedWorld(Point2D.Double worldPoint, MouseEvent e) {

    boolean isShift = e.isShiftDown();
//		boolean isControl = e.isControlDown();

    DElement currentElement;

    switch (viewportMode) {

        case INSERT_BOX:
        case INSERT_INSIDE_LABEL:
            if (isShift) {
                _drawing.setTrackingMode(DDrawing.TrackingMode.BOXES);
            } else {
                _drawing.setTrackingMode(DDrawing.TrackingMode.NONE);
            }
            _mainView.startNewBox(worldPoint);
            break;

        case EDIT_MOVE_RESIZE:
            if (isShift) {
                // shift : move/resize with re-parenting
                _drawing.setTrackingMode(DDrawing.TrackingMode.BOXES);
            } else {
                // no shift : move/resize flat
                _drawing.setTrackingMode(DDrawing.TrackingMode.NONE);
            }

            currentElement = _drawing.getDiagram().getHit(worldPoint);
            _drawing.setCurrent(currentElement);

            if ((currentElement == null) || (currentElement == _drawing.getDiagram())) {

                if (isShift) {
                    // TODO
                    _mainView.startNewBox(worldPoint);

                } else {
                    _mainView.startPaneTool(worldPoint);
                }

                break;
            }

            if (false) {
                _currentBoxWorldRect = _currentBox.getBounds();
                _mainView.startMoveResizeBox(_currentBoxWorldRect, worldPoint);
            } else {

                _currentBox = null;
                _currentLabel = null;
                _currentLine = null;

                if (currentElement instanceof DLine) {

                    _currentLine = (DLine) currentElement;
                    if (_currentLine.getBaseLineType() == DLine.DLineType.ORTHO) {
                        _mainView.startReshapeOrthoLine(_currentLine.getPoints(), e);
                    } else if (_currentLine.getBaseLineType() == DLine.DLineType.STRAIGHT) {
                        _mainView.startReshapePolyLine(_currentLine.getPoints(), e);
                    } else if (_currentLine.getBaseLineType() == DLine.DLineType.POLY) {
                        _mainView.startReshapePolyLine(_currentLine.getPoints(), e);
                    }

                } else if (currentElement instanceof DInsideLabel) {
                    _currentLabel = (DInsideLabel) currentElement;

                    _mainView.startMoveResizeBox(_currentLabel.getBounds(), worldPoint);
                } else if (currentElement instanceof DLineLabel) {
                    _currentLabel = (DLineLabel) currentElement;

                    _mainView.startMoveResizeBox(_currentLabel.getBounds(), worldPoint);

                } else if (currentElement instanceof DBoxLabel) {
                    _currentLabel = (DBoxLabel) currentElement;

                    _mainView.startMoveResizeBox(_currentLabel.getBounds(), worldPoint);

                } else if (currentElement instanceof DBox) {
                    _currentBox = (DBox) currentElement;
                    _currentBoxWorldRect = _currentBox.getBounds();
                    _mainView.startMoveResizeBox(_currentBox.getBounds(), worldPoint);

                }

            }
            break;

        case INSERT_SIMPLE_LINE:
//		case INSERT_SIMPLE_LINE_AUTO:
            // TODO
            _drawing.setTrackingMode(DDrawing.TrackingMode.BOXES);
            _mainView.startNewLine(e);
            break;

        case INSERT_ORTHO_LINE:
//		case INSERT_ORTHO_LINE_AUTO:
            // TODO
            _drawing.setTrackingMode(DDrawing.TrackingMode.BOXES);
            _mainView.startNewOrthoLine(e);
            break;

        case INSERT_POLY_LINE:
//		case INSERT_POLY_LINE_AUTO:
            // TODO
            _drawing.setTrackingMode(DDrawing.TrackingMode.BOXES);
            _mainView.startNewPolyLine(e);
            break;

        case INSERT_OUTSIDE_LABEL:
        case INSERT_LINE_LABEL:
            _drawing.setTrackingMode(DDrawing.TrackingMode.NONE);
            _mainView.startNewBox(worldPoint);

            break;

        default:

    }

}

public void mouseRightPressedWorld(Point2D.Double worldPoint, MouseEvent e) {


    JPopupMenu popupMenu;

    DBox b = _drawing.getDiagram().getHitBox(worldPoint);
    if (b != null) {
        popupMenu = b.getElementMenu();
        popupMenu.show(e.getComponent(), e.getX(), e.getY());
        updateAll();
        return;
    } else {
        DElement el = _drawing.getDiagram().getHit(worldPoint);
        if (el != null) {
            popupElement = el;

            popupMenu = el.getElementMenu();
            popupMenu.show(e.getComponent(), e.getX(), e.getY()); //*

        } else {
            popupElement = null;

            popupMenu = _drawing.getDiagram().getElementMenu();
            popupMenu.show(e.getComponent(), e.getX(), e.getY()); //*
        }
    }
    updateAll();
}

// ------------------------------------------------------
public void lineAdded(ArrayList<Point2D.Double> points) {

    startWait();

    log.outputToLog("line drawn by user");

    switch (viewportMode) {
        case INSERT_ORTHO_LINE:
            _drawing.addLine(points, DLine.DLineType.ORTHO);
            break;

//		case INSERT_ORTHO_LINE_AUTO:
//			_drawing.addLine(points, LineType.ORTHOGONAL, true);
//			break;

        case INSERT_SIMPLE_LINE:
            _drawing.addLine(points, DLine.DLineType.STRAIGHT);
            break;

//		case INSERT_SIMPLE_LINE_AUTO:
//			_drawing.addLine(points, LineType.STRAIGHT, true);
//			break;

        case INSERT_POLY_LINE:
            _drawing.addLine(points, DLine.DLineType.POLY);
            break;

//		case INSERT_POLY_LINE_AUTO:
//			_drawing.addLine(points, LineType.POLYLINE, true);
//			break;

        default:
            System.out.println("unexpected mode:" + viewportMode.toString());
    }

    endWait();


    resetMode();
}

// ------------------------------------------------------
public void boxAdded(Rectangle2D.Double r, Point2D.Double p, boolean isShift, boolean isCtrl) {


    boolean addAsNested = isShift;

    DElement currentElement;
    DElement newElement = null;

    boolean insertAtCurrent = false;

    Point2D.Double newCenter = new Point2D.Double(r.getCenterX(), r.getCenterY());

    switch (viewportMode) {
        case INSERT_BOX:
            startWait();
            log.outputToLog(String.format("box drawn by user: %s, nested=%b",
                    log.dumpRectangle(r),
                    addAsNested));
            newElement = _drawing.addBox(r, addAsNested, p); // insert at the last corner
            endWait();
            break;

        case INSERT_INSIDE_LABEL:
            startWait();
            log.outputToLog(String.format("inside label drawn by user: %s, nested=%b",
                    log.dumpRectangle(r),
                    addAsNested));
            newElement = _drawing.addInsideLabel(r, addAsNested, p); // insert at the last corner
            endWait();
            break;

        case INSERT_OUTSIDE_LABEL:
            startWait();
            log.outputToLog(String.format(
                    "outside label drawn by user: %s",
                    log.dumpRectangle(r)));

            currentElement = _drawing.getCurrent();

            if (!insertAtCurrent || (currentElement == null) || !(currentElement instanceof DBox)) {

                DBox nearestBox = _drawing.findNearestBox(newCenter);
                if (nearestBox != null)
                    newElement = _drawing.addOutsideLabel(r, newCenter, nearestBox);

            } else {
                newElement = _drawing.addOutsideLabel(r, newCenter, (DBox) currentElement);
            }

            endWait();
            break;

        case INSERT_LINE_LABEL:
            startWait();
            log.outputToLog(String.format(
                    "line label drawn by user: %s",
                    log.dumpRectangle(r)));

            currentElement = _drawing.getCurrent();

            if (!insertAtCurrent || (currentElement == null) || !(currentElement instanceof DLine)) {

                DLine nearestLine = _drawing.findNearestLine(newCenter);
                if (nearestLine != null)
                    newElement = _drawing.addLineLabel(r, newCenter, nearestLine);

            } else {
                newElement = _drawing.addLineLabel(r, newCenter, (DLine) currentElement);
            }

            endWait();
            break;

        case EDIT_MOVE_RESIZE:
            System.out.println("selection");
            _drawing.selectBoxes(r, isShift, isCtrl);
            break;

        default:
            // nothing
    }

    if (newElement != null) {

        if (newElement instanceof DLabel)
            _drawing.setCurrent(newElement.getOwner());
        else
            _drawing.setCurrent(newElement);

    }


    resetMode();
}

private Integer getGridRowFromPoint(GridLayoutConstraints glc, Point2D.Double point) {

    for (int i = 1; i <= glc.getRowCount(); i++) {
        if ((glc.getRowTop(i) <= point.y) && (glc.getRowBottom(i) >= point.y))
            return i;
    }
    return null;
}

private Integer getGridColumnFromPoint(GridLayoutConstraints glc, Point2D.Double point) {

    for (int i = 1; i <= glc.getColumnCount(); i++) {
        if ((glc.getColumnLeft(i) <= point.x) && (glc.getColumnRight(i) >= point.x))
            return i;
    }

    return null;
}

// ------------------------------------------------------
public void boxMoved(Rectangle2D.Double r, Point2D.Double mousePoint, boolean isShift, boolean isCtrl) {

    if (_currentBox != null) {

        boolean changeOwner = isShift;

//			log.outputToLog("box moved by user");

        Point2D.Double newCenter = new Point2D.Double(r.getCenterX(), r.getCenterY());
        Point2D.Double insertPoint = mousePoint;
        Point2D.Double growPoint = insertPoint;

        if (_currentBoxWorldRect.contains(mousePoint)) {
            // stop, ja pele atrodas saakumtaisnstuurii - jo nevar kasti ielikt sevii;
            changeOwner = false;
        }

        startWait();
        if (changeOwner) {

            // move box from the old position (world:_currentBox.getBounds() to viewport:r

            DContainer currentOwner = (DContainer) _currentBox.getOwner();

            DContainer newOwner = _drawing.getDiagram().getHitBox(insertPoint);
            if (newOwner == null)
                newOwner = _drawing.getDiagram();

//				Box newOwner = _drawing.getHitBoxOrNull(insertPoint);

            if (newOwner.getLayoutConstraints().getType() == ConstraintType.GRID) {

                GridLayoutConstraints glc = (GridLayoutConstraints) newOwner.getLayoutConstraints();

                _currentBox.move(
                        newCenter,
                        newOwner,
                        growPoint,
                        getGridRowFromPoint(glc, mousePoint),
                        getGridColumnFromPoint(glc, mousePoint)
                );

            } else {

                _currentBox.move(
                        newCenter,
                        newOwner,
                        growPoint
                );

            }

        } else {

            _currentBox.move(
                    newCenter,
                    _currentBox.getOwner(),
                    growPoint
            );

        }

        _drawing.setCurrent(_currentBox);
        endWait();

    } else if (_currentLabel != null) {


        Point2D.Double newCenter = new Point2D.Double(r.getCenterX(), r.getCenterY());
        Point2D.Double insertPoint = mousePoint;
        Point2D.Double growPoint = insertPoint;

        startWait();

        if (_currentLabel instanceof DInsideLabel) {
            DInsideLabel label = (DInsideLabel) _currentLabel;

            DContainer newOwner = label.getOwner();
            if (isShift) {
                newOwner = _drawing.getDiagram().getHitBox(mousePoint);
                if (newOwner == null)
                    newOwner = _drawing.getDiagram();

                if (newOwner.getLayoutConstraints().getType() == ConstraintType.GRID) {

                    GridLayoutConstraints glc = (GridLayoutConstraints) newOwner.getLayoutConstraints();

                    label.move(
                            newCenter,
                            newOwner,
                            growPoint,
                            getGridRowFromPoint(glc, mousePoint),
                            getGridColumnFromPoint(glc, mousePoint)
                    );

                } else {

                    label.move(
                            newCenter,
                            newOwner,
                            growPoint
                    );

                }

            } else {

                label.move(
                        newCenter,
                        newOwner,
                        growPoint
                );

            }


        } else if (_currentLabel instanceof DLineLabel) {
            DLineLabel label = (DLineLabel) _currentLabel;
            DLine newOwner;

            if (!isShift) {
                label.move(mousePoint, label.getOwner());
            } else {
                newOwner = _drawing.findNearestLine(newCenter);
                if (newOwner != null) {
                    label.move(newCenter, newOwner);
                } else {
                    label.move(newCenter, label.getOwner());
                }
            }

        } else if (_currentLabel instanceof DBoxLabel) {
            DBoxLabel label = (DBoxLabel) _currentLabel;
            DBox newOwner;

            if (!isShift) {
                label.move(mousePoint, label.getOwner());
            } else {
                newOwner = _drawing.findNearestBox(newCenter);
                if (newOwner != null) {
                    label.move(newCenter, newOwner);
                } else {
                    label.move(newCenter, label.getOwner());
                }
            }
        }

        _drawing.setCurrent(_currentLabel.getOwner());
        endWait();


    }
}

public void boxResized(Rectangle2D.Double r) {
    if (_currentBox != null) {


        log.outputToLog("box resized by user");
        startWait();

        BlockSize dim = _currentBox.measure();

        r.width = Math.max(r.width, dim.width);
        r.height = Math.max(r.height, dim.height);

        _currentBox.resize(r);
        _drawing.setCurrent(_currentBox);
        endWait();

    } else if (_currentLabel != null) {

        log.outputToLog("label resized by user to " + r.toString());

        startWait();

        _currentLabel.resize(r);
        _drawing.setCurrent(_currentBox);


        endWait();

    }
}

public void lineReshaped(ArrayList<Point2D.Double> points) {
    if (_currentLine != null) {
        startWait();
        _currentLine.setPoints(points, _drawing.hasLineAutoCleanup());
        _drawing.setCurrent(_currentLine);
        endWait();
    }
}

// ------------------------------------------------------
void beep() {
    Toolkit.getDefaultToolkit().beep();
}

public void close() {
    if (_host instanceof JFrame) {
        ((JFrame) _host).dispatchEvent(new WindowEvent((JFrame) _host, WindowEvent.WINDOW_CLOSING));
    }
}

public void updateInfo() {
    Rectangle2D bounds = _drawing.getBounds();

    if (bounds == null)
        bounds = new Rectangle2D.Double();

    Runtime runtime = Runtime.getRuntime();
    final int MEGABYTE = 1024 * 1024;

    String t = String.format("Mode: %s(%s); %s, Bounds: (%1.1f, %1.1f, %1.1f, %1.1f) " +
            "Boxes: %d [%d], " +
            "Lines: %d [%d,%s] " +
            "Last optime: %,dms; Debug: %b, %d, Devmode: %b, Mem: %dMB",
            viewportMode.toString(),
            isViewportModeSticky ? "S" : "",
            _mainView.zoomer.getViewportInfo(),
            bounds.getMinX(), bounds.getMinY(), bounds.getMaxX(), bounds.getMaxY(),

            _drawing.getBoxCount(),
            _globals.boxGenerationCount,
            _drawing.getLineCount(),
            _globals.lineGenerationCount,
            _drawing.getCurrentLineType().toString(),

            lapTime,
            Globals.debugOutputIsOn,
            Globals.randomSeed,
            Globals.isDeveloperMode(),
            (runtime.totalMemory() - runtime.freeMemory()) / MEGABYTE);

    _host.setStatusText(t);
}

public void zoomToContents() {
    _mainView.zoomer.zoomToViewport();
    updateAll();
}

public void updateAll() {
    updateInfo();
    _mainView.updateView();
}

void startWait() {
    startTime = System.currentTimeMillis();
    _mainView.startWait();
}

void endWait() {
    _mainView.endWait();
    lapTime = System.currentTimeMillis() - startTime;
    updateInfo();
}

private ViewportMode setMode(ViewportMode newMode) {
    return setMode(newMode, false);
//		return setMode(newMode, newMode.isSticky());
}

private ViewportMode setMode(ViewportMode newMode, boolean overrideSticky) { // sticky --> overrideSticky ???

    if (newMode != viewportMode) {
        viewportMode = newMode;

        switch (viewportMode) {
            case EDIT_MOVE_RESIZE:
                actions.editModeAction.putValue(AbstractAction.SELECTED_KEY, true);
                break;

            case INSERT_BOX:
                actions.insertBoxAction.putValue(AbstractAction.SELECTED_KEY, true);
                break;

            case INSERT_ORTHO_LINE:
                actions.insertOrthoLineAction.putValue(AbstractAction.SELECTED_KEY, true);
                break;

            case INSERT_POLY_LINE:
                actions.insertPolyLineAction.putValue(AbstractAction.SELECTED_KEY, true);
                break;

            case INSERT_SIMPLE_LINE:
                actions.insertSimpleLineAction.putValue(AbstractAction.SELECTED_KEY, true);
                break;
        }

        if (Globals.isDeveloperMode()) {
            isViewportModeSticky = !overrideSticky;
        } else {
            isViewportModeSticky = overrideSticky ? !newMode.isSticky() : newMode.isSticky();
        }

        updateAll();
    }
    return viewportMode;
}

ViewportMode resetMode() {
    if (isViewportModeSticky) return viewportMode;

    return setMode(ViewportMode.defaultValue());
}

// ------------------------------------------------------
public void cycleModes(boolean backwards) {

    if (backwards) {
        setMode(viewportMode.prevValue());
    } else {
        setMode(viewportMode.nextValue());
    }

}

private void showHelpDialog() {

    StringBuilder s2 = new StringBuilder();

    try (
            Scanner in = new Scanner(getClass().getResourceAsStream("/texts/help_lv.txt"))
    ) {
        while (in.hasNext()) {
            s2.append(in.nextLine());
            s2.append("\n");
        }
    }

    String helpText = s2.toString();

    JOptionPane.showMessageDialog(
            null,
            helpText,
            "Help",
            JOptionPane.INFORMATION_MESSAGE);
}

private void showHelpDialogHTML() {
    StringBuilder s2 = new StringBuilder();
    try (
            Scanner in = new Scanner(getClass().getResourceAsStream("/texts/help_lv.html"))
    ) {
        while (in.hasNext()) {
            s2.append(in.nextLine());
            s2.append("\n");
        }
    }

    String helpText = s2.toString();
    final JEditorPane helpArea = new JEditorPane("text/html; charset=UTF-8", helpText);
    helpArea.setEditable(false);
    helpArea.setOpaque(false);
    helpArea.setPreferredSize(new Dimension(900, 720));

    helpArea.addHierarchyListener(new HierarchyListener() {
        @Override
        public void hierarchyChanged(HierarchyEvent e) {
            Window window = SwingUtilities.getWindowAncestor(helpArea);
            if (window instanceof Dialog) {
                Dialog dialog = (Dialog) window;
                if (!dialog.isResizable()) {
                    dialog.setResizable(true);
                }
            }
        }
    });

    JOptionPane.showMessageDialog(
            null,
            new JScrollPane(helpArea),
            "Help",
            JOptionPane.INFORMATION_MESSAGE);
}

private String getTimestampedFilename(String prefix, String extension) {

    final String DATE_TIME_FORMAT = "yyyy-MM-dd-HH-mm-ss";
    SimpleDateFormat sdf = new SimpleDateFormat(DATE_TIME_FORMAT);

    Calendar c = Calendar.getInstance();

    return ((prefix != null) ? prefix : "") + sdf.format(c.getTime()) + ((extension != null) ? extension : "");
}

private void saveDiagramToFile(String comment) {

    String IMAGE_FILENAME = getTimestampedFilename(SAVE_PREFIX, ".xml");
    _drawing.saveToFile(IMAGE_FILENAME, comment);


}

private void loadDiagramFromFile() {

    final JFileChooser fc = new JFileChooser();
    fc.setCurrentDirectory(new File(LOCAL_FOLDER));

    FileNameExtensionFilter filter = new FileNameExtensionFilter(
            "Diagram XML files", "xml", "dlf", "ddf");
    fc.setFileFilter(filter);

    fc.setDialogTitle("Open _drawing");

    int returnVal = fc.showOpenDialog(getUiParentComponent());
    if (returnVal == JFileChooser.APPROVE_OPTION) {
        File file = fc.getSelectedFile();
        _drawing.loadFromXMLFile(file);
        updateAll();
    }

//		_drawing.loadFromFile(LOCAL_FOLDER + "test.xml");
}

private void saveDiagramToDump() {

    String s = _drawing.dumpDrawing();
    String DUMP_FILENAME = getTimestampedFilename(SAVE_PREFIX, ".dump.txt");

    try {
        BufferedWriter out = new BufferedWriter(new FileWriter(DUMP_FILENAME));
        out.write(s);
        out.close();
    } catch (IOException e) {
        e.printStackTrace();
    }

}

private void saveDiagramToBitmapImage() {
    _drawing.saveToImage(getTimestampedFilename(SAVE_PREFIX, null), PICTURE_FORMAT_PNG);
}

private void arrangeDiagram() {
    startWait();
    _drawing.arrangeDiagram();
    endWait();
    setMode(ViewportMode.EDIT_MOVE_RESIZE);
    _mainView.zoomer.zoomToViewport();
    updateAll();
}

private void setArrangeStyle(ArrangeStyle arrangeStyle) {
    startWait();
    _drawing.setArrangeStyle(arrangeStyle);
    setMode(ViewportMode.EDIT_MOVE_RESIZE);
    _mainView.zoomer.zoomToViewport();
    updateAll();
    endWait();
}

private void setLineType(DLine.DLineType lineType) {
    startWait();

    _drawing.convertAllLinesToType(lineType);

    setMode(ViewportMode.EDIT_MOVE_RESIZE);
    _mainView.zoomer.zoomToViewport();
    updateAll();

    endWait();
}

private void gridifyContainerChildren(DContainer container) {
    _drawing.getContentManipulator().gridifyContainer(container);
}

public void setupInputAndActionMaps(InputMap im, ActionMap am) {
    am.put(DActions.CLOSE_WINDOW_ACTION, actions.closeWindowAction);
    am.put(DActions.NEW_DIAGRAM_ACTION, actions.newDiagramAction);
//		am.put(DActions.DELETE_LINES_ACTION, actions.deleteLinesAction);
    am.put(DActions.GENERATE_RANDOM_BOXES, actions.generateRandomBoxesAction);
    am.put(DActions.GENERATE_RANDOM_LINES, actions.generateRandomLinesAction);
    am.put(DActions.GENERATE_NESTED_RANDOM_BOXES, actions.generateNestedRandomBoxesAction);
    am.put(DActions.GENERATE_SHORT_RANDOM_LINES, actions.generateShortRandomLinesAction);
    am.put(DActions.CYCLE_MODES_ACTION, actions.cycleModesAction);
    am.put(DActions.SET_BOX_SPACING_ACTION, actions.setBoxSpacingAction);
    am.put(DActions.SET_LINE_SPACING_ACTION, actions.setLineSpacingAction);
    am.put(DActions.ZOOM_TO_CONTENTS_ACTION, actions.zoomToContentsAction);
    am.put(DActions.RESET_ZOOM_AND_SCROLL_ACTION, actions.resetZoomAndScrollAction);
    am.put(DActions.SHOW_HELP_ACTION, actions.showHelpAction);
    am.put(DActions.SET_RANDOM_SEED_ACTION, actions.setRandomSeedAction);
    am.put(DActions.SAVE_TO_PNG_ACTION, actions.saveToBitmapImageAction);
    am.put(DActions.TOGGLE_MENUBAR_ACTION, actions.toggleMenuBarAction);
    am.put(DActions.TOGGLE_TOOLBAR_ACTION, actions.toggleToolBarAction);
    am.put(DActions.TOGGLE_STATUSBAR_ACTION, actions.toggleStatusBarAction);
    am.put(DActions.ZOOM_IN_ACTION, actions.zoomInAction);
    am.put(DActions.ZOOM_OUT_ACTION, actions.zoomOutAction);
    am.put(DActions.BENCHMARK_ACTION, actions.benchmarkAction);
    am.put(DActions.ANIMATED_SIMULATION_ACTION, actions.animatedSimulationAction);
    am.put(DActions.TOGGLE_DEBUG_ACTION, actions.toggleDebugAction);
    am.put(DActions.STOP_ANIMATION_ACTION, actions.stopAnimationAction);
    am.put(DActions.TOGGLE_OVERVIEW_ACTION, actions.toggleOverviewAction);
    am.put(DActions.TOGGLE_MAGNIFIER_ACTION, actions.toggleMagnifierAction);
    am.put(DActions.TOGGLE_COMMAND_PROPERTIES_ACTION, actions.showHideCommandPropertiesAction);
    am.put(DActions.FAST_GENERATION_ACTION, actions.fastGenerationAction);
    am.put(DActions.SAVE_TO_DUMP_ACTION, actions.saveToDumpAction);
    am.put(DActions.GENERATE_TREE_ACTION, actions.generateTreeAction);
    am.put(DActions.GENERATE_RANDOM_SINGLE_LINES_ACTION, actions.generateRandomSingleLinesAction);
    am.put(DActions.SHOW_OPTIONS_DIALOG, actions.showOptionsDialogAction);
    am.put(DActions.TOGGLE_LINE_TYPE_ACTION, actions.toggleLineTypeAction);
    am.put(DActions.GENERATE_GRAPH_ACTION, actions.generateGraphAction);
    am.put(DActions.EDIT_DELETE_ACTION, actions.editDeleteAction);

    im.put(KeyStroke.getKeyStroke("control shift A"), DActions.ANIMATED_SIMULATION_ACTION);
    im.put(KeyStroke.getKeyStroke("control shift B"), DActions.BENCHMARK_ACTION);
    im.put(KeyStroke.getKeyStroke("control shift D"), DActions.TOGGLE_DEBUG_ACTION);
    im.put(KeyStroke.getKeyStroke("control shift E"), DActions.GENERATE_RANDOM_SINGLE_LINES_ACTION);
    im.put(KeyStroke.getKeyStroke("control shift F"), DActions.FAST_GENERATION_ACTION);
    im.put(KeyStroke.getKeyStroke("control G"), DActions.GENERATE_RANDOM_BOXES);
    im.put(KeyStroke.getKeyStroke("control shift G"), DActions.GENERATE_RANDOM_LINES);
    im.put(KeyStroke.getKeyStroke("control H"), DActions.GENERATE_NESTED_RANDOM_BOXES);
    im.put(KeyStroke.getKeyStroke("control shift H"), DActions.GENERATE_SHORT_RANDOM_LINES);
    im.put(KeyStroke.getKeyStroke("control shift I"), DActions.SAVE_TO_DUMP_ACTION);
    im.put(KeyStroke.getKeyStroke("control K"), DActions.GENERATE_TREE_ACTION);
//		im.put(KeyStroke.getKeyStroke(""), DActions.TOGGLE_LINE_TYPE_ACTION);
    im.put(KeyStroke.getKeyStroke("control shift K"), DActions.GENERATE_GRAPH_ACTION);
    im.put(KeyStroke.getKeyStroke("control I"), DActions.SAVE_TO_PNG_ACTION);
    im.put(KeyStroke.getKeyStroke("control M"), DActions.CYCLE_MODES_ACTION);
    im.put(KeyStroke.getKeyStroke("control shift M"), DActions.CYCLE_MODES_ACTION);
    im.put(KeyStroke.getKeyStroke("control shift O"), DActions.TOGGLE_COMMAND_PROPERTIES_ACTION);
    im.put(KeyStroke.getKeyStroke("control shift P"), DActions.SHOW_OPTIONS_DIALOG);
    im.put(KeyStroke.getKeyStroke("control R"), DActions.SET_RANDOM_SEED_ACTION);
    im.put(KeyStroke.getKeyStroke("control shift R"), DActions.RANDOMIZE_ACTION);
    im.put(KeyStroke.getKeyStroke("control S"), DActions.SET_BOX_SPACING_ACTION);
    im.put(KeyStroke.getKeyStroke("control shift S"), DActions.SET_BOX_SPACING_ACTION);
    im.put(KeyStroke.getKeyStroke("control T"), DActions.SET_LINE_SPACING_ACTION);
    im.put(KeyStroke.getKeyStroke("control shift T"), DActions.SET_LINE_SPACING_ACTION);
    im.put(KeyStroke.getKeyStroke("control W"), DActions.CLOSE_WINDOW_ACTION);
    im.put(KeyStroke.getKeyStroke("control shift X"), DActions.STOP_ANIMATION_ACTION);
    im.put(KeyStroke.getKeyStroke("control 0"), DActions.ZOOM_TO_CONTENTS_ACTION);
    im.put(KeyStroke.getKeyStroke("control 1"), DActions.RESET_ZOOM_AND_SCROLL_ACTION);
    im.put(KeyStroke.getKeyStroke("F1"), DActions.SHOW_HELP_ACTION);
    im.put(KeyStroke.getKeyStroke("F4"), DActions.TOGGLE_MENUBAR_ACTION);
    im.put(KeyStroke.getKeyStroke("F5"), DActions.TOGGLE_TOOLBAR_ACTION);
    im.put(KeyStroke.getKeyStroke("F6"), DActions.TOGGLE_STATUSBAR_ACTION);
    im.put(KeyStroke.getKeyStroke("F7"), DActions.TOGGLE_OVERVIEW_ACTION);
    im.put(KeyStroke.getKeyStroke("F8"), DActions.TOGGLE_MAGNIFIER_ACTION);
    im.put(KeyStroke.getKeyStroke("control ADD"), DActions.ZOOM_IN_ACTION);
    im.put(KeyStroke.getKeyStroke("control DELETE"), DActions.NEW_DIAGRAM_ACTION);
    im.put(KeyStroke.getKeyStroke("DELETE"), DActions.EDIT_DELETE_ACTION);
//		im.put(KeyStroke.getKeyStroke("control shift DELETE"), DActions.DELETE_LINES_ACTION);
    im.put(KeyStroke.getKeyStroke("control SUBTRACT"), DActions.ZOOM_OUT_ACTION);
}

/**
 * Create the main menu for the application
 *
 * @return the new menu
 */
public JMenuBar createMainMenu() {
    JMenuBar mb = new JMenuBar();

    JMenu fileMenu = new JMenu("File");
    fileMenu.add(actions.closeWindowAction);

    mb.add(fileMenu);

    JMenu editMenu = new JMenu("Edit");
    editMenu.add(actions.insertBoxAction);
    editMenu.add(actions.insertOrthoLineAction);
    editMenu.add(actions.insertSimpleLineAction);
    editMenu.add(actions.editModeAction);
    editMenu.addSeparator();
    editMenu.add(actions.newDiagramAction);
//		editMenu.add(actions.deleteLinesAction);

    mb.add(editMenu);

    JMenu testMenu = new JMenu("Test");
    testMenu.add(actions.generateRandomBoxesAction);
    testMenu.add(actions.generateRandomLinesAction);
    testMenu.add(actions.generateNestedRandomBoxesAction);
    testMenu.add(actions.generateShortRandomLinesAction);

    mb.add(testMenu);

    JMenu viewMenu = new JMenu("View");
    viewMenu.add(actions.zoomToContentsAction);
    viewMenu.add(actions.resetZoomAndScrollAction);

    mb.add(viewMenu);

    JMenu customMenu = createUserMenu();
    if (customMenu != null) {
        customMenu.setName("Custom");
        mb.add(customMenu);
    }

    JMenu helpMenu = new JMenu("Help");
    helpMenu.add(actions.showHelpAction);

    mb.add(helpMenu);

    return mb;
}

/**
 * User menu items go here
 *
 * @return
 */
public JMenu createUserMenu() {
    return null;
}

/**
 * Adds the buttons to the toolbar.
 *
 * @param tb toolbar to which the buttons should be added
 */
public void addToolbarButtons(JToolBar tb) {
    addToolbarUserButtons(tb);
    tb.addSeparator();
    addToolbarSystemButtons(tb);
}

/**
 * User buttons go here
 *
 * @param tb toolbar to which the buttons should be added
 */
public void addToolbarUserButtons(JToolBar tb) {
}

/**
 * Adds the system buttons to the toolbar
 *
 * @param tb toolbar to which the buttons should be added
 */
private void addToolbarSystemButtons(JToolBar tb) {

    boolean addDevButtons = false;

    JToggleButton b;
    ButtonGroup bg = new ButtonGroup();

    b = new JToggleButton(actions.editModeAction);
    b.setHideActionText(true);
    bg.add(b);
    tb.add(b);

    tb.addSeparator();

    b = new JToggleButton(actions.insertBoxAction);
    b.setHideActionText(true);
    bg.add(b);
    tb.add(b);

    b = new JToggleButton(actions.insertInsideLabelAction);
    b.setHideActionText(true);
    bg.add(b);
    tb.add(b);

    b = new JToggleButton(actions.insertOutsideLabelAction);
    b.setHideActionText(true);
    bg.add(b);
    tb.add(b);

    b = new JToggleButton(actions.insertLineLabelAction);
    b.setHideActionText(true);
    bg.add(b);
    tb.add(b);

    b = new JToggleButton(actions.insertSimpleLineAction);
    b.setHideActionText(true);
    bg.add(b);
    tb.add(b);

    b = new JToggleButton(actions.insertOrthoLineAction);
    b.setHideActionText(true);
    bg.add(b);
    tb.add(b);

    b = new JToggleButton(actions.insertPolyLineAction);
    b.setHideActionText(true);
    bg.add(b);
    tb.add(b);


//		bg.add(new JToggleButton(actions.setSelectionModeAction));
//		bg.add(new JToggleButton(actions.insertBoxAction));
//		bg.add(new JToggleButton(actions.insertSimpleLineAction));
//		bg.add(new JToggleButton(actions.insertOrthoLineAction));
//		bg.add(new JToggleButton(actions.insertPolyLineAction));
//
//		tb.add(bg);

//		tb.add(actions.setSelectionModeAction);
//		tb.addSeparator();
//		tb.add(actions.insertBoxAction);
//		tb.add(actions.insertSimpleLineAction);
//		tb.add(actions.insertOrthoLineAction);
//		tb.add(actions.insertPolyLineAction);

    if (addDevButtons) {
        tb.addSeparator();

        tb.add(actions.generateRandomBoxesAction);
        tb.add(actions.generateNestedRandomBoxesAction);
        tb.add(actions.generateRandomLinesAction);
        tb.add(actions.generateRandomSingleLinesAction);
        tb.add(actions.generateShortRandomLinesAction);
        tb.add(actions.generateTreeAction);
        tb.add(actions.generateGraphAction);
        tb.add(actions.generateRandomInsideLabelsAction);
        tb.add(actions.generateRandomOutsideLabelsAction);
        tb.add(actions.generateRandomLineLabelsAction);
    }

		/**/
    tb.addSeparator();
    tb.add(actions.arrangeDiagramAction);
    tb.add(actions.toggleLineTypeAction);
/**/
    if (addDevButtons) {
        tb.addSeparator();
        tb.add(actions.animatedSimulationAction);
        tb.add(actions.stopAnimationAction);
        tb.add(actions.fastGenerationAction);
        tb.add(actions.showHideCommandPropertiesAction);
        tb.add(actions.benchmarkAction);
        tb.add(actions.throwDiceAction);
    }
//		tb.add(actions.toggleOverviewAction);
//		tb.add(actions.toggleMagnifierAction);

    tb.addSeparator();

    tb.add(actions.newDiagramAction);
    tb.add(actions.loadFromFileAction);
    tb.add(actions.saveToFileAction);
//		tb.add(actions.saveToPngAction);
    tb.add(actions.saveToBitmapImageAction);

    tb.addSeparator();
    tb.add(actions.zoomInAction);
    tb.add(actions.zoomOutAction);
    tb.add(actions.resetZoomAndScrollAction);
    tb.add(actions.zoomToContentsAction);
    tb.addSeparator();
    tb.add(actions.showHelpAction);

    tb.addSeparator();

    JToggleButton toggle;

    if (addDevButtons) {
        toggle = new JToggleButton(actions.toggleDebugAction);
        toggle.setHideActionText(true);
        tb.add(toggle);
    }

    toggle = new JToggleButton(actions.toggleDevModeAction);
    toggle.setHideActionText(true);
    tb.add(toggle);

    // placeholder for a user-defined action
    tb.add(actions.userAction);


}

public void setDiagram(DDiagram newDiagram) {
    this._drawing.setDiagram(newDiagram);
}

private Component getUiParentComponent() {
    if (_host instanceof JFrame)
        return (JFrame) _host;
    if (_host instanceof JApplet)
        return (JApplet) _host;
    return null;
}

// ------------------------------------------------------
private enum ViewportMode {
    //		INSERT_BOX(true),
    INSERT_BOX(false),
    INSERT_INSIDE_LABEL(false),
    INSERT_OUTSIDE_LABEL(false),
    INSERT_LINE_LABEL(false),
    EDIT_MOVE_RESIZE(true),
    //		INSERT_SIMPLE_LINE_AUTO(true),
//		INSERT_SIMPLE_LINE_AUTO(false),
//		INSERT_SIMPLE_LINE(true),
    INSERT_SIMPLE_LINE(false),
    //		INSERT_ORTHO_LINE_AUTO(true),
//		INSERT_ORTHO_LINE(true),
    INSERT_ORTHO_LINE(false),
    //		INSERT_SPLINE_LINE_AUTO(true),
//		INSERT_SPLINE_LINE(true),
//		INSERT_POLY_LINE_AUTO(true),
//		INSERT_POLY_LINE(true),
    INSERT_POLY_LINE(false);
    private boolean m_isSticky = false;

    private ViewportMode(boolean sticky) {
        this.m_isSticky = sticky;
    }

    public static ViewportMode defaultValue() {
        return EDIT_MOVE_RESIZE;
    }

    public boolean isSticky() {
        return this.m_isSticky;
    }

    public ViewportMode nextValue() {
        ViewportMode[] v = values();
        int i = this.ordinal();
        int n = i < v.length - 1 ? i + 1 : 0;
        return v[n];
    }

    public ViewportMode prevValue() {
        ViewportMode[] v = values();
        int i = this.ordinal();
        int n = i > 0 ? i - 1 : v.length - 1;
        return v[n];
    }
}

public class DActions {
    public static final int CLOSE_WINDOW_ACTION = 1;
    public static final int SAVE_TO_FILE_ACTION = 11;
    public static final int LOAD_FROM_FILE_ACTION = 12;
    public static final int NEW_DIAGRAM_ACTION = 13;
    public static final int SAVE_TO_DUMP_ACTION = 14;
    public static final int SAVE_TO_PNG_ACTION = 15;
    public static final int TOGGLE_MENUBAR_ACTION = 21;
    public static final int TOGGLE_TOOLBAR_ACTION = 22;
    public static final int TOGGLE_STATUSBAR_ACTION = 23;
    public static final int TOGGLE_OVERVIEW_ACTION = 24;
    public static final int TOGGLE_MAGNIFIER_ACTION = 25;
    public static final int ZOOM_IN_ACTION = 31;
    public static final int ZOOM_OUT_ACTION = 32;
    public static final int ZOOM_TO_CONTENTS_ACTION = 33;
    public static final int RESET_ZOOM_AND_SCROLL_ACTION = 34;
    public static final int SHOW_HELP_ACTION = 41;
    public static final int SHOW_OPTIONS_DIALOG = 42;
    public static final int SET_BOX_SPACING_ACTION = 43;
    public static final int SET_LINE_SPACING_ACTION = 44;
    public static final int EDIT_MODE_ACTION = 51;
    public static final int INSERT_BOX_ACTION = 52;
    public static final int INSERT_ORTHOLINE_ACTION = 53;
    public static final int INSERT_SIMPLELINE_ACTION = 54;
    public static final int INSERT_POLYLINE_ACTION = 55;
    public static final int INSERT_INSIDE_LABEL_ACTION = 56;
    public static final int INSERT_OUTSIDE_LABEL_ACTION = 57;
    public static final int INSERT_LINE_LABEL_ACTION = 58;
    public static final int CYCLE_MODES_ACTION = 59;
    public static final int DELETE_LINES_ACTION = 62;
    public static final int PERFORM_LAYOUT_ACTION = 63;
    public static final int UNDO_ACTION = 71;
    public static final int REDO_ACTION = 72;
    public static final int EDIT_CUT_ACTION = 73;
    public static final int EDIT_COPY_ACTION = 74;
    public static final int EDIT_PASTE_ACTION = 75;
    public static final int EDIT_DELETE_ACTION = 76;
    public static final int GENERATE_RANDOM_BOXES = 101;
    public static final int GENERATE_RANDOM_LINES = 102;
    public static final int GENERATE_NESTED_RANDOM_BOXES = 103;
    public static final int GENERATE_SHORT_RANDOM_LINES = 104;
    public static final int GENERATE_TREE_ACTION = 105;
    public static final int GENERATE_GRAPH_ACTION = 106;
    public static final int GENERATE_RANDOM_SINGLE_LINES_ACTION = 107;
    public static final int SET_RANDOM_SEED_ACTION = 108;
    public static final int RANDOMIZE_ACTION = 109;
    public static final int TOGGLE_LINE_TYPE_ACTION = 111;
    public static final int TOGGLE_DEBUG_ACTION = 112;
    public static final int TOGGLE_DEVMODE_ACTION = 113;
    public static final int BENCHMARK_ACTION = 121;
    public static final int ANIMATED_SIMULATION_ACTION = 122;
    public static final int STOP_ANIMATION_ACTION = 123;
    public static final int FAST_GENERATION_ACTION = 124;
    public static final int TOGGLE_COMMAND_PROPERTIES_ACTION = 125;
    @SuppressWarnings("serial")
    public Action toggleLineTypeAction = new AbstractAction("Toggle line type ortho/straight") {
        {
//				putValue(SHORT_DESCRIPTION, "Toggle type of existing and generated lines between straight and ortho");
            putValue(SHORT_DESCRIPTION, "Cycle through all line types");
            putValue(SMALL_ICON, new ImageIcon(getClass().getResource("/images/layout-togglelines.png")));
        }

        @Override
        public void actionPerformed(ActionEvent e) {

            // cycle through
            DLine.DLineType currentLineType = DLine.DLineType.values()[
                    (_drawing.getCurrentLineType().ordinal() + 1) % DLine.DLineType.values().length];

            setLineType(currentLineType);
        }
    };
    @SuppressWarnings("serial")
    public Action showHideCommandPropertiesAction = new AbstractAction("Change generation properties") {
        {
            putValue(SHORT_DESCRIPTION, "Show/hide operation properties configuration dialog");
            putValue(SMALL_ICON, new ImageIcon(getClass().getResource("/images/settings.png")));
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            _drawing.getContentGenerator().showHideOptions();
        }

    };
    @SuppressWarnings("serial")
    public Action toggleDebugAction = new AbstractAction("Toggle debug output") {
        {
            putValue(SHORT_DESCRIPTION, "Toggle debug output");
            putValue(SMALL_ICON, new ImageIcon(getClass().getResource("/images/test-togglelog.png")));
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            if ((e.getSource() instanceof JCheckBox) ||
                    (e.getSource() instanceof JToggleButton)) {
                AbstractButton btn = (AbstractButton) (e.getSource());
                Globals.debugOutputIsOn = btn.isSelected();
            } else {
                Globals.debugOutputIsOn = !Globals.debugOutputIsOn;
            }

            System.out.println("debug: " + Globals.debugOutputIsOn);
            updateInfo();
        }

    };
    @SuppressWarnings("serial")
    public Action toggleDevModeAction = new AbstractAction("Toggle devmode") {
        {
            putValue(SHORT_DESCRIPTION, "Toggle developer mode");
            putValue(SMALL_ICON, new ImageIcon(getClass().getResource("/images/test-toggledevmode.png")));
            putValue(SELECTED_KEY, true);
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            if ((e.getSource() instanceof JCheckBox) ||
                    (e.getSource() instanceof JToggleButton)) {
                AbstractButton btn = (AbstractButton) (e.getSource());
                Globals.setDeveloperMode(btn.isSelected());
            } else {
                Globals.setDeveloperMode(!Globals.isDeveloperMode());
            }

            System.out.println("devmode: " + Globals.isDeveloperMode());
            updateAll();
//				updateInfo();
        }

    };
    @SuppressWarnings("serial")
    public Action newDiagramAction = new AbstractAction("New _drawing") {
        {
            putValue(SHORT_DESCRIPTION, "Clear the _drawing");
            putValue(SMALL_ICON, new ImageIcon(getClass().getResource("/images/document-new.png")));
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            startWait();
            _drawing.clear();
            _mainView.zoomer.resetViewport();
            Globals.resetRandom();
            endWait();
            updateAll();
        }
    };
    @SuppressWarnings("serial")
    public Action closeWindowAction = new AbstractAction("Close") {
        {
            putValue(SHORT_DESCRIPTION, "Close program");
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            close();
        }
    };
    @SuppressWarnings("serial")
    public Action generateRandomBoxesAction = new AbstractAction("Generate Boxes") {
        {
            putValue(SMALL_ICON, new ImageIcon(getClass().getResource("/images/test-generateboxes.png")));
        }

        @Override
        public void actionPerformed(ActionEvent e) {

            if (isFullAction(e)) {
                String s = JOptionPane.showInputDialog(
                        getUiParentComponent(),
                        "Number of boxes to be generated:",
                        _globals.boxGenerationCount);
                if (s == null) {
                    return;
                }

                int elementCount;
                try {
                    elementCount = Integer.parseInt(s);
                } catch (NumberFormatException ex) {
                    elementCount = Globals.DEFAULT_BOX_GENERATION_COUNT;
                }
                _globals.boxGenerationCount = elementCount;
                updateHints();
            }

            startWait();
            _drawing.getContentGenerator().generateRandomBoxes(
                    _globals.boxGenerationCount,
                    Globals.defaultGeneratorArea);
            endWait();
            setMode(ViewportMode.EDIT_MOVE_RESIZE);
            updateAll();
        }
    };
    @SuppressWarnings("serial")
    public Action generateRandomInsideLabelsAction = new AbstractAction("Generate Inside Labels") {
        {
            putValue(SMALL_ICON, new ImageIcon(getClass().getResource("/images/test-generateinsidelabels.png")));
        }

        @Override
        public void actionPerformed(ActionEvent e) {

            if (isFullAction(e)) {
                String s = JOptionPane.showInputDialog(
                        getUiParentComponent(),
                        "Number of inside labels to be generated:",
                        _globals.labelGenerationCount);
                if (s == null) {
                    return;
                }

                int elementCount;
                try {
                    elementCount = Integer.parseInt(s);
                } catch (NumberFormatException ex) {
                    elementCount = Globals.DEFAULT_LABEL_GENERATION_COUNT;
                }
                _globals.labelGenerationCount = elementCount;
                updateHints();
            }

            startWait();
            _drawing.getContentGenerator().generateRandomInsideLabels(
                    _globals.labelGenerationCount);
            endWait();
            setMode(ViewportMode.EDIT_MOVE_RESIZE);
            updateAll();
        }
    };
    @SuppressWarnings("serial")
    public Action generateRandomOutsideLabelsAction = new AbstractAction("Generate Outside Labels") {
        {
            putValue(SMALL_ICON, new ImageIcon(getClass().getResource("/images/test-generateoutsidelabels.png")));
        }

        @Override
        public void actionPerformed(ActionEvent e) {

            if (isFullAction(e)) {
                String s = JOptionPane.showInputDialog(
                        getUiParentComponent(),
                        "Number of outside labels to be generated:",
                        _globals.labelGenerationCount);
                if (s == null) {
                    return;
                }

                int elementCount;
                try {
                    elementCount = Integer.parseInt(s);
                } catch (NumberFormatException ex) {
                    elementCount = Globals.DEFAULT_LABEL_GENERATION_COUNT;
                }
                _globals.labelGenerationCount = elementCount;
                updateHints();
            }

            startWait();
            _drawing.getContentGenerator().generateRandomOutsideLabels(
                    _globals.labelGenerationCount);
            endWait();
            setMode(ViewportMode.EDIT_MOVE_RESIZE);
            updateAll();
        }
    };
    @SuppressWarnings("serial")
    public Action generateRandomLineLabelsAction = new AbstractAction("Generate Line Labels") {
        {
            putValue(SMALL_ICON, new ImageIcon(getClass().getResource("/images/test-generatelinelabels.png")));
        }

        @Override
        public void actionPerformed(ActionEvent e) {

            if (isFullAction(e)) {
                String s = JOptionPane.showInputDialog(
                        getUiParentComponent(),
                        "Number of line labels to be generated:",
                        _globals.labelGenerationCount);
                if (s == null) {
                    return;
                }

                int elementCount;
                try {
                    elementCount = Integer.parseInt(s);
                } catch (NumberFormatException ex) {
                    elementCount = Globals.DEFAULT_LABEL_GENERATION_COUNT;
                }
                _globals.labelGenerationCount = elementCount;
                updateHints();
            }

            startWait();
            _drawing.getContentGenerator().generateRandomLineLabels(
                    _globals.labelGenerationCount);
            endWait();
            setMode(ViewportMode.EDIT_MOVE_RESIZE);
            updateAll();
        }
    };
    @SuppressWarnings("serial")
    public Action generateTreeAction = new AbstractAction("Generate Tree") {
        {
//				putValue(SHORT_DESCRIPTION, String.format("Generate tree with %d boxes; Shift to change N",
//				_globals.boxGenerationCount));
//		putValue(SMALL_ICON, new ImageIcon("resources/images/generateboxes.png"));
            putValue(SMALL_ICON, new ImageIcon(getClass().getResource("/images/test-generatetree.png")));
        }

        @Override
        public void actionPerformed(ActionEvent e) {

            if (isFullAction(e)) {

                String s = JOptionPane.showInputDialog(
                        getUiParentComponent(),
                        "Number of tree nodes to be generated:",
                        _globals.boxGenerationCount);
                if (s == null) {
                    return;
                }

                int elementCount;
                try {
                    elementCount = Integer.parseInt(s);
                } catch (NumberFormatException ex) {
                    elementCount = Globals.DEFAULT_BOX_GENERATION_COUNT;
                }
                _globals.boxGenerationCount = elementCount;
                updateHints();

            }

            startWait();
            ArrangeStyle oldAS = _drawing.getArrangeStyle();
            _drawing.getContentGenerator().generateTree(
                    _globals.boxGenerationCount,
                    Globals.defaultGeneratorArea,
                    _drawing.getCurrentLineType());
            _drawing.setArrangeStyle(oldAS);

            _drawing.arrangeDiagram();

            endWait();

            setMode(ViewportMode.EDIT_MOVE_RESIZE);
            _mainView.zoomer.zoomToViewport();
            updateAll();
        }
    };
    @SuppressWarnings("serial")
    public Action generateGraphAction = new AbstractAction("Generate Graph") {
        {
            putValue(SMALL_ICON, new ImageIcon(getClass().getResource("/images/test-generatetreeplus.png")));
        }

        @Override
        public void actionPerformed(ActionEvent e) {

            if (isFullAction(e)) {

            }

            startWait();
            // start with a tree
            ArrangeStyle oldAS = _drawing.getArrangeStyle();
            _drawing.getContentGenerator().generateTree(
                    _globals.boxGenerationCount,
                    Globals.defaultGeneratorArea,
                    _drawing.getCurrentLineType());


            // then add extra edges to it
            _drawing.getContentGenerator().generateSingleRandomLines(
                    _globals.lineGenerationCount,
                    _drawing.getCurrentLineType());


            _drawing.setArrangeStyle(oldAS);
            _drawing.arrangeDiagram();

            endWait();

            setMode(ViewportMode.EDIT_MOVE_RESIZE);
            _mainView.zoomer.zoomToViewport();
            updateAll();
        }
    };
    @SuppressWarnings("serial")
    public Action generateRandomLinesAction = new AbstractAction("Generate Lines") {
        {
//				putValue(SHORT_DESCRIPTION, String.format("Generate %d random lines; Shift to change N",
//				_globals.lineGenerationCount));
//		putValue(SMALL_ICON, new ImageIcon("resources/images/generatelines.png"));
            putValue(SMALL_ICON, new ImageIcon(getClass().getResource("/images/test-generatelines.png")));
        }

        @Override
        public void actionPerformed(ActionEvent e) {

            if (isFullAction(e)) {
                String s = JOptionPane.showInputDialog(
                        getUiParentComponent(),
                        "Number of lines to be generated:",
                        _globals.lineGenerationCount);
                if (s == null) {
                    return;
                }

                int elementCount;
                try {
                    elementCount = Integer.parseInt(s);
                } catch (NumberFormatException ex) {
                    elementCount = Globals.DEFAULT_LINE_GENERATION_COUNT;
                }
                _globals.lineGenerationCount = elementCount;
                updateHints();
            }

            startWait();
            _drawing.getContentGenerator().generateRandomLines(
                    _globals.lineGenerationCount,
                    _drawing.getCurrentLineType());
            endWait();
            setMode(ViewportMode.EDIT_MOVE_RESIZE);
            updateAll();
        }
    };
    @SuppressWarnings("serial")
    public Action generateRandomSingleLinesAction = new AbstractAction("Generate Single Lines") {
        {
            putValue(SMALL_ICON, new ImageIcon(getClass().getResource("/images/test-generatesinglelines.png")));
        }

        @Override
        public void actionPerformed(ActionEvent e) {

            if (isFullAction(e)) {
                String s = JOptionPane.showInputDialog(
                        getUiParentComponent(),
                        "Number of single lines to be generated:",
                        _globals.lineGenerationCount);
                if (s == null) {
                    return;
                }

                int elementCount;
                try {
                    elementCount = Integer.parseInt(s);
                } catch (NumberFormatException ex) {
                    elementCount = Globals.DEFAULT_LINE_GENERATION_COUNT;
                }
                _globals.lineGenerationCount = elementCount;
                updateHints();
            }

            startWait();
            _drawing.getContentGenerator().generateSingleRandomLines(
                    _globals.lineGenerationCount,
                    _drawing.getCurrentLineType());
            endWait();
            setMode(ViewportMode.EDIT_MOVE_RESIZE);
            updateAll();
        }
    };
    @SuppressWarnings("serial")
    public Action generateNestedRandomBoxesAction = new AbstractAction("Generate Nested Boxes") {
        {
            putValue(SMALL_ICON, new ImageIcon(getClass().getResource("/images/test-generateinnerboxes.png")));
        }

        @Override
        public void actionPerformed(ActionEvent e) {

            if (isFullAction(e)) {
                String s = JOptionPane.showInputDialog(
                        getUiParentComponent(),
                        "Number of nested boxes to be generated:",
                        _globals.boxGenerationCount);
                if (s == null)
                    return;

                int elementCount;
                try {
                    elementCount = Integer.parseInt(s);
                } catch (NumberFormatException ex) {
                    elementCount = Globals.DEFAULT_BOX_GENERATION_COUNT;
                }
                _globals.boxGenerationCount = elementCount;
                updateHints();
            }

            startWait();
            _drawing.getContentGenerator().generateNestedRandomBoxes(_globals.boxGenerationCount);
            endWait();
            setMode(ViewportMode.EDIT_MOVE_RESIZE);
            updateAll();
        }
    };
    @SuppressWarnings("serial")
    public Action generateShortRandomLinesAction = new AbstractAction("Generate Short Lines") {
        {
            putValue(SMALL_ICON, new ImageIcon(getClass().getResource("/images/test-generateshortlines.png")));
        }

        @Override
        public void actionPerformed(ActionEvent e) {

            if (isFullAction(e)) {
                String s = JOptionPane.showInputDialog(
                        getUiParentComponent(),
                        "Number of short lines to be generated:",
                        _globals.lineGenerationCount);
                if (s == null) {
                    return;
                }

                int elementCount;
                try {
                    elementCount = Integer.parseInt(s);
                } catch (NumberFormatException ex) {
                    elementCount = Globals.DEFAULT_LINE_GENERATION_COUNT;
                }
                _globals.lineGenerationCount = elementCount;
                updateHints();
            }

            startWait();
            _drawing.getContentGenerator().generateShortRandomLines(
                    _globals.lineGenerationCount,
                    _drawing.getCurrentLineType());
            endWait();
            setMode(ViewportMode.EDIT_MOVE_RESIZE);
            updateAll();
        }
    };
    @SuppressWarnings("serial")
    public Action showOptionsDialogAction = new AbstractAction("Show Options") {
        {
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            _globals.showDialog();
        }

    };
    @SuppressWarnings("serial")
    public Action cycleModesAction = new AbstractAction("cycle modes") {
        {
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            if ((e.getModifiers() & KeyEvent.SHIFT_DOWN_MASK) != 0)
                cycleModes(true);
            else
                cycleModes(false);
        }
    };
    @SuppressWarnings("serial")
    public Action setBoxSpacingAction = new AbstractAction("set box spacing") {
        {
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            // set the box spacing
            double oldBoxSpacing = _drawing.getCurrentBoxSpacing();

            startWait();
            if ((e.getModifiers() & ActionEvent.SHIFT_MASK) != 0) {
                _drawing.setCurrentBoxSpacing(Math.max(oldBoxSpacing - 1.0, 1.0));
            } else {
                _drawing.setCurrentBoxSpacing(oldBoxSpacing + 1.0);
            }
            endWait();
            updateAll();

        }

    };
    @SuppressWarnings("serial")
    public Action setLineSpacingAction = new AbstractAction("set line spacing") {
        {
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            // set the line spacing
            double oldLineSpacing = _drawing.getCurrentLineSpacing();

            startWait();
            if ((e.getModifiers() & ActionEvent.SHIFT_MASK) != 0) {
                _drawing.setCurrentLineSpacing(Math.max(oldLineSpacing - 1.0, 1.0));
            } else {
                _drawing.setCurrentLineSpacing(oldLineSpacing + 1.0);
            }
            endWait();
            updateAll();

        }
    };
    @SuppressWarnings("serial")
    public Action zoomToContentsAction = new AbstractAction("Zoom to Contents") {
        {
            putValue(SHORT_DESCRIPTION, "Zoom viewport to _drawing contents");
            putValue(SMALL_ICON, new ImageIcon(getClass().getResource("/images/zoom-fit.png")));
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            _mainView.zoomer.zoomToViewport();
            updateAll();

        }
    };
    @SuppressWarnings("serial")
    public Action resetZoomAndScrollAction = new AbstractAction("Reset Zoom") {
        {
            putValue(SHORT_DESCRIPTION, "Reset zoom and scroll");
            putValue(SMALL_ICON, new ImageIcon(getClass().getResource("/images/zoom-original.png")));
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            _mainView.zoomer.resetViewport();
            updateAll();
        }

    };
    @SuppressWarnings("serial")
    public Action zoomInAction = new AbstractAction("Zoom in") {
        {
            putValue(SHORT_DESCRIPTION, "Zoom in viewport");
            putValue(SMALL_ICON, new ImageIcon(getClass().getResource("/images/zoom-in.png")));
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            _mainView.zoomer.zoomInViewport(); // TODO
            updateAll();
        }

    };
    @SuppressWarnings("serial")
    public Action zoomOutAction = new AbstractAction("Zoom out") {
        {
            putValue(SHORT_DESCRIPTION, "Zoom out viewport");
            putValue(SMALL_ICON, new ImageIcon(getClass().getResource("/images/zoom-out.png")));
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            _mainView.zoomer.zoomOutViewport(); // TODO
            updateAll();

        }

    };
    @SuppressWarnings("serial")
    public Action editCutAction = new AbstractAction("Cut") {
        {
            putValue(SHORT_DESCRIPTION, "Cut selection to clipboard");
            putValue(SMALL_ICON, new ImageIcon(getClass().getResource("/images/edit-cut.png")));
        }

        @Override
        public void actionPerformed(ActionEvent e) {

        }

    };
    @SuppressWarnings("serial")
    public Action editCopyAction = new AbstractAction("Copy") {
        {
            putValue(SHORT_DESCRIPTION, "Copy selection to clipboard");
            putValue(SMALL_ICON, new ImageIcon(getClass().getResource("/images/edit-copy.png")));
        }

        @Override
        public void actionPerformed(ActionEvent e) {

        }

    };
    @SuppressWarnings("serial")
    public Action editPasteAction = new AbstractAction("Paste") {
        {
            putValue(SHORT_DESCRIPTION, "Paste from clipboard");
            putValue(SMALL_ICON, new ImageIcon(getClass().getResource("/images/edit-paste.png")));
        }

        @Override
        public void actionPerformed(ActionEvent e) {

        }

    };
    @SuppressWarnings("serial")
    public Action editDeleteAction = new AbstractAction("Delete") {
        {
            putValue(SHORT_DESCRIPTION, "Delete selection");
            putValue(SMALL_ICON, new ImageIcon(getClass().getResource("/images/edit-delete.png")));
        }

        @Override
        public void actionPerformed(ActionEvent e) {

            DElement currentElement = _drawing.getCurrent();

            if (currentElement != null) {
                _drawing.removeElement(currentElement);
                updateAll();
            }
        }

    };
    @SuppressWarnings("serial")
    public Action showHelpAction = new AbstractAction("Help") {
        {
            putValue(SHORT_DESCRIPTION, "Show help with the available commands");
            putValue(SMALL_ICON, new ImageIcon(getClass().getResource("/images/help.png")));
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            showHelpDialogHTML();
        }

    };
    @SuppressWarnings("serial")
    public Action userAction = new AbstractAction("UserAction") {
        {
            putValue(SHORT_DESCRIPTION, "Perform user defined action");
            putValue(SMALL_ICON, new ImageIcon(getClass().getResource("/images/test-useraction.png")));
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            startWait();

            _drawing.clear();
            DDiagram d = _drawing.getDiagram();
            d.setArrangeStyle(ArrangeStyle.SPRING_EMBEDDED);

            // do something
            DGroup[] groups = new DGroup[10];
            DBox[] boxes = new DBox[50];
            DLine[] lines = new DLine[100];

            DGroup g;
            DBox b;
            DLine l;

            _drawing.startTransaction();
            for (int i = 0; i < groups.length; i++) {
                g = d.createGroup();
                groups[i] = g;
                g.createBox();
                g.createBox();
                g.createBox();
                g.createBox();
                g.createBox();
            }
            _drawing.endTransaction();

            _drawing.startTransaction();
            for (int i = 0; i < boxes.length; i++) {
                b = d.createBox(new Rectangle2D.Double(10, 10, 40, 40));
                b.getStyle().setFillColor(new Color(0.5f, 1.0f, 0.5f, 1.0f));
                b.getStyle().setGradientColor(new Color(0.5f, 1.0f, 0.5f, 0.2f));
                b.getStyle().setShadow(false);
                b.getStyle().setStrokeColor(new Color(0.2f, 0.5f, 0.2f, 1.0f));
                boxes[i] = b;
            }
            _drawing.endTransaction();

            _drawing.startTransaction();
            for (int i = 0; i < boxes.length; i++) {
                boxes[i].connectTo(boxes[(i + 1) % boxes.length]);
            }
            _drawing.endTransaction();


            for (DLine line : lines) {

            }

            d.arrange();


            _mainView.zoomer.zoomToViewport();
            endWait();
            updateAll();
        }

    };
    @SuppressWarnings("serial")
    public Action saveToBitmapImageAction = new AbstractAction("Save as picture") {
        {
            putValue(SHORT_DESCRIPTION, "Save _drawing as picture");
//				putValue(SMALL_ICON, new ImageIcon(getClass().getResource("/images/pngimage.png")));
            putValue(SMALL_ICON, new ImageIcon(getClass().getResource("/images/document-snapshot.png")));
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            saveDiagramToBitmapImage();
        }

    };
    @SuppressWarnings("serial")
    public Action saveToDumpAction = new AbstractAction("Save dump") {
        {
            putValue(SHORT_DESCRIPTION, "Save _drawing dump to text file");
            putValue(SMALL_ICON, new ImageIcon(getClass().getResource("/images/pngimage.png"))); // TODO
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            saveDiagramToDump();
            saveDiagramToFile(null);
            saveDiagramToBitmapImage();
        }

    };
    @SuppressWarnings("serial")
    public Action saveToFileAction = new AbstractAction("Save to file") {
        {
            putValue(SHORT_DESCRIPTION, "Save _drawing to file");
            putValue(SMALL_ICON, new ImageIcon(getClass().getResource("/images/document-save.png"))); // TODO
        }

        @Override
        public void actionPerformed(ActionEvent e) {

            String saveComment = null;

            if (isFullAction(e)) {

                saveComment = JOptionPane.showInputDialog(
                        getUiParentComponent(),
                        "Enter the comment for this save:");
                if (saveComment == null) {
                    return;
                }

            }

            saveDiagramToFile(saveComment);
        }

    };
    @SuppressWarnings("serial")
    public Action loadFromFileAction = new AbstractAction("Load from file") {
        {
            putValue(SHORT_DESCRIPTION, "Load _drawing from file");
            putValue(SMALL_ICON, new ImageIcon(getClass().getResource("/images/document-open.png"))); // TODO
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            loadDiagramFromFile();
            _mainView.zoomer.zoomToViewport();
            setMode(ViewportMode.EDIT_MOVE_RESIZE);
            updateAll();
        }

    };
    @SuppressWarnings("serial")
    public Action undoAction = new AbstractAction("Undo") {
        {
            putValue(SHORT_DESCRIPTION, "Undo last operation");
            putValue(SMALL_ICON, new ImageIcon(getClass().getResource("/images/edit-undo.png"))); // TODO
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            // FIXME
        }

    };
    @SuppressWarnings("serial")
    public Action redoAction = new AbstractAction("Redo") {
        {
            putValue(SHORT_DESCRIPTION, "Redo last operation");
            putValue(SMALL_ICON, new ImageIcon(getClass().getResource("/images/edit-redo.png"))); // TODO
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            // FIXME
        }

    };
    @SuppressWarnings("serial")
    public Action insertBoxAction = new AbstractAction("Insert Box") {
        {
            putValue(SHORT_DESCRIPTION, "Insert new box (Shift = insert into");
//				putValue(SMALL_ICON, new ImageIcon("resources/images/box.png"));
            putValue(SMALL_ICON, new ImageIcon(getClass().getResource("/images/insert-box.png")));
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            if ((e.getModifiers() & ActionEvent.SHIFT_MASK) != 0) {
                setMode(ViewportMode.INSERT_BOX, true);
            } else {
                setMode(ViewportMode.INSERT_BOX, false);
            }
//				setMode(ViewportMode.INSERTBOX);
        }

    };
    @SuppressWarnings("serial")
    public Action insertInsideLabelAction = new AbstractAction("Insert Inside Label") {
        {
            putValue(SHORT_DESCRIPTION, "Insert new inside label (Shift = insert into)");
            putValue(SMALL_ICON,
                    new ImageIcon(getClass().getResource("/images/insert-inside-label.png")));
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            if ((e.getModifiers() & ActionEvent.SHIFT_MASK) != 0) {
                setMode(ViewportMode.INSERT_INSIDE_LABEL, true);
            } else {
                setMode(ViewportMode.INSERT_INSIDE_LABEL, false);
            }
        }

    };
    @SuppressWarnings("serial")
    public Action insertOutsideLabelAction = new AbstractAction("Insert Outside Label") {
        {
            putValue(SHORT_DESCRIPTION, "Insert new outside label");
            putValue(SMALL_ICON,
                    new ImageIcon(getClass().getResource("/images/insert-outside-label.png")));
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            if ((e.getModifiers() & ActionEvent.SHIFT_MASK) != 0) {
                setMode(ViewportMode.INSERT_OUTSIDE_LABEL, true);
            } else {
                setMode(ViewportMode.INSERT_OUTSIDE_LABEL, false);
            }
        }

    };
    @SuppressWarnings("serial")
    public Action insertLineLabelAction = new AbstractAction("Insert Line Label") {
        {
            putValue(SHORT_DESCRIPTION, "Insert new line label");
            putValue(SMALL_ICON,
                    new ImageIcon(getClass().getResource("/images/insert-line-label.png")));
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            if ((e.getModifiers() & ActionEvent.SHIFT_MASK) != 0) {
                setMode(ViewportMode.INSERT_LINE_LABEL, true);
            } else {
                setMode(ViewportMode.INSERT_LINE_LABEL, false);
            }
        }

    };
    @SuppressWarnings("serial")
    public Action insertOrthoLineAction = new AbstractAction("Insert Line") {
        {
            putValue(SHORT_DESCRIPTION, "Insert new ortho line");
            putValue(SMALL_ICON, new ImageIcon(getClass().getResource("/images/insert-line-ortho.png")));
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            setMode(ViewportMode.INSERT_ORTHO_LINE, (e.getModifiers() & ActionEvent.SHIFT_MASK) != 0);
        }

    };
    @SuppressWarnings("serial")
    public Action insertSimpleLineAction = new AbstractAction("Insert Straight Line") {
        {
            putValue(SHORT_DESCRIPTION, "Insert new straight line");
            putValue(SMALL_ICON, new ImageIcon(getClass().getResource("/images/insert-line-straight.png")));
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            setMode(ViewportMode.INSERT_SIMPLE_LINE, (e.getModifiers() & ActionEvent.SHIFT_MASK) != 0);
        }

    };
    @SuppressWarnings("serial")
    public Action insertPolyLineAction = new AbstractAction("Insert Polyline") {
        {
            putValue(SHORT_DESCRIPTION, "Insert new poly line");
            putValue(SMALL_ICON, new ImageIcon(getClass().getResource("/images/insert-line-poly.png")));
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            setMode(ViewportMode.INSERT_POLY_LINE, (e.getModifiers() & ActionEvent.SHIFT_MASK) != 0);
        }

    };
    @SuppressWarnings("serial")
    public Action editModeAction = new AbstractAction("Select") {
        {
            putValue(SHORT_DESCRIPTION, "Select / move / resize");
            putValue(SMALL_ICON, new ImageIcon(getClass().getResource("/images/select.png")));
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            setMode(ViewportMode.EDIT_MOVE_RESIZE);
        }

    };
    @SuppressWarnings("serial")
    public Action arrangeDiagramAction = new AbstractAction("Arrange") {
        {
            putValue(SHORT_DESCRIPTION, "Recalculate the _drawing layout from scratch");
            putValue(SMALL_ICON, new ImageIcon(getClass().getResource("/images/layout-arrange.png")));
        }

        @Override
        public void actionPerformed(ActionEvent e) {

            arrangeDiagram();
        }

    };
    @SuppressWarnings("serial")
    public Action toggleMenuBarAction = new AbstractAction("Toggle menu bar") {
        {
            putValue(SHORT_DESCRIPTION, "Toggle the main menu bar visibility");
//				putValue(SMALL_ICON, new ImageIcon("resources/images/select.png"));
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            _host.toggleMenuBar();
        }

    };
    @SuppressWarnings("serial")
    public Action toggleToolBarAction = new AbstractAction("Toggle tool bar") {
        {
            putValue(SHORT_DESCRIPTION, "Toggle the tool bar visibility");
//				putValue(SMALL_ICON, new ImageIcon("resources/images/select.png"));
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            _host.toggleToolBar();
        }

    };
    @SuppressWarnings("serial")
    public Action toggleStatusBarAction = new AbstractAction("Toggle status bar") {
        {
            putValue(SHORT_DESCRIPTION, "Toggle the status bar visibility");
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            _host.toggleStatusBar();
        }

    };
    @SuppressWarnings("serial")
    public Action toggleOverviewAction = new AbstractAction("Toggle overview pane") {
        {
            putValue(SHORT_DESCRIPTION, "Toggle the overview pane visibility");
//				putValue(SMALL_ICON, new ImageIcon("resources/images/select.png"));
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            _host.toggleOverview();
        }

    };
    @SuppressWarnings("serial")
    public Action toggleMagnifierAction = new AbstractAction("Toggle magnifier pane") {
        {
            putValue(SHORT_DESCRIPTION, "Toggle the magnifier pane visibility");
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            _host.toggleMagnifier();
        }

    };
    @SuppressWarnings("serial")
    public Action benchmarkAction = new AbstractAction("Perform benchmark") {
        {
            putValue(SHORT_DESCRIPTION, "Perform benchmark");
            putValue(SMALL_ICON, new ImageIcon(getClass().getResource("/images/test-benchmark.png")));
        }

        @Override
        public void actionPerformed(ActionEvent e) {

            final int N = 10;

            startWait();

            if (_drawing.getBoxCount() > 0) {
                doFastGeneration(1);

            } else {
                _drawing.clear();
                Globals.debugOutputIsOn = false;
                Globals.randomGenerator.setSeed(42);

                doInitialGeneration(N);

            }

            endWait();

            System.out.println("foo ... done");

            _mainView.zoomer.zoomToViewport();
            setMode(ViewportMode.EDIT_MOVE_RESIZE);
            updateAll();

        }

        private void doInitialGeneration(int num) {
            ContentGenerator cm = _drawing.getContentGenerator();
//				cm.setDebug(true);

            // total: 5 * num boxes and 10 * num lines
//				cm.generateRandomBoxes(num, area);
            cm.generateRandomBoxes(num, Globals.defaultGeneratorArea);
            cm.generateNestedRandomBoxes(num);
            cm.generateRandomLines(2 * num, DLine.DLineType.ORTHO);
            cm.generateNestedRandomBoxes(num);
            cm.generateRandomLines(2 * num, DLine.DLineType.ORTHO);
            cm.generateNestedRandomBoxes(num);
            cm.generateShortRandomLines(3 * num, DLine.DLineType.ORTHO);
            cm.generateNestedRandomBoxes(num);
            cm.generateRandomLines(3 * num, DLine.DLineType.ORTHO);
        }

    };
    @SuppressWarnings("serial")
    public Action fastGenerationAction = new AbstractAction("Perform fast generation") {
        {
            putValue(SHORT_DESCRIPTION, "Perform fast generation");
            putValue(SMALL_ICON, new ImageIcon(getClass().getResource("/images/test-fastforward.png")));
        }

        @Override
        public void actionPerformed(ActionEvent e) {

            int numSteps;
            String s = JOptionPane.showInputDialog(
                    getUiParentComponent(),
                    "Steps to fast forward:",
                    _globals.animationSteps);
            if (s == null) {
                return;
            }

            try {
                numSteps = Integer.parseInt(s);
            } catch (NumberFormatException ex) {
                numSteps = Globals.DEFAULT_ANIMATION_STEPS;
            }
            _globals.animationSteps = numSteps;

            startWait();
            doFastGeneration(numSteps);
            endWait();

            System.out.println("foo ... done");

            _mainView.zoomer.zoomToViewport();
            setMode(ViewportMode.EDIT_MOVE_RESIZE);
            updateAll();

        }

    };
    @SuppressWarnings("serial")
    public Action animatedSimulationAction = new AbstractAction("Start animated edit simulation") {
        {
            putValue(SHORT_DESCRIPTION, "Start animated simulation");
            putValue(SMALL_ICON, new ImageIcon(getClass().getResource("/images/test-animate.png")));
        }

        @Override
        public void actionPerformed(ActionEvent e) {

            int numSteps;

            String s = JOptionPane.showInputDialog(
                    getUiParentComponent(),
                    "Steps to animate:",
                    _globals.animationSteps);
            if (s == null) {
                return;
            }

            try {
                numSteps = Integer.parseInt(s);
            } catch (NumberFormatException ex) {
                numSteps = Globals.DEFAULT_ANIMATION_STEPS;
            }
            _globals.animationSteps = numSteps;

            startWait();

            doAnimatedSimulation(numSteps);

        }

        private void doAnimatedSimulation(int num) {

            ContentManipulator cm = _drawing.getContentManipulator();
            StepAnimator animator = new StepAnimator(cm);
            animator.doSteps(num);

        }
    };
    @SuppressWarnings("serial")
    public Action stopAnimationAction = new AbstractAction("Stop animation") {
        {
            putValue(SHORT_DESCRIPTION, "Stop current animation");
            putValue(SMALL_ICON, new ImageIcon(getClass().getResource("/images/test-stop.png")));
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            if (_globals.animationStepsLeft > 0) {

                System.out.println("STOP received...");

                _globals.animationStepsLeft = 0;
                beep();

            }
        }

    };
    @SuppressWarnings("serial")
    public Action setRandomSeedAction = new AbstractAction("Set Random Seed") {
        {
            putValue(SHORT_DESCRIPTION, "Set Random Seed");
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            String s = JOptionPane.showInputDialog(
                    getUiParentComponent(),
                    "Enter the seed:");
            if (s == null) {
                return;
            }

            long newSeed;
            try {
                newSeed = Long.parseLong(s);
            } catch (NumberFormatException ex) {
                newSeed = 100;
            }
            Globals.randomGenerator.setSeed(newSeed);
//				setRandomSeed();
        }

    };
    @SuppressWarnings("serial")
    public Action throwDiceAction = new AbstractAction("Randomize") {
        {
            putValue(SHORT_DESCRIPTION, "Randomize the random seed");
            putValue(SMALL_ICON, new ImageIcon(getClass().getResource("/images/test-setrandom.png")));
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            _globals.randomize();
            updateInfo();
        }

    };


    public DActions() {
        updateHints();
    }

    private void updateHints() {

        generateRandomBoxesAction.putValue(AbstractAction.SHORT_DESCRIPTION,
                String.format("Generate %d random boxes; Shift to change N",
                        _globals.boxGenerationCount));

        generateTreeAction.putValue(AbstractAction.SHORT_DESCRIPTION,
                String.format("Generate tree with %d boxes; Shift to change N",
                        _globals.boxGenerationCount));

        generateGraphAction.putValue(AbstractAction.SHORT_DESCRIPTION,
                String.format("Generate graph (tree with %d boxes, then %d extra lines)",
                        _globals.boxGenerationCount,
                        _globals.lineGenerationCount));

        generateRandomLinesAction.putValue(AbstractAction.SHORT_DESCRIPTION,
                String.format("Generate %d random lines; Shift to change N",
                        _globals.lineGenerationCount));

        generateRandomSingleLinesAction.putValue(AbstractAction.SHORT_DESCRIPTION,
                String.format("Generate %d random lines, skipping multiple; Shift to change N",
                        _globals.lineGenerationCount));

        generateNestedRandomBoxesAction.putValue(AbstractAction.SHORT_DESCRIPTION,
                String.format("Generate %d nested random boxes; Shift to change N",
                        _globals.boxGenerationCount));

        generateShortRandomLinesAction.putValue(AbstractAction.SHORT_DESCRIPTION,
                String.format("Generate %d short random lines; Shift to change N",
                        _globals.lineGenerationCount));

        generateRandomInsideLabelsAction.putValue(AbstractAction.SHORT_DESCRIPTION,
                String.format("Generate %d inside labels; Shift to change N",
                        _globals.labelGenerationCount));

        generateRandomOutsideLabelsAction.putValue(AbstractAction.SHORT_DESCRIPTION,
                String.format("Generate %d outside labels; Shift to change N",
                        _globals.labelGenerationCount));

        generateRandomLineLabelsAction.putValue(AbstractAction.SHORT_DESCRIPTION,
                String.format("Generate %d line labels; Shift to change N",
                        _globals.labelGenerationCount));

    }

    private boolean isFullAction(ActionEvent e) {
        if (e.getSource().getClass() == DViewport.class) {
            return true;
        } else {
            if ((e.getModifiers() & ActionEvent.SHIFT_MASK) != 0) {
                return true;
            }
            return false;
        }
    }

    /**
     * Perform {@code num} generation steps without visual feedback.
     *
     * @param num number of steps
     */
    private void doFastGeneration(int num) {

        ContentManipulator cm = _drawing.getContentManipulator();
        for (int i = 1; i <= num; i++) {
            cm.doOneEditingStep();
            if (i % 100 == 0) {
                System.out.println(String.format("%d steps performed", i));
                updateAll();
            }
        }

    }

    private class StepAnimator implements ActionListener {

        private Timer stepTimer;
        private ContentManipulator cm;

        public StepAnimator(ContentManipulator cm) {
            this.cm = cm;
        }

        public void doSteps(int numSteps) {
            _globals.animationStepsLeft = (numSteps > 0 ? numSteps : 1);

            stepTimer = new Timer(Globals.ANIMATION_STEP_DELAY, this);
            stepTimer.start();
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            _globals.animationStepsLeft--;
            if (_globals.animationStepsLeft < 0) {
                stepTimer.stop();
                stepTimer = null;

                System.out.println("foo ... done");
                endWait();
                setMode(ViewportMode.EDIT_MOVE_RESIZE);
            } else {
                cm.doOneEditingStep();
                _mainView.zoomer.zoomToViewport();
                updateAll();
            }
        }

    }

}

}
