/*
*
* Copyright (c) 2013,2014 Institute of Mathematics and Computer Science, University of Latvia (IMCS UL). 
*
* This file is part of DEEK (diagram editor engine kit). 
*
* You can redistribute it and/or modify it under the terms of the GNU General Public License 
* as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
*
* You should have received a copy of the GNU General Public License along with DEEK If not, see http://www.gnu.org/licenses/.
*
*/

package lv.lumii.deek.ui;

import graph.Edge;
import graph.Graph;
import graph.Node;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.RenderingHints;
import java.awt.color.ColorSpace;
import java.awt.geom.Point2D;
import java.awt.geom.Point2D.Double;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.awt.image.BufferedImageOp;
import java.awt.image.ColorConvertOp;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.*;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.imageio.ImageIO;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerFactoryConfigurationError;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import lv.lumii.deek.model.*;
import lv.lumii.deek.util.*;
import lv.lumii.deek.test.ContentGenerator;
import lv.lumii.deek.test.ContentManipulator;
import lv.lumii.deek.test.ContentManipulatorOne;
import org.w3c.dom.DOMException;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;

import actionblock.ActionBlock;
import lv.lumii.layoutengine.ArrangeData.ArrangeStyle;
import project.ActionBlockProject;
import project.io.xml.ActionBlockProjectXMLReader;


import lv.lumii.layoutengine.LayoutConstraints;
import lv.lumii.layoutengine.Element;
import lv.lumii.layoutengine.LayoutConstraints.ConstraintType;
import lv.lumii.layoutengine.LayoutConstraints.GridLayoutConstraints;
import lv.lumii.layoutengine.OutsideLabel.LineLabel.Orientation;

public class DDrawing {

private static double DISTANCE_THRESHOLD = 100;
public DLogger log = new DLogger();
ArrayList<DBox> selectedBoxes = new ArrayList<>();
private DDiagram _diagram;
private DController _controller;
private Globals _globals = Globals.getInstance();
private long f_transactionStartTime;
private String f_transactionTag;
private boolean f_isInTransaction = false;
private TrackingMode f_trackingMode = TrackingMode.NONE;
private ContentManipulator f_contentManipulator = new ContentManipulatorOne(this);
private boolean f_autoArrange = false;
// -------------------------------------------------------------------------------------------------
private boolean f_autoCleanup = false;
private DElement f_currHover = null;
private DElement f_prevHover = null;
private DElement f_currentElement = null;
// -------------------------------------------------------------------------------------------------
private DLine.DLineType _currentLineType = DLine.DLineType.ORTHO;
private Selection selection = new Selection();
private DiagramBackground f_diagramBackground;

public DDrawing() {
    initDrawing(false);
}

private void initDrawing(boolean resetForTest) {

    ArrangeStyle oldArrangeStyle =
            (_diagram != null)
                    ? _diagram.getArrangeStyle()
                    : ArrangeStyle.FLOW;

    lv.lumii.layoutengine.Diagram newLayout = new lv.lumii.layoutengine.Diagram(
            new Rectangle2D.Double(0, 0, 0, 0),
            oldArrangeStyle,
            ConstraintType.NONE
    );

    _diagram = new DDiagram(newLayout, this);

    f_diagramBackground = new DiagramBackground();

    Globals.resetLogCounter();

    if (resetForTest)
        Globals.resetRandom();
}

public DDiagram getDiagram() {
    return _diagram;
}

public void setDiagram(DDiagram newDiagram) {
    initDrawing(false);
    _diagram = newDiagram;
}

public void clear() {
    initDrawing(false);
}

public void requestUpdate() {
    if (_controller != null)
        _controller.updateAll();
}

public void zoomToContents() {
    _controller.zoomToContents();
}

public void setController(DController controller) {
    _controller = controller;
}

private void setLayout(lv.lumii.layoutengine.Diagram newLayout) {
}

// -------------------------------------------------------------------------------------------------
public void draw(Graphics2D g) {
    getDiagram().draw(g);
}

// -------------------------------------------------------------------------------------------------
public DBox addBox(Rectangle2D.Double r) {
    return addBox(r, false, null);
}

public DBox addBox(Rectangle2D.Double r,
                   boolean nested,
                   Point2D.Double insertPoint) {

    DBox newBox;
    DContainer theOwner;

    if (!nested) {

        theOwner = getDiagram();

        log.appendToLog(String.format("add top level box %s",
                log.dumpRectangle(r)));

    } else {

        theOwner = getDiagram().getHitBox(insertPoint);
        if (theOwner == null) {
            theOwner = getDiagram();

            log.appendToLog(String.format("add nested box %s at location %s which hits nothing",
                    log.dumpRectangle(r),
                    log.dumpPoint(insertPoint)));
        } else {

            log.appendToLog(String.format("add nested box %s at location %s belonging to #%d %s",
                    log.dumpRectangle(r),
                    log.dumpPoint(insertPoint),
                    theOwner.getUuid(),
                    log.dumpContainer(theOwner)));

        }

    }

    Rectangle2D.Double newRect;
    if ((r.width < _globals.getBoxResizeWidthMin()) || (r.height < _globals.getBoxResizeHeightMin())) {
        newRect = new Rectangle2D.Double(r.x, r.y,
                _globals.getBoxAutosizeDefaultWidth(), _globals.getBoxAutosizeDefaultHeight());
    } else {
        newRect = new Rectangle2D.Double(r.x, r.y,
                Math.max(r.width, _globals.getBoxResizeWidthMin()),
                Math.max(r.height, _globals.getBoxResizeHeightMin()));
    }

    if (nested && (theOwner.getLayoutConstraints().getType() == ConstraintType.GRID)) {

        GridLayoutConstraints glc = (GridLayoutConstraints) theOwner.getLayoutConstraints();

        _dumpGridBorders(glc);
        System.out.println(String.format("point (%f %f) assigned to grid(C%s R%s)",
                insertPoint.x, insertPoint.y,
                getGridColumnFromPoint(glc, insertPoint),
                getGridRowFromPoint(glc, insertPoint)));
        startTransaction("create Box in grid (!)"); // TODO temp workaround - remove when fixed
        newBox = theOwner.createBox(newRect,
                ArrangeStyle.INHERITED,
                ConstraintType.NONE,
                getCurrentBoxSpacing(),
                getGridRowFromPoint(glc, insertPoint),
                getGridColumnFromPoint(glc, insertPoint));
        endTransaction(); // TODO temp workaround - remove when fixed

    } else {
        newBox = theOwner.createBox(newRect);
    }

    log.appendToLog(String.format("box #%d added: %s",
            newBox.getUuid(),
            log.dumpContainer(newBox)));

    f_currentElement = newBox;

    tryAutoArrange();

    return newBox;
}

public DInsideLabel addInsideLabel(java.awt.geom.Rectangle2D.Double r, boolean nested,
                                   Double insertPoint) {

    DInsideLabel newLabel;
    DContainer theOwner;

    if (!nested) {

        theOwner = getDiagram();

    } else {

        theOwner = getDiagram().getHitBox(insertPoint);
        if (theOwner == null) {
            theOwner = getDiagram();
        }

    }

    Rectangle2D.Double newRect;
    if ((r.width < _globals.getBoxResizeWidthMin()) || (r.height < _globals.getBoxResizeHeightMin())) {
        newRect = new Rectangle2D.Double(r.x, r.y,
                _globals.getBoxAutosizeDefaultWidth(), _globals.getBoxAutosizeDefaultHeight());
    } else {
        newRect = new Rectangle2D.Double(r.x, r.y,
                Math.max(r.width, _globals.getBoxResizeWidthMin()),
                Math.max(r.height, _globals.getBoxResizeHeightMin()));
    }

    if (nested && (theOwner.getLayoutConstraints().getType() == ConstraintType.GRID)) {

        GridLayoutConstraints glc = (GridLayoutConstraints) theOwner.getLayoutConstraints();

        newLabel = theOwner.createInsideLabel(
                newRect,
                ConstraintType.NONE,
                getCurrentBoxSpacing(),
                getGridRowFromPoint(glc, insertPoint),
                getGridColumnFromPoint(glc, insertPoint));

    } else {
        newLabel = theOwner.createInsideLabel(
                newRect,
                ConstraintType.NONE,
                getCurrentBoxSpacing(),
                null, null);
    }

    f_currentElement = newLabel;

    tryAutoArrange();

    return newLabel;
}

public DOutsideLabel addOutsideLabel(java.awt.geom.Rectangle2D.Double r,
                                     Double insertPoint, DBox box) {

    DOutsideLabel newLabel = box.createOutsideLabel(
            Math.max(r.width, _globals.labelAutosizeWidth),
            Math.max(r.height, _globals.labelAutosizeHeight),
            insertPoint,
            ConstraintType.NONE,
            getCurrentLineSpacing());

    f_currentElement = newLabel;

    tryAutoArrange();

    return newLabel;
}

public DLineLabel addLineLabel(java.awt.geom.Rectangle2D.Double r,
                               Point2D.Double insertPoint, DLine line) {

    DLineLabel newLabel = line.createLabel(
            Math.max(r.width, _globals.labelAutosizeWidth),
            Math.max(r.height, _globals.labelAutosizeHeight),
            insertPoint,
            Orientation.LEFT_TOP,
            ConstraintType.NONE,
            getCurrentLineSpacing());

    f_currentElement = newLabel;

    tryAutoArrange();

    return newLabel;
}

public DBox addBox(Rectangle2D.Double r, DBox owner) {

    log.appendToLog(String.format("add nested box %s to box #%d %s",
            log.dumpRectangle(r),
            owner.getUuid(),
            log.dumpContainer(owner)));

    DBox newBox = owner.createBox(r);

    f_currentElement = newBox;

    log.appendToLog(String.format("nested box #%d added; result: %s in #%d %s",
            newBox.getUuid(),
            log.dumpContainer(newBox),
            owner.getUuid(),
            log.dumpContainer(owner)
    ));

    return newBox;
}

private Integer getGridRowFromPoint(GridLayoutConstraints glc, Point2D.Double point) {

    for (int i = 1; i <= glc.getRowCount(); i++) {
        if ((glc.getRowTop(i) <= point.y) && (glc.getRowBottom(i) >= point.y))
            return i;
    }
    return null;
}

private Integer getGridColumnFromPoint(GridLayoutConstraints glc, Point2D.Double point) {


    for (int i = 1; i <= glc.getColumnCount(); i++) {
        if ((glc.getColumnLeft(i) <= point.x) && (glc.getColumnRight(i) >= point.x))
            return i;
    }

    return null;
}

private void _dumpGridBorders(GridLayoutConstraints glc) {
    System.out.println(String.format("GRID %d x %d", glc.getColumnCount(), glc.getRowCount()));
    System.out.println("COLUMNS:");
    for (int i = 1; i <= glc.getColumnCount(); i++) {
        System.out.println(String.format("L=%f W=%f R=%f", glc.getColumnLeft(i), glc.getColumnWidth(i), glc.getColumnRight(i)));
    }
    System.out.println("ROWS:");
    for (int i = 1; i <= glc.getRowCount(); i++) {
        System.out.println(String.format("T=%f H=%f B=%f", glc.getRowTop(i), glc.getRowHeight(i), glc.getRowBottom(i)));
    }
}

// ---------------------------------------------------
public DLine addLine(ArrayList<Double> points, DLine.DLineType lineType) {

    Point2D.Double startPoint = points.get(0);
    Point2D.Double endPoint = points.get(points.size() - 1);

    log.appendToLog(String.format("create line from point %s to point %s",
            log.dumpPoint(startPoint),
            log.dumpPoint(endPoint)
    ));

    DBox startBox = getDiagram().getHitBox(startPoint);

    if (startBox != null) {

        DBox endBox = getDiagram().getHitBox(endPoint);
        if (endBox != null) {

            switch (lineType) {
                case STRAIGHT:
                    if (hasLineAutoCleanup()) {
                        return addLine(startBox, endBox, lineType);
                    } else {
                        return addLine(startBox, endBox, lineType, points);
                    }

                case ORTHO:
                    if (points.size() == 2) {
                        return addLine(startBox, endBox, lineType);
                    } else {
                        return addLine(startBox, endBox, lineType, points);
                    }

                case POLY:
                    return addLine(startBox, endBox, lineType, points);

                default:
                    return addLine(startBox, endBox, lineType, points);

            }

        }
    }

    return null;
}

public DLine addLine(DBox startBox, DBox endBox, DLine.DLineType lineType) {

    return addLine(startBox, endBox, lineType, null);
}

public DLine addLine(DBox startBox, DBox endBox, DLine.DLineType lineType, ArrayList<Double> points) {
    log.appendToLog(String.format("create %s line from box #%d %s to box #%d %s (trace: %s)",
            lineType.toString(),
            startBox.getUuid(),
            log.dumpContainer(startBox),
            endBox.getUuid(),
            log.dumpContainer(endBox),
            points != null ? points.toString() : "none"));

//		if (startBox == endBox) return null; // don't create loops

    DLine newLine;

    if (points != null) {
        newLine = startBox.connectTo(endBox,
                lineType,
                getCurrentLineSpacing(),
                points,
                hasLineAutoCleanup());
    } else {
        newLine = startBox.connectTo(endBox,
                lineType,
                getCurrentLineSpacing());
    }

    log.appendToLog(String.format("result: %s",
            log.dumpLine(newLine)));

    f_currentElement = newLine;

    tryAutoArrange();

    return newLine;
}

// ---------------------------------------------------
public Rectangle2D.Double getBounds() {
    return getDiagram().getBounds();
}

// ---------------------------------------------------
public void arrangeDiagram() {

    // cannot arrange NONE
    if (getDiagram().getArrangeStyle() == ArrangeStyle.NONE) {
        return;
    }

    for (DLine l : getDiagram().allLines()) {
        l.resetStartSides();
        l.resetEndSides();
    }

    getDiagram().arrange();
}

// ---------------------------------------------------

// ---------------------------------------------------
public double getCurrentBoxSpacing() {
    return _globals.getCurrentBoxSpacing();
}

public void setCurrentBoxSpacing(double newSpacing) {
    _globals.setCurrentBoxSpacing(newSpacing);

    tryAutoArrange();
}

public double getCurrentLineSpacing() {
    return _globals.getCurrentLineSpacing();
}

public void setCurrentLineSpacing(double newSpacing) {
    _globals.setCurrentLineSpacing(newSpacing);
}

// ---------------------------------------------------
public boolean hasTransactionsSupported() {
    if (isManualMode())
        return false;

    return true;
}

public void startTransaction() {
    startTransaction("noname");
}

public void startTransaction(String tag) {

    if (hasTransactionsSupported()) {
        getDiagram().startTransaction();
        f_isInTransaction = true;
        f_transactionTag = tag;

        log.appendToLog(String.format("*** Transaction '%s' started ***", tag));

        f_transactionStartTime = System.currentTimeMillis();
    }

}

public void endTransaction() {
    if (hasTransactionsSupported()) {

        long inTime = System.currentTimeMillis();
        f_isInTransaction = false;
        getDiagram().endTransaction();

        long outTime1 = System.currentTimeMillis();

        long deltaClosing = outTime1 - inTime;
        long deltaInside = inTime - f_transactionStartTime;
        long deltaFull = outTime1 - f_transactionStartTime;

        System.out.println(String.format("Transaction '%s' time (s): ( %.2f + %.2f = %.2f )",
                f_transactionTag,
                deltaInside / 1000.0, deltaClosing / 1000.0, deltaFull / 1000.0));

        log.appendToLog(String.format("*** Transaction finished in %d ms ***", deltaClosing));

        if (tryAutoArrange()) {
            long deltaTime2 = System.currentTimeMillis() - outTime1;
            log.appendToLog(String.format("*** Post-Transaction Arrange finished in %d ms ***", deltaTime2));
        }

    }
}

public boolean isInTransaction() {
    return f_isInTransaction;
}

public int getBoxCount() {
    int i = 0;
    for (DBox e : getDiagram().allBoxes())
        i++;
    return i;
}

public int getLineCount() {
    int i = 0;
    for (DLine e : getDiagram().allLines())
        i++;
    return i;
}


// ---------------------------------

public String dumpDrawing() {
    return DebugHelper.dumpDrawing(this);
}

private BufferedImage getBufferedImage() {

    Rectangle2D.Double r = getBounds();

    int w = Math.max(1, (int) r.getWidth());
    int h = Math.max(1, (int) r.getHeight());

    BufferedImage bi = new BufferedImage(w, h, BufferedImage.TYPE_INT_RGB);
    Graphics2D g2 = bi.createGraphics();

    g2.setRenderingHint(
            RenderingHints.KEY_TEXT_ANTIALIASING,
            RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
    g2.setRenderingHint(
            RenderingHints.KEY_ANTIALIASING,
            RenderingHints.VALUE_ANTIALIAS_ON);
    g2.setRenderingHint(
            RenderingHints.KEY_STROKE_CONTROL,
            RenderingHints.VALUE_STROKE_NORMALIZE);

    g2.setColor(Color.WHITE);
    g2.fillRect(0, 0, w, h);

    g2.translate(-r.getX(), -r.getY());
    draw(g2);

    return bi;
}

public boolean saveToImage(String filename, String imageFormat) {

    if ((filename == null) || (imageFormat == null)) {
        throw new IllegalArgumentException();
    }

    File imageFile = new File(String.format("%s.%s", filename, imageFormat));

    try {
        System.out.print(String.format("Saving as %s to %s ...",
                imageFormat, imageFile.getAbsolutePath()));

        ImageIO.write(getBufferedImage(), imageFormat, imageFile);

        System.out.println(" done");
    } catch (IOException e) {
        e.printStackTrace();
    }

    return true;
}

public TrackingMode setTrackingMode(TrackingMode newMode) {
    if (newMode == null)
        f_trackingMode = TrackingMode.NONE;
    else
        f_trackingMode = newMode;

    return f_trackingMode;
}

private long getElementId(Element e) {
    return DElement.getThisId(e);
}

public ContentGenerator getContentGenerator() {
    return f_contentManipulator;
}

public ContentManipulator getContentManipulator() {
    return f_contentManipulator;
}

public void removeElement(DElement e) {
    e.remove();
}

public ArrangeStyle getArrangeStyle() {
    return getDiagram().getArrangeStyle();
}

public void setArrangeStyle(ArrangeStyle arrangeStyle) {
    if (getDiagram().getArrangeStyle() != arrangeStyle) {
        getDiagram().setArrangeStyle(arrangeStyle);
        arrangeDiagram();
    }
}

public void setAutoArrange(boolean autoArrange) {
    this.f_autoArrange = autoArrange;
}

public boolean hasAutoArrange() {
    return f_autoArrange;
}

public boolean hasLineAutoCleanup() {
    return f_autoCleanup;
}

public void setLineAutoCleanup(boolean autoCleanup) {
    this.f_autoCleanup = autoCleanup;
}

private boolean tryAutoArrange() {
    if (!hasAutoArrange()) return false;

    // if we are in a transaction then arrange at its end
    if (isInTransaction()) return false;

    getDiagram().arrange();
    return true;
}

public void retraceLine(DLine l) {
    l.retrace();
}

public void retraceAllLines() {
    startTransaction();

    for (DLine l : getDiagram().allLines()) {
        if (l.getLineType() == DLine.DLineType.ORTHO) {
            retraceLine(l);
        }
    }

    endTransaction();
}

public void cleanupOrthoLine(DLine l) {

    if (l.getLineType() != DLine.DLineType.ORTHO) {
//			throw new UnsupportedOperationException("You can clean up ortholinear lines only.");
    }

    l.cleanup();
}

public void cleanupAllOrthoLines() {
    startTransaction();

    for (DLine l : getDiagram().allLines()) {
        if (l.getLineType() == DLine.DLineType.ORTHO) {
            cleanupOrthoLine(l);
        }
    }

    endTransaction();
}

public void convertAllLinesToType(DLine.DLineType lineType) {

    setCurrentLineType(lineType);

    startTransaction();

    for (DLine l : getDiagram().allLines()) {
        if (l.getLineType() != lineType)
            l.setLineType(lineType);
    }

    endTransaction();
}

public void setBackground(BufferedImage img) {
    f_diagramBackground.setImage(img);
    f_diagramBackground.makeGrayscale();
}

public boolean setBackground(File file) {
    return f_diagramBackground.setImage(file);
}

public boolean isBackgroundVisible() {
    return f_diagramBackground.isVisible();
}

public void setBackgroundVisible(boolean visible) {
    f_diagramBackground.setVisible(visible);
}

public boolean recognizeBackground() {

    if (f_diagramBackground.hasImage()) {

        System.out.println("Recognizer started ...");

        ActionBlockProjectXMLReader reader = new ActionBlockProjectXMLReader();
        reader.read(new File("lib\\tvizRealEdges.abpx"));
        ActionBlockProject project = reader.getProject();

        project.setInitialMemoryValue("InputImage", f_diagramBackground.getImage());

        project.run();

        ActionBlock ab = project.getMainBlock();
//			System.out.println("ab : " + (ab != null));
        Map<String, Object> mem = ab.getMemory();
//			System.out.println("mem : " + (mem != null));
        Graph g = (Graph) mem.get("OutputGraph");

//			Graph g = (Graph) project.getMainBlock().getMemory().get("OutputGraph");

        System.out.println("Recognizer finished, transforming result ...");

        return transformGraphToDiagram(g);
    } else {
        System.out.println("No background to recognize!");
        return false;
    }
}

private boolean transformGraphToDiagram(Graph g) {

    System.out.println("Grafs ar : " + g.nodes().size() + " nodem");
    System.out.println("un " + g.edges().size() + " skautnem");

    Map<Node, Integer> mi = new HashMap<>();

    for (Node n : g.nodes()) {
        mi.put(n, mi.size());
        System.out.println("Node (" + mi.get(n) + ") Center (" +
                n.getCenter().getX() + ";" + n.getCenter().getY() + ")");
    }

    for (Edge e : g.edges()) {
        System.out.println("Edge: " + mi.get(e.getSourceNode()) + " " +
                mi.get(e.getTargetNode()));
    }

    DBox b1, b2;
    Node n1, n2;

    DLine l;

    geometry.DPoint gp;
    List<geometry.DPoint> ebp;

    Point2D.Double lp;
    ArrayList<Point2D.Double> llp;

    Map<Node, DBox> n2b = new HashMap<>();


    // start with an empty diagram
    this.clear();

    // reset constraints
    getDiagram().setConstraintType(ConstraintType.NONE);

    // reset arrange style
    getDiagram().setArrangeStyle(ArrangeStyle.NONE);

    // build boxes, map nodes to boxes
    startTransaction(String.format("construct %d boxes", g.nodes().size()));
    for (Node n : g.nodes()) {
        b1 = addBox(new Rectangle2D.Double(
                n.getX() - n.getWidth() / 2,
                n.getY() - n.getHeight() / 2,
                n.getWidth(),
                n.getHeight()
        ));
//			new DBox(b1);
        n2b.put(n, b1);
    }
    endTransaction();

    // connect the boxes
    startTransaction(String.format("connect %d lines", g.edges().size()));
    for (Edge e : g.edges()) {
        n1 = e.getSourceNode();
        b1 = n2b.get(n1);

        n2 = e.getTargetNode();
        b2 = n2b.get(n2);

        // set the trace
        llp = new ArrayList<>();

        ebp = e.getBreakPoints();
//			System.out.println("edge points: " + ebp.size());

        // check if we need to add a start point
        boolean addStartPoint = false;
        if (ebp.size() < 1) {
            addStartPoint = true;
        } else {
            gp = ebp.get(0);
            lp = new Point2D.Double(gp.getX(), gp.getY());
//				if (!b1.contains(lp, 0)) {
            if (!b1.isSelfHit(lp)) {
                addStartPoint = true;
            }
        }
        if (addStartPoint)
            llp.add(new Point2D.Double(n1.getX(), n1.getY()));

        // add edge break points
        for (geometry.DPoint p : ebp) {
            llp.add(new Point2D.Double(p.getX(), p.getY()));
        }

        // check if we need to add a finish point
        boolean addFinishPoint = false;
        if (ebp.size() < 2) {
            addFinishPoint = true;
        } else {
            gp = ebp.get(ebp.size() - 1);
            lp = new Point2D.Double(gp.getX(), gp.getY());
//				if (!b2.contains(lp, 0)) {
            if (!b2.isSelfHit(lp)) {
                addFinishPoint = true;
            }
        }
        if (addFinishPoint)
            llp.add(new Point2D.Double(n2.getX(), n2.getY()));

        DLine.DLineType lt = DLine.DLineType.STRAIGHT;

        // ad hoc - guess the line type
        if (llp.size() == 2) {

            lt = (llp.get(0).getX() == llp.get(1).getX()) ||
                    (llp.get(0).getY() == llp.get(1).getY()) ?
                    DLine.DLineType.ORTHO : DLine.DLineType.STRAIGHT;

        } else if (llp.size() > 2) {

            boolean isOrtho = true;

            for (int i = 0; i < llp.size() - 1; i++) {

                isOrtho = isOrtho &&
                        (llp.get(i).getX() == llp.get(i + 1).getX()) ||
                        (llp.get(i).getY() == llp.get(i + 1).getY());

            }

            lt = isOrtho
                    ? DLine.DLineType.ORTHO
                    : DLine.DLineType.POLY;

        }

        // make the connection
        l = b1.connectTo(b2, lt, getCurrentLineSpacing(), llp);
//			new DLine(l);

        // set the line points
//			l.setPoints(llp);

    }
    endTransaction();

    n2b = null;

    return true;
}

public BufferedImage getDiagramImage() {
    return getBufferedImage();
}

public LayoutConstraints getLayoutConstraints() {
    return getDiagram().getLayoutConstraints();
}

public void setConstraintType(ConstraintType constraintType) {
    getDiagram().setConstraintType(constraintType);
}

public void saveToFile(String filename, String comment) {
    saveToXMLFile(new File(filename), comment);
}

public void loadFromFile(String filename) {
    loadFromXMLFile(new File(filename));
}

public void loadFromGraph(File file) {
    List<String> sl = new ArrayList<>();
    try (BufferedReader br = new BufferedReader(new FileReader(file))) {
        String line;
        char firstChar;
        while ((line = br.readLine()) != null) {
            line = line.trim();

            // skip empty lines
            if (line.length() == 0) continue;

            // skip comment lines
            if (line.charAt(0) == '#') continue;


        }


    } catch (IOException e) {
        e.printStackTrace();
    }
}

//**********************************************************//

public void saveToGraph(File file) {
    GraphIO.saveTo(file);
}

private void saveAttributes(org.w3c.dom.Element e) {
    e.setAttribute("idSeed", Long.toString(_globals.randomSeed));
}

private void loadAttributes(org.w3c.dom.Element e) {
    _globals.randomSeed = Long.valueOf(e.getAttribute("idSeed"));
}

public void saveToXMLFile(File file, String comment) {
    try {
        System.out.println("saving to " + file.getName());

        DocumentBuilderFactory docFactory = DocumentBuilderFactory.newInstance();
        DocumentBuilder docBuilder = docFactory.newDocumentBuilder();
        Document doc = docBuilder.newDocument();

        org.w3c.dom.Element e = doc.createElement(XMLHelper.DRAWING_ELEMENT);

        e.setAttribute("xmlns:" + XMLHelper.NAMESPACE, XMLHelper.NAMESPACE_URI);

        if (comment != null) {
            e.setAttribute("comment", comment);
        }

        saveAttributes(e);

        e.appendChild(getDiagram().saveToXML(doc));
        doc.appendChild(e);

        // write document to file
        TransformerFactory transformerFactory = TransformerFactory.newInstance();
        Transformer transformer = transformerFactory.newTransformer();
        transformer.setOutputProperty(OutputKeys.INDENT, "yes");
        transformer.setOutputProperty("{http://xml.apache.org/xslt}indent-amount", "4");
        DOMSource source = new DOMSource(doc);
        StreamResult result = new StreamResult(file);
        transformer.transform(source, result);

    } catch (ParserConfigurationException | DOMException | TransformerFactoryConfigurationError | IllegalArgumentException | TransformerException ex) {
        Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, "problem when saving", ex);
    }
}

public void loadFromXMLFile(File file) {

    try {
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        DocumentBuilder loader = factory.newDocumentBuilder();

        org.w3c.dom.Document doc = loader.parse(file);
        org.w3c.dom.Element documentElement = doc.getDocumentElement();
        documentElement.normalize();

        System.out.println("Root element = " + documentElement.getNodeName());

//        loadAttributes(documentElement);
//
//        if (documentElement.getNodeName().equals(XMLHelper.DRAWING_ELEMENT)) {
//            diagramElement = documentElement;
//        } else {
//            node = documentElement.getElementsByTagName(XMLHelper.DRAWING_ELEMENT).item(0);
//            diagramElement = (org.w3c.dom.Element) node;
////				System.out.println(layoutElement.getNodeName());
//        }

        org.w3c.dom.Node node;
        org.w3c.dom.Element diagramElement;

        if (documentElement.getNodeName().equals(XMLHelper.DRAWING_ELEMENT)) {
            loadAttributes(documentElement);

            node = documentElement.getElementsByTagName(XMLHelper.DIAGRAM_ELEMENT).item(0);
            diagramElement = (org.w3c.dom.Element) node;

        } else if (documentElement.getNodeName().equals(XMLHelper.DIAGRAM_ELEMENT)) {
            diagramElement = documentElement;
        } else {
            Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, "input file error: diagram element not found");
            throw new IOException("diagram element not found");
        }

        org.w3c.dom.Element layoutElement;

        node = diagramElement.getElementsByTagName(XMLHelper.LAYOUT_ELEMENT).item(0);
        layoutElement = (org.w3c.dom.Element) node;

        LinkedHashMap<Integer, Object> objectMap = new LinkedHashMap<>();

        lv.lumii.layoutengine.Diagram newLayout = new lv.lumii.layoutengine.Diagram(layoutElement, objectMap);


        DDiagram d = new DDiagram(diagramElement, objectMap, newLayout, this);

        setDiagram(d);


    } catch (IOException | SAXException | ParserConfigurationException | FactoryConfigurationError ex) {
        Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, "problem when loading", ex);

        setDiagram(null);
    }

}

public boolean hoverTest(Point2D.Double p) {
    f_prevHover = f_currHover;

    DElement hitElement = getDiagram().getHit(p);

    switch (f_trackingMode) {

        case NONE:
            f_currHover = null;
            break;

        case BOXES:
            if (hitElement instanceof DBox)
                f_currHover = hitElement;
            break;

        case LINES:
            if (hitElement instanceof DLine)
                f_currHover = hitElement;
            break;

        case ALL:
            f_currHover = hitElement;
            break;

    }
    // currHover = getHitElementOrNull(p); // ttt
    // currHover = getHitBoxOrNull(p);

    if (f_currHover != f_prevHover) {
        return true;
    } else
        return false;
}

public DLine.DLineType getCurrentLineType() {
    return _currentLineType;
}

public void setCurrentLineType(DLine.DLineType _currentLineType) {
    this._currentLineType = _currentLineType;
    //TODO
}

public DElement setCurrent(DElement e) {
    resetSelection();
    f_currentElement = e;
    return e;
}

public DElement getCurrent() {
    return f_currentElement;
}

public boolean isCurrent(DElement e) {
    return e == f_currentElement;
}

public boolean isSelected(DElement e) {
    if (!(e instanceof DBox))
        return false;
//        DBox b = (DBox)e;
    return selectedBoxes.contains(e);
}

public void selectBoxes(Rectangle2D.Double r, boolean isShift, boolean isCtrl) {

    if (!isShift)
        selectedBoxes.clear();

    for (DBox b : getDiagram().boxes()) {
        if (r.contains(b.getBounds()))
            selectedBoxes.add(b);
    }
}

public void resetSelection() {
    selectedBoxes.clear();
}

public boolean isManualMode() {
    return getDiagram().isManualMode();
}

public void setManualMode(boolean manual) {
    getDiagram().setManualMode(manual);
}


//----------------------------------------

public DLine findNearestLine(Point2D.Double point) {

    DLine nearestLine = null;
    double nearestDistance = DISTANCE_THRESHOLD;
    double thisDistance;

    for (DLine l : getDiagram().allLines()) {

        thisDistance = HitTest.distanceFromPoint(l.getPoints(), point);

        if (thisDistance < nearestDistance) {
            nearestDistance = thisDistance;
            nearestLine = l;
        }
    }

    return nearestLine;
}

public DBox findNearestBox(Point2D.Double point) {

    DBox nearestBox = null;
    double nearestDistance = DISTANCE_THRESHOLD;
    double thisDistance;

    for (DBox b : getDiagram().allBoxes()) {

        thisDistance = HitTest.distanceFromPoint(b.getBounds(), point);

        if (thisDistance < nearestDistance) {
            nearestDistance = thisDistance;
            nearestBox = b;
        }
    }

    return nearestBox;
}

public enum TrackingMode {NONE, BOXES, LINES, ALL}

private static class GraphIO {
    ArrayList<IntPair> edges;
    private int nodeCount = 0;
    private ArrayList<lv.lumii.layoutengine.Box> nodes;
    private ReaderState readerState;

    public static void saveTo(File f) {

    }

    private void init() {
        nodes = new ArrayList<>();
        edges = new ArrayList<>();


    }

    public void loadFrom(File f) {

    }

    private void ensureNodeCount(int n) {
        if (nodes.size() < n) {
            nodes.ensureCapacity(n);
        }
    }

    private void onEdgeLine(int nStart, int nFinish) {
        edges.add(new IntPair(nStart, nFinish));
        nodeCount = Math.max(nStart, nodeCount);
        nodeCount = Math.max(nFinish, nodeCount);
    }

    private void onNestingLine(int nParent, int nChild) {

    }

    private boolean isInteger(String s) {
        return true;
    }

    private int asInteger(String s) {
        return 0;
    }

    private boolean isIdentifier(String s) {
        return true;
    }

    private enum ReaderState {EDGES, CHILDNODES, NONE}


}

private class Selection {
    private DElement element = null;
    private LinkedHashSet<DBox> elements = new LinkedHashSet<>();
    private DContainer root = null;

    public void clear() {
        elements.clear();
        element = null;
        root = null;
    }

    public boolean isEmpty() {
        return elements.size() == 0;
    }

    public boolean isSingle() {
        return elements.size() == 1;
    }

    public boolean isMulti() {
        return elements.size() > 1;
    }

    public boolean isSelected(DBox b) {
        return false;
    }

    public void add(Collection<DBox> toAdd) {
        for (DBox b : toAdd) {
            elements.add(b);
        }
    }

    public void add(DBox b) {
        if (isEmpty()) {
            elements.add(b);
            root = b.getOwner();
        } else {
            if (root != b.getOwner())
                return;
            elements.add(b);
        }
    }

    public void toggle(DBox b) {
        if (isEmpty())
            elements.add(b);

    }

    public void setCurrent(DBox e) {
    }

//    public Iterable<DBox> selected() {
//        return new Iterable<DBox>() {
//            @Override
//            public Iterator<DBox> iterator() {
//                return new Iterator<DBox>() {
//                    @Override
//                    public boolean hasNext() {
//                        return false;
//                    }
//
//                    @Override
//                    public DBox next() {
//                        return null;
//                    }
//
//                    @Override
//                    public void remove() {
//
//                    }
//                };
//            }
//        };
//    }

    public void onAll(Command cmd) {
        for (DBox b : elements) {
            cmd.execute(b);
        }
    }

    void alignSelection() {
        selection.onAll(new Selection.Command() {
            @Override
            void execute(DBox box) {
                setManualMode(true);

                setManualMode(false);
            }
        });
    }

    public abstract class Command {
        abstract void execute(DBox box);
    }
}

private class DiagramBackground {

    private Image f_diagramBackgroundImage;
    private boolean f_drawDiagramBackground = false;
    private double f_backgroundTransparency = 0.6;

    public DiagramBackground() {
        init();
    }

    public void init() {
//			diagramBackgroundImage = null;
//			f_drawDiagramBackground = false;
    }

    public boolean hasImage() {
        return f_diagramBackgroundImage != null;
    }

    public Image getImage() {
        return f_diagramBackgroundImage;
    }

    public void setImage(BufferedImage img) {
        f_diagramBackgroundImage = img;
        this.setVisible(img != null);
    }

    public boolean isVisible() {
        return f_drawDiagramBackground && hasImage();
    }

    public void setVisible(boolean visible) {
        f_drawDiagramBackground = visible;
    }

    public void draw(Graphics2D g) {
        g.drawImage(f_diagramBackgroundImage, 0, 0, null);
    }

    public boolean setImage(File file) {
        BufferedImage img;

        try {
//				final String DEFAULT_TEST_BACKGROUND = "/images/background.png";
//				URL imageUrl = getClass().getResource(DEFAULT_TEST_BACKGROUND);
//				img = ImageIO.read(imageUrl);

//				img = ImageIO.read(new File(filename));
            img = ImageIO.read(file);

            setBackground(img);
            return true;

        } catch (IOException ignored) {
        }

        return false;

    }

    public void setGrayImage(BufferedImage img) {
        this.setImage(img);
        if (img != null)
            if (img == null) {
                this.init();
            } else {
                f_diagramBackgroundImage = convertToGrayscale(img);
                setVisible(true);
            }
    }

    public void makeGrayscale() {
        if (f_diagramBackgroundImage != null) {
            f_diagramBackgroundImage = convertToGrayscale((BufferedImage) f_diagramBackgroundImage);
        }
    }

    private BufferedImage convertToGrayscale(BufferedImage source) {
        BufferedImageOp op = new ColorConvertOp(
                ColorSpace.getInstance(ColorSpace.CS_GRAY), null);
        return op.filter(source, null);
    }

}


}
