/*
*
* Copyright (c) 2013,2014 Institute of Mathematics and Computer Science, University of Latvia (IMCS UL). 
*
* This file is part of DEEK (diagram editor engine kit). 
*
* You can redistribute it and/or modify it under the terms of the GNU General Public License 
* as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
*
* You should have received a copy of the GNU General Public License along with DEEK If not, see http://www.gnu.org/licenses/.
*
*/

package lv.lumii.deek.ui;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.Toolkit;
//import java.awt.Toolkit;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelEvent;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;

import javax.swing.ActionMap;
import javax.swing.InputMap;
import javax.swing.JComponent;
import javax.swing.Scrollable;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.ToolTipManager;

import lv.lumii.deek.ui.tools.*;
import lv.lumii.test.OpusHost;
import lv.lumii.deek.util.Globals;
import lv.lumii.deek.ui.tools.DElementTool;

@SuppressWarnings("serial")
public class DViewport
        extends JComponent
        implements Scrollable {

public Zoomer zoomer = new Zoomer();
private OpusHost _appHost;
private DDrawing _diagram;
private DController controller;
private DElementTool elementTool;
private Rectangle currentBoxViewportRect;
private boolean enableEvents = true;
private MouseHandler mouseHandler;
private MouseWheelZoomHandler mouseWheelZoomHandler;
// ---------------------------------------------------------------------
private Cursor previousCursor;
private boolean eventsEnabled = true;
private Point2D.Double mouseWorldPoint;

public DViewport(OpusHost host, DDrawing diagram) {
    this._appHost = host;
    this._diagram = diagram;

    initComponents();
}

@Override
public String getToolTipText() {
    return null;
}

private void initComponents() {

    mouseHandler = new MouseHandler();
    addMouseListener(mouseHandler);
    addMouseMotionListener(mouseHandler);

    mouseWheelZoomHandler = new MouseWheelZoomHandler();
    addMouseWheelListener(mouseWheelZoomHandler);
    addMouseListener(mouseWheelZoomHandler);
    addMouseMotionListener(mouseWheelZoomHandler);

    setFocusable(true);

    ToolTipManager.sharedInstance().registerComponent(this);
    ToolTipManager.sharedInstance().setDismissDelay(1000);
//		setToolTipText("<html>this is a <b>tooltip</b><br>this is a tooltip<br>this is a tooltip<br>this is a tooltip\nthis is a tooltip</html>");

    zoomer.resetViewport();
}

public void setController(DController controller) {
    this.controller = controller;
    setInputAndActionMaps();
}

private void setInputAndActionMaps() {
    InputMap inputMap = getInputMap(WHEN_IN_FOCUSED_WINDOW);
    ActionMap actionMap = getActionMap();

    controller.setupInputAndActionMaps(inputMap, actionMap);
}

@Override
public void paintComponent(Graphics g) {

    Graphics2D g2 = (Graphics2D) g.create();
    g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);

    zoomer.setCurrentTransform(g2);

    _diagram.draw(g2);

    drawGrid(g2);

    g2.dispose();
    _appHost.getOverview().updateView(); // TODO - te nav īsti labāka vieta šim
}

@Override
public Dimension getPreferredSize() {
    return zoomer.getScaledWorldDimension();
}

private void drawGrid(Graphics2D g2) {
    final double GRIDSTEP = 20.0;
    SnapGridType gridType = SnapGridType.NONE;

    Rectangle2D.Double bounds = _diagram.getBounds();
    if (bounds != null) {
        g2.setColor(Color.GRAY);
        Rectangle2D erase = bounds.createIntersection(g2.getClipBounds());

        double xStart = Math.floor(erase.getMinX() / GRIDSTEP + 1.0) * GRIDSTEP;
        double yStart = Math.floor(erase.getMinY() / GRIDSTEP + 1.0) * GRIDSTEP;

        switch (gridType) {
            case DOTS:
                for (double xPoint = xStart; xPoint < erase.getMaxX(); xPoint += GRIDSTEP) {
                    for (double yPoint = yStart; yPoint < erase.getMaxY(); yPoint += GRIDSTEP) {
                        g2.drawLine((int) xPoint, (int) yPoint, (int) xPoint, (int) yPoint);
                    }
                }
                break;

            case LINES:
                for (double xPoint = xStart; xPoint < erase.getMaxX(); xPoint += GRIDSTEP) {
                    g2.drawLine((int) xPoint, (int) bounds.getMinY(), (int) xPoint, (int) bounds.getMaxY());
                }
                for (double yPoint = yStart; yPoint < erase.getMaxY(); yPoint += GRIDSTEP) {
                    g2.drawLine((int) bounds.getMinX(), (int) yPoint, (int) bounds.getMaxX(), (int) yPoint);
                }
                break;

        }

    }

}

private void drawOverviewPane(Graphics2D g2) {
    Rectangle2D bounds = _diagram.getBounds();

    if (bounds == null) {
        bounds = new Rectangle2D.Double(0, 0, 0, 0); // work around
    }

    bounds.add(bounds.getMaxX() + Globals.WORLD_PADDING, bounds.getMaxY() + Globals.WORLD_PADDING);
    bounds.add(bounds.getMinX() - Globals.WORLD_PADDING, bounds.getMinY() - Globals.WORLD_PADDING);

    bounds.add(0.0, 0.0);
    bounds.add(getWidth(), getHeight());

    double maxDim = Math.max(bounds.getWidth(), bounds.getHeight());


    int x = 0;
    int width = (int) (Globals.OVERVIEW_PANE_SIZE * bounds.getWidth() / maxDim);
    int height = (int) (Globals.OVERVIEW_PANE_SIZE * bounds.getHeight() / maxDim);
    int y = getHeight() - height;

    g2.setColor(Color.WHITE);
    g2.fillRect(x, y, width, height);
    g2.setColor(Color.GREEN);
    g2.drawRect(x, y, width, height);

    // calculate transform
//		double sf = width / bounds.getWidth();
    double sf = Globals.OVERVIEW_PANE_SIZE / maxDim;
    double tx = -bounds.getMinX() * sf;
    double ty = -bounds.getMinY() * sf + y;

    g2.translate(tx, ty);
    g2.scale(sf, sf);


    // draw _diagram
    _diagram.draw(g2);

    // draw position
    Rectangle2D positionRect = zoomer.getVisibleArea();
    Rectangle overviewRect = new Rectangle(x, y, width, height);


    g2.setColor(new Color(1.0f, 0.0f, 0.0f, 0.2f));
//			g2.fillRect((int)anchorX, (int)anchorY, (int) (getWidth()/scaleFactor), (int) (getHeight()/scaleFactor));
    g2.fill(positionRect);
    g2.setColor(Color.RED);
//			g2.drawRect((int)anchorX, (int)anchorY, (int) (getWidth()/scaleFactor), (int) (getHeight()/scaleFactor));
    g2.draw(positionRect);

//		}

}

private void drawMagnifierPane(Graphics2D g2) {

    Point2D mousePoint = getMousePosition();

    if (mousePoint != null) {
        g2.setColor(Color.BLUE);
        g2.drawOval((int) mousePoint.getX(), (int) mousePoint.getY(), 4, 4);
    }

}

public void startTool() {
    _appHost.setGlassPane(elementTool);

    addMouseListener(elementTool);
    addMouseMotionListener(elementTool);
    addKeyListener(elementTool);

    mouseHandler.setEnabled(false);

    setEnabled(false);

    _appHost.getGlassPane().setVisible(true);
}

public void endTool() {
    removeMouseListener(elementTool);
    removeMouseMotionListener(elementTool);
    removeKeyListener(elementTool);

    enableEvents = true;
    mouseHandler.setEnabled(true);

    setEnabled(true);

    _appHost.getGlassPane().setVisible(false);

//		System.out.println("operation completed");
//		controller.beep();

    elementTool = null;
}

public boolean hoverHitTest(Point2D.Double hitPoint) {
    if (_diagram.hoverTest(hitPoint))
        repaint();

    return true;
}

public boolean hoverHitTest(Point hitPoint) {
    return hoverHitTest(zoomer.translateViewportToWorld(hitPoint));
}

public void startPaneTool(Point2D.Double p) {
    startPaneTool(zoomer.translateWorldToViewport(p));
}

private void startPaneTool(int x, int y) {
    elementTool = new DPaneTool(this, x, y);
    startTool();
}

private void startPaneTool(Point p) {
    elementTool = new DPaneTool(this, p);
    startTool();
}

public void endPaneTool() {
    endTool();
}

public void startWait() {
    previousCursor = getCursor();
    setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
    enableEvents(false);
}

public void endWait() {
    enableEvents(true);
    setCursor(previousCursor);
}

private void enableEvents(boolean enable) {
    eventsEnabled = enable;
}

// ---------------------------------------------------------------------
public void startNewBox() {
    elementTool = new DNewBoxTool(this);
    startTool();
}

public void startNewBox(Point2D.Double p) {
    Point glasspanePoint = zoomer.translateWorldToGlasspane(p);
    startNewBox(glasspanePoint);
}

private void startNewBox(Point p) {
    elementTool = new DNewBoxTool(this, p);
    startTool();
}

public void endNewBox(Rectangle r, Point downPoint, Point upPoint, boolean isShift, boolean isCtrl) {
    endTool();

    if (r != null) {
        Rectangle2D.Double wrBox = zoomer.translateGlasspaneToWorld(r);
        Point2D.Double wpDownPoint = zoomer.translateGlasspaneToWorld(downPoint);
        Point2D.Double wpUpPoint = zoomer.translateGlasspaneToWorld(upPoint);

        controller.boxAdded(wrBox, wpDownPoint, isShift, isCtrl);
    }
}

// ---------------------------------------------------------------------
public void startPaning(Point p) {
    elementTool = new DPaneTool(this, p);
    startTool();
}

public void endPaning() {
    endTool();
}

public void startNewLine(MouseEvent e) {
    elementTool = new DNewLineTool(this, e);
    startTool();
}
// ---------------------------------------------------------------------

public void startNewOrthoLine(MouseEvent e) {
    elementTool = new DNewOrthoLineTool(this, e);
    startTool();
}

public void startNewPolyLine(MouseEvent e) {
    elementTool = new DNewPolyLineTool(this, e);
    startTool();
}

public void endNewLine(ArrayList<Point> line) {
    endTool();

    if (line != null) {
        // add line to _diagram
//			System.out.println(line.toString());

        ArrayList<Point2D.Double> points = new ArrayList<>();
        for (Point p : line) {
            points.add(zoomer.translateViewportToWorld(p));
        }

        controller.lineAdded(points);
    }
}

public void endNewOrthoLine(ArrayList<Point> line) {
    endTool();

    if (line != null) {

        ArrayList<Point2D.Double> points = new ArrayList<>();
        for (Point p : line) {
            points.add(zoomer.translateViewportToWorld(p));
        }

        controller.lineAdded(points);
    }
}

/**
 * *************************
 */
private boolean validateWorldLine(ArrayList<Point2D.Double> points) {

    Point2D.Double p0, p1;
    for (int i = 0; i < points.size() - 1; i++) {
        p0 = points.get(i);
        p1 = points.get(i + 1);
        if ((p0.x != p1.x) && (p0.y != p1.y)) {
            System.out.println(String.format("*** wrong world segment: %d, (%f, %f) (%f, %f)", i, p0.x, p0.y, p1.x, p1.y));
        }
    }

    return true;
}

private boolean validateViewportLine(ArrayList<Point> points) {


    Point p0, p1;
    for (int i = 0; i < points.size() - 1; i++) {
        p0 = points.get(i);
        p1 = points.get(i + 1);
        if ((p0.x != p1.x) && (p0.y != p1.y)) {
            System.out.println(String.format("*** wrong viewport segment: %di, (%d, %d) (%d, %d)", i, p0.x, p0.y, p1.x, p1.y));
        }
    }

    return true;
}

/**
 * *************************
 */

public void startReshapeOrthoLine(ArrayList<Point2D.Double> points, MouseEvent e) {

    validateWorldLine(points);

    ArrayList<Point> line = new ArrayList<>();
    for (Point2D.Double p : points) {
        line.add(zoomer.translateWorldToViewport(p));
    }

    validateViewportLine(line);

    elementTool = new DReshapeOrthoLineTool(this, line, e);
    startTool();
}

public void endReshapeOrthoLine(ArrayList<Point> line) {
    endTool();

    if (line != null) {

        validateViewportLine(line);

        ArrayList<Point2D.Double> points = new ArrayList<>();
        for (Point p : line) {
            points.add(zoomer.translateViewportToWorld(p));
        }

        validateWorldLine(points);

        controller.lineReshaped(points);
    }
}

public void startReshapePolyLine(ArrayList<Point2D.Double> points, MouseEvent e) {

    ArrayList<Point> line = new ArrayList<>();
    for (Point2D.Double p : points) {
        line.add(zoomer.translateWorldToViewport(p));
    }

    elementTool = new DReshapePolyLineTool(this, line, e);
    startTool();
}

public void endReshapePolyLine(ArrayList<Point> line) {

    endTool();

    if (line != null) {

        ArrayList<Point2D.Double> points = new ArrayList<>();
        for (Point p : line) {
            points.add(zoomer.translateViewportToWorld(p));
        }

        controller.lineReshaped(points);
    }
}

public void startMoveResizeBox(Rectangle2D.Double r, Point2D.Double point) {
    startMoveResizeBox(zoomer.translateWorldToViewport(r), zoomer.translateWorldToViewport(point));
}

// ---------------------------------------------------------------------

private void startMoveResizeBox(Rectangle r, Point p) {
    elementTool = new DBoxMoveResizeTool(this, r, p);
    startTool();
}

public void endMoveBox(Rectangle r, Point mousePoint, boolean isShift, boolean isCtrl) {
    // move the selected box
    controller.boxMoved(zoomer.translateViewportToWorld(r), zoomer.translateViewportToWorld(mousePoint), isShift, isCtrl);

    endTool();
}

public void endResizeBox(Rectangle r) {
    controller.boxResized(zoomer.translateViewportToWorld(r));
    endTool();
}

public void endDoingNothing() {
    endTool();
}

// ---------------------------------------------------------------------
public void updateView() {
    repaint();
//		controller.updateInfo();
}

// TODO: interface Scrollable
@Override
public Dimension getPreferredScrollableViewportSize() {
    return getPreferredSize();
}

// ---------------------------------------------------------------------

@Override
public int getScrollableUnitIncrement(Rectangle visibleRect,
                                      int orientation, int direction) {
    return 10;
}

@Override
public int getScrollableBlockIncrement(Rectangle visibleRect,
                                       int orientation, int direction) {
    // TODO: better behaviour for small dimensions
    if (orientation == SwingConstants.VERTICAL) {
        return getHeight() - 50;
    } else {
        return getWidth() - 50;
    }
}

@Override
public boolean getScrollableTracksViewportWidth() {
    return true;
}

@Override
public boolean getScrollableTracksViewportHeight() {
    return true;
}


enum SnapGridType {LINES, DOTS, NONE}

private class RMouseHandler implements MouseListener, MouseMotionListener {

    @Override
    public void mouseDragged(MouseEvent e) {
        // TODO Auto-generated method stub

    }

    @Override
    public void mouseMoved(MouseEvent e) {
        // TODO Auto-generated method stub

    }

    @Override
    public void mouseClicked(MouseEvent e) {
        // TODO Auto-generated method stub

    }

    @Override
    public void mousePressed(MouseEvent e) {
        // TODO Auto-generated method stub

    }

    @Override
    public void mouseReleased(MouseEvent e) {
        // TODO Auto-generated method stub

    }

    @Override
    public void mouseEntered(MouseEvent e) {
        // TODO Auto-generated method stub

    }

    @Override
    public void mouseExited(MouseEvent e) {
        // TODO Auto-generated method stub

    }

}

private class MouseHandler implements MouseListener, MouseMotionListener {
    private boolean enabled = true;

    public void setEnabled(boolean enabled) {
        this.enabled = enabled;
    }

    @Override
    public void mouseClicked(MouseEvent e) {
        if (!enabled) return;

        if (e.getButton() == MouseEvent.BUTTON2) {
            if (e.isControlDown()) {
                if (e.isShiftDown()) {
                    // control+shift: clear _diagram
                    _diagram.clear();
                    // TODO
                }
                // control: reset viewport
                zoomer.resetViewport();
//					updateView();

            } else if (e.isShiftDown()) {
                // shift: cycle viewport mode backwards
                controller.cycleModes(true);

            } else {
                // cycle viewport mode
                controller.cycleModes(false);
            }

        }

    }

    @Override
    public void mousePressed(MouseEvent e) {
        if (!enabled) return;

        if (e.getButton() == MouseEvent.BUTTON1)
            controller.mouseLeftPressedWorld(zoomer.translateViewportToWorld(e.getPoint()), e);
        else if (e.getButton() == MouseEvent.BUTTON3)
            controller.mouseRightPressedWorld(zoomer.translateViewportToWorld(e.getPoint()), e);
    }

    @Override
    public void mouseReleased(MouseEvent e) {
        if (!enabled) return;

        //

    }

    @Override
    public void mouseEntered(MouseEvent e) {
        if (!enabled) return;

        //

    }

    @Override
    public void mouseExited(MouseEvent e) {
        if (!enabled) return;
        //

    }

    @Override
    public void mouseDragged(MouseEvent e) {
        if (!enabled) return;
        //
    }

    @Override
    public void mouseMoved(MouseEvent e) {
        if (!enabled) return;

        controller.mouseMovedWorld(zoomer.translateViewportToWorld(e.getPoint()), e);
        _appHost.getMagnifier().setTarget(zoomer.translateViewportToWorld(e.getPoint())); // test

    }

}

private class MouseWheelZoomHandler extends MouseAdapter {
    private final int AUTO_SCROLL_DISTANCE = 50;
    private Point prevMousePoint, thisMousePoint;

    @Override
    public void mouseWheelMoved(MouseWheelEvent e) {
        int roll = e.getWheelRotation();
        boolean isControl = e.isControlDown();
        boolean isShift = e.isShiftDown();

        mouseWorldPoint = zoomer.translateViewportToWorld(e.getPoint());

        if (isControl) {
            if (roll < 0) {
                zoomer.zoomInViewport(mouseWorldPoint);
            } else {
                zoomer.zoomOutViewport(mouseWorldPoint);
            }
        } else if (isShift) {
            if (roll > 0) {
                zoomer.scrollRight();
//					zoomer.pageRight();
            } else {
                zoomer.scrollLeft();
//					zoomer.pageLeft();
            }
        } else {
            if (roll > 0) {
                zoomer.scrollDown();
            } else {
                zoomer.scrollUp();
            }
        }
        controller.updateInfo();
    }

    @Override
    public void mousePressed(MouseEvent e) {
        prevMousePoint = e.getPoint();
    }

    @Override
    public void mouseDragged(MouseEvent e) {
        thisMousePoint = e.getPoint();

        int dx = thisMousePoint.x - prevMousePoint.x;
        int dy = thisMousePoint.y - prevMousePoint.y;

        if ((thisMousePoint.x > getWidth() - AUTO_SCROLL_DISTANCE) && (dx > 0)) {
            zoomer.stepRight(AUTO_SCROLL_DISTANCE + thisMousePoint.x - getWidth());
//				updateView();
        } else if ((thisMousePoint.y > getHeight() - AUTO_SCROLL_DISTANCE) && (dy > 0)) {
            zoomer.stepDown(AUTO_SCROLL_DISTANCE + thisMousePoint.y - getHeight());
//				updateView();
        } else if ((thisMousePoint.x < AUTO_SCROLL_DISTANCE) && (dx < 0)) {
            zoomer.stepLeft(AUTO_SCROLL_DISTANCE - thisMousePoint.x);
//				updateView();
        } else if ((thisMousePoint.y < AUTO_SCROLL_DISTANCE) && (dy < 0)) {
            zoomer.stepUp(AUTO_SCROLL_DISTANCE - thisMousePoint.y);
//				updateView();
        }
        prevMousePoint = thisMousePoint;
    }
}

public class Zoomer {
    private static final int SCROLL_AMOUNT = 100;
    private static final int SMALL_SCROLL_AMOUNT = 25;
    private static final double ZOOM_AMOUNT = 1.1;
    private static final double ZOOM_IN_AMOUNT = ZOOM_AMOUNT;
    private static final double ZOOM_OUT_AMOUNT = 1.0 / ZOOM_IN_AMOUNT;
    private double anchorX;
    private double anchorY;
    private double scaleFactor; // 1.0 = 100%
    private double defaultScaleFactor = 0.0;


    public Zoomer() {

    }

    public String getViewportInfo() {
        return String.format("VP: (0, 0, %d, %d), zoom: %5.2f, A: (%4.1f, %4.1f) ",
                getWidth(), getHeight(),
                scaleFactor,
                anchorX, anchorY);
    }

    public void scrollDown() {
        paneViewport(0.0, SCROLL_AMOUNT / scaleFactor);
    }

    public void scrollUp() {
        paneViewport(0.0, -SCROLL_AMOUNT / scaleFactor);
    }

    public void scrollRight() {
        paneViewport(SCROLL_AMOUNT / scaleFactor, 0.0);
    }

    public void scrollLeft() {
        paneViewport(-SCROLL_AMOUNT / scaleFactor, 0.0);
    }

    public void pageDown() {
        paneViewport(0.0, getHeight() / scaleFactor);
    }

    public void pageUp() {
        paneViewport(0.0, -getHeight() / scaleFactor);
    }

    public void pageRight() {
        paneViewport(getWidth() / scaleFactor, 0.0);
    }

    public void pageLeft() {
        paneViewport(-getWidth() / scaleFactor, 0.0);
    }

    public void stepDown() {
        stepDown(SMALL_SCROLL_AMOUNT);
//			paneViewport(0.0,  SMALL_SCROLL_AMOUNT);
    }

    public void stepUp() {
        stepUp(SMALL_SCROLL_AMOUNT);
//			paneViewport(0.0, -SMALL_SCROLL_AMOUNT);
    }

    public void stepRight() {
        stepRight(SMALL_SCROLL_AMOUNT);
//			paneViewport( SMALL_SCROLL_AMOUNT, 0.0);
    }

    public void stepLeft() {
        stepLeft(SMALL_SCROLL_AMOUNT);
//			paneViewport(-SMALL_SCROLL_AMOUNT, 0.0);
    }

    public void stepDown(int stepSize) {
        paneViewport(0.0, stepSize);
    }

    public void stepUp(int stepSize) {
        paneViewport(0.0, -stepSize);
    }

    public void stepRight(int stepSize) {
        paneViewport(stepSize, 0.0);
    }

    public void stepLeft(int stepSize) {
        paneViewport(-stepSize, 0.0);
    }

    private void paneViewport(double paneByX, double paneByY) {
        if ((Math.abs(paneByX) * scaleFactor < getWidth()) && (Math.abs(paneByY) * scaleFactor < getHeight())) {
            // TODO: this can be optimized shifting the rendered part (graphics.copyArea) and repainting only the uncovered part
            Rectangle oldArea = new Rectangle(0, 0, getWidth(), getHeight());
            Rectangle newArea = new Rectangle(oldArea);
            newArea.x = (int) (paneByX * scaleFactor);
            newArea.y = (int) (paneByY * scaleFactor);


            repaint(); // FIXME

            anchorX += paneByX;
            anchorY += paneByY;

            controller.updateInfo();

        } else {

            anchorX += paneByX;
            anchorY += paneByY;
            updateView();
        }

    }

    private void zoomViewport(double zoomBy) {
        scaleFactor *= zoomBy;
        updateView();
    }

    private void zoomViewport(double zoomBy, Point2D.Double zoomAnchor) {
        scaleFactor *= zoomBy;

        anchorX = (zoomAnchor.x * (zoomBy - 1) + anchorX) / zoomBy;
        anchorY = (zoomAnchor.y * (zoomBy - 1) + anchorY) / zoomBy;

        updateView();
    }

    public void zoomInViewport() {
        zoomViewport(ZOOM_IN_AMOUNT);
    }

    public void zoomOutViewport() {
        zoomViewport(ZOOM_OUT_AMOUNT);
    }

    private void zoomInViewport(Point2D.Double zoomAnchor) {
        zoomViewport(ZOOM_IN_AMOUNT, zoomAnchor);
    }

    private void zoomOutViewport(Point2D.Double zoomAnchor) {
        zoomViewport(ZOOM_OUT_AMOUNT, zoomAnchor);
    }

    private double getDefaultScaleFactor() {
        // calculate the recommended scale factor for given screen DPI at 100% zoom
        if (defaultScaleFactor == 0.0) {

            int resolution = Toolkit.getDefaultToolkit().getScreenResolution();

            defaultScaleFactor = resolution / 96.0;
        }
        return defaultScaleFactor;
    }

    public void resetViewport() {
        anchorX = 0.0;
        anchorY = 0.0;

        scaleFactor = getDefaultScaleFactor();

        updateView();
    }

    public void zoomToViewport() {
        Rectangle2D bounds = _diagram.getBounds();
        if (bounds == null) {
            resetViewport();
            return;
        }

        bounds.add(bounds.getMaxX() + Globals.WORLD_PADDING, bounds.getMaxY() + Globals.WORLD_PADDING);
        bounds.add(bounds.getMinX() - Globals.WORLD_PADDING, bounds.getMinY() - Globals.WORLD_PADDING);

        double scaleX = getWidth() / bounds.getWidth();
        double scaleY = getHeight() / bounds.getHeight();

        anchorX = bounds.getX();
        anchorY = bounds.getY();

        scaleFactor = Math.min(Math.min(scaleX, scaleY), getDefaultScaleFactor());
    }

    public void setCurrentTransform(Graphics2D g2) {
        g2.scale(scaleFactor, scaleFactor);
        g2.translate(-anchorX, -anchorY);

    }

    public Point translateWorldToViewport(double x, double y) {
        return new Point((int) ((x - anchorX) * scaleFactor), (int) ((y - anchorY) * scaleFactor));
    }

    public Point2D.Double translateViewportToWorld(int x, int y) {
        return new Point2D.Double(x / scaleFactor + anchorX, y / scaleFactor + anchorY);
    }

    public Point translateWorldToViewport(Point2D.Double p) {
        return translateWorldToViewport(p.x, p.y);
    }

    public Point2D.Double translateViewportToWorld(Point p) {
        return translateViewportToWorld(p.x, p.y);
    }

    public Rectangle2D.Double translateViewportToWorld(Rectangle r) {
        Point2D.Double c = translateViewportToWorld(r.x, r.y);
        return new Rectangle2D.Double(c.x, c.y, r.width / scaleFactor, r.height / scaleFactor);
    }

    public Point2D.Double translateGlasspaneToWorld(Point p) {
        return translateGlasspaneToWorld(p.x, p.y);
    }

    public Point2D.Double translateGlasspaneToWorld(int x, int y) {
        Point p = SwingUtilities.convertPoint(_appHost.getGlassPane(), x, y, _appHost.getContentPane());
//			System.out.printf("from G(%d. %d) to V(%d, %d)\n", x, y, p.x, p.y);
        Point2D.Double p2 = new Point2D.Double(p.x / scaleFactor + anchorX, p.y / scaleFactor + anchorY);
        return p2;
    }

    public Rectangle2D.Double translateGlasspaneToWorld(Rectangle r) {
        Point2D.Double c = translateGlasspaneToWorld(r.x, r.y);
        return new Rectangle2D.Double(c.x, c.y, r.width / scaleFactor, r.height / scaleFactor);
    }

    public Rectangle translateWorldToViewport(Rectangle2D.Double r) {
        Point c = translateWorldToViewport(r.x, r.y);
        return new Rectangle(c.x, c.y, (int) (r.width * scaleFactor), (int) (r.height * scaleFactor));
    }

    public Point translateWorldToGlasspane(Point2D.Double p) {
        Point p2 = translateWorldToViewport(p);
        Point p3 = SwingUtilities.convertPoint(_appHost.getContentPane(), p2.x, p2.y, _appHost.getGlassPane());
        return p3;
    }

    public Rectangle2D.Double getVisibleArea() {
        return new Rectangle2D.Double(anchorX, anchorY, getWidth() / scaleFactor, getHeight() / scaleFactor);
    }

    public Point2D.Double getVisibleAreaCenter() {
        return new Point2D.Double(anchorX + 0.5 * getWidth() / scaleFactor, anchorY + 0.5 * getHeight() / scaleFactor);
    }

    public Dimension getScaledWorldDimension() {
        Rectangle2D.Double r = _diagram.getBounds();
        return new Dimension((int) (r.width * scaleFactor), (int) (r.height * scaleFactor));
    }

    public void doPaning(int deltaX, int deltaY) {
        paneViewport(-deltaX / scaleFactor, -deltaY / scaleFactor);
    }

    public void doPaning(double deltaX, double deltaY) {
        paneViewport(deltaX, deltaY);
    }

    public void doPaningByRatio(double ratioX, double ratioY) {
        Rectangle2D.Double r = _diagram.getBounds();

        paneViewport(r.width * ratioX, r.height * ratioY);
    }


}
}
