/*
*
* Copyright (c) 2013,2014 Institute of Mathematics and Computer Science, University of Latvia (IMCS UL). 
*
* This file is part of DEEK (diagram editor engine kit). 
*
* You can redistribute it and/or modify it under the terms of the GNU General Public License 
* as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
*
* You should have received a copy of the GNU General Public License along with DEEK If not, see http://www.gnu.org/licenses/.
*
*/

package lv.lumii.deek.ui;

import java.awt.geom.Point2D;
import java.util.ArrayList;
import java.util.List;

import lv.lumii.layoutengine.Box;
import lv.lumii.layoutengine.Container;
import lv.lumii.layoutengine.Element;
import lv.lumii.layoutengine.Line;

/**
 * Helper methods for debug output
 */
abstract class DebugHelper {


private static void dumpThisBox(Box b, StringBuilder sb, String nl) {
    sb.append(nl);
//		sb.append(String.format("box (L R T B mw mh s): %.42f %.42f %.42f %.42f %.42f %.42f %.42f", 
    sb.append(String.format("box (L R T B mw mh s): %a %a %a %a %a %a %a",
            b.getLeft(), b.getRight(), b.getTop(), b.getBottom(), b.getMinWidth(), b.getMinHeight(), b.getSpacing()));
}

private static void dumpThisLine(Line l, StringBuilder sb, String nl) {
    sb.append(nl);
    sb.append("line ([points] s): [ ");
    for (Point2D.Double p : l.getPoints()) {
//			sb.append(String.format("[%.42f, %.42f] ", p.x, p.y));
        sb.append(String.format("[%a, %a] ", p.x, p.y));
//			sb.append(String.format("[%s, %s] ", Double.toString(p.x), Double.toString(p.y)));
    }
//		sb.append(String.format("] %f.42", l.getSpacing()));
    sb.append(String.format("] %a", l.getSpacing()));
//		sb.append(String.format("] %s", Double.toString(l.getSpacing())));
}

private static void dumpThisContainer(Container c, StringBuilder sb, String nl) {

    List<Box> boxes = new ArrayList<>(c.getBoxes());
    if (boxes.size() > 0) {
        sb.append(nl);
        sb.append(nl);
        sb.append("BOXES: [");
        for (Box b : boxes) {
            dumpThisBox(b, sb, nl + "  ");
            dumpThisContainer(b, sb, nl + "   ");
        }

        sb.append(nl);
        sb.append("]");
        sb.append(nl);
    }

    List<Container> containers = new ArrayList<>(c.getPureContainers());
    if (containers.size() > 0) {
        sb.append("*** CONTAINERS ***");
        sb.append(nl);
        for (Container cc : containers) {
            dumpThisContainer(cc, sb, nl + "   ");
        }
    }

    List<Line> lines = new ArrayList<>(c.getLines());
    if (lines.size() > 0) {
        sb.append(nl);
        sb.append("LINES: [");
        for (Line l : lines) {
            dumpThisLine(l, sb, nl + "  ");
        }
        sb.append(nl);

        sb.append("]");
        sb.append(nl);
    }

}

//	public static String dumpDrawing(lv.lumii.layoutengine.Diagram layout) {
public static String dumpDrawing(DDrawing drawing) {
    StringBuilder dump = new StringBuilder(2048);

    String nl = System.getProperty("line.separator");
    dump.append("*** LAYOUT ***");

    dumpThisContainer(drawing.getDiagram().getLayoutElement(), dump, nl + "  ");

    return dump.toString();
}

public static String dumpDrawing2(lv.lumii.layoutengine.Diagram layout) {
    StringBuilder dump = new StringBuilder(2048);

    for (Element e : layout.getChildren()) {
        dump.append(e.toString());
    }

    return dump.toString();
}
}
