/*
*
* Copyright (c) 2013,2014 Institute of Mathematics and Computer Science, University of Latvia (IMCS UL). 
*
* This file is part of DEEK (diagram editor engine kit). 
*
* You can redistribute it and/or modify it under the terms of the GNU General Public License 
* as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
*
* You should have received a copy of the GNU General Public License along with DEEK If not, see http://www.gnu.org/licenses/.
*
*/

package lv.lumii.deek.ui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;

import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JSlider;

import lv.lumii.test.OpusFrame;

public class OpusMagnifier extends JDialog {

private final int MAGNIFIER_WIDTH = 200;
private final int MAGNIFIER_HEIGHT = MAGNIFIER_WIDTH;
//	private DrawingInterface drawing;
private DDrawing drawing;
private OpusFrame frame;
private MagnifierPanel magnifierPanel;
private ControlPanel controlPanel;
private Point2D.Double targetPoint = new Point2D.Double(200, 200);

//	public OpusMagnifier(DrawingInterface drawing, OpusFrame mainFrame) {
public OpusMagnifier(DDrawing drawing, OpusFrame mainFrame) {

    this(mainFrame);
    this.drawing = drawing;
    this.frame = mainFrame;

    initComponents();
}

public OpusMagnifier(OpusFrame mainFrame) {
    super(mainFrame);
}

public void initComponents() {
//		controlPanel = new ControlPanel();
//		add(controlPanel, BorderLayout.SOUTH);

    magnifierPanel = new MagnifierPanel();
    add(magnifierPanel, BorderLayout.CENTER);

    setSize(MAGNIFIER_WIDTH, MAGNIFIER_HEIGHT);
    setLocation(0, 300);

    setDefaultCloseOperation(JFrame.HIDE_ON_CLOSE);
    setTitle("Magnifier");
    setAutoRequestFocus(false);
    setType(Type.UTILITY);

    addComponentListener(new ComponentListener() {

        @Override
        public void componentResized(ComponentEvent e) {
            updateView();
        }

        @Override
        public void componentMoved(ComponentEvent e) {
        }

        @Override
        public void componentShown(ComponentEvent e) {
            updateView();
        }

        @Override
        public void componentHidden(ComponentEvent e) {
        }

    });


}

public void updateView() {
    repaint();
}

public void setTarget(Point2D.Double targetPoint) {
    this.targetPoint = targetPoint;
    updateView();
}

private class MagnifierPanel extends JComponent {

    private AffineTransform at = new AffineTransform();

    public void recalculate() {
        double scaleFactor = 3.0;

        double anchorX = targetPoint.x - getWidth() / scaleFactor / 2;
        double anchorY = targetPoint.y - getHeight() / scaleFactor / 2;

        at.setToIdentity();
        at.scale(scaleFactor, scaleFactor);
        at.translate(-anchorX, -anchorY);
    }

    private void setQuality(Graphics2D g2) {

        // TODO - atrast pareizo kvalitati

        RenderingHints rh = new RenderingHints(
                RenderingHints.KEY_ANTIALIASING,
                RenderingHints.VALUE_ANTIALIAS_ON);

        rh.put(
                RenderingHints.KEY_STROKE_CONTROL,
                RenderingHints.VALUE_STROKE_NORMALIZE);

        g2.setRenderingHints(rh);
    }

    @Override
    public void paintComponent(Graphics g) {
        g.drawLine(0, 0, getWidth(), getHeight());
        g.drawLine(0, getHeight(), getWidth(), 0);

        Graphics2D g2 = (Graphics2D) g.create();

        setTransformation(g2);
        setQuality(g2);
        drawing.draw(g2);

        g2.dispose();
    }

    public void setTransformation(Graphics2D g) {
        recalculate();
        g.setTransform(at);
    }

}

private class ControlPanel extends JPanel {
    public ControlPanel() {
        initComponents();
    }

    public void initComponents() {
        add(new JCheckBox(), FlowLayout.LEFT);
        add(new JSlider(JSlider.HORIZONTAL));
        pack();
    }

    @Override
    public Dimension getMinimumSize() {
        return new Dimension(100, 40);
    }

    @Override
    public Dimension getPreferredSize() {
        return getMinimumSize();
    }
}
}
