/*
*
* Copyright (c) 2013,2014 Institute of Mathematics and Computer Science, University of Latvia (IMCS UL). 
*
* This file is part of DEEK (diagram editor engine kit). 
*
* You can redistribute it and/or modify it under the terms of the GNU General Public License 
* as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
*
* You should have received a copy of the GNU General Public License along with DEEK If not, see http://www.gnu.org/licenses/.
*
*/

package lv.lumii.deek.ui.tools;

import java.awt.Cursor;
import java.awt.Point;
import java.awt.Rectangle;

import lv.lumii.deek.util.Globals;
import lv.lumii.deek.ui.DViewport;

@SuppressWarnings("serial")
public class DBoxMoveResizeTool extends DBoxTool {

private Rectangle oldBox;
private int x1, y1, x2, y2;
private ResizeType resizeType;


public DBoxMoveResizeTool(DViewport owner, final Rectangle box, final Point anchorPoint) {
    super(owner);

    this.tempBox = (Rectangle) box.clone();
    this.oldBox = (Rectangle) box.clone();

    x1 = box.x;
    y1 = box.y;
    x2 = box.x + box.width;
    y2 = box.y + box.height;


    mouseCurrPoint.setLocation(anchorPoint);
    mousePrevPoint.setLocation(anchorPoint);

    resizeType = getResizeType(box, anchorPoint.x, anchorPoint.y);

    if (resizeType == ResizeType.NONE) {
        abort();
    } else {
        setCursor(Cursor.getPredefinedCursor(getCursorType(resizeType)));
        repaint(box);
    }
}

public void onDrag() {

    //		Rectangle r2 = (Rectangle) tempBox.clone(); // FIXME

    int minX = tempBox.x;
    int minY = tempBox.y;
    int maxX = minX + tempBox.width;
    int maxY = minY + tempBox.height;


    if (resizeType == ResizeType.MOVE) {
        owner.hoverHitTest(mouseCurrPoint);
    }

    int dw, dh, dx, dy;

    switch (resizeType) {
        case MOVE:
            tempBox.x += mouseCurrPoint.x - mousePrevPoint.x;
            tempBox.y += mouseCurrPoint.y - mousePrevPoint.y;
            break;

        case W:
            dx = mouseCurrPoint.x;
            if (x2 - dx < globals.getBoxResizeWidthMin()) dx = x2 - globals.getBoxResizeWidthMin();
            tempBox.x = dx;
            tempBox.width = x2 - dx;

            break;

        case E:
            dw = mouseCurrPoint.x - x1;
            if (dw < globals.getBoxResizeWidthMin()) dw = globals.getBoxResizeWidthMin();
            tempBox.width = dw;

            break;

        case N:
            dy = mouseCurrPoint.y;
            if (y2 - dy < globals.getBoxResizeHeightMin()) dy = y2 - globals.getBoxResizeHeightMin();
            tempBox.y = dy;
            tempBox.height = y2 - dy;

            break;

        case S:
            dh = mouseCurrPoint.y - y1;
            if (dh < globals.getBoxResizeHeightMin()) dh = globals.getBoxResizeHeightMin();
            tempBox.height = dh;

            break;

        case NW:
            dx = mouseCurrPoint.x;
            if (x2 - dx < globals.getBoxResizeWidthMin()) dx = x2 - globals.getBoxResizeWidthMin();
            tempBox.x = dx;
            tempBox.width = x2 - dx;

            dy = mouseCurrPoint.y;
            if (y2 - dy < globals.getBoxResizeHeightMin()) dy = y2 - globals.getBoxResizeHeightMin();
            tempBox.y = dy;
            tempBox.height = y2 - dy;

            break;

        case NE:
            dw = mouseCurrPoint.x - x1;
            if (dw < globals.getBoxResizeWidthMin()) dw = globals.getBoxResizeWidthMin();
            tempBox.width = dw;

            dy = mouseCurrPoint.y;
            if (y2 - dy < globals.getBoxResizeHeightMin()) dy = y2 - globals.getBoxResizeHeightMin();
            tempBox.y = dy;
            tempBox.height = y2 - dy;

            break;

        case SW:
            dx = mouseCurrPoint.x;
            if (x2 - dx < globals.getBoxResizeWidthMin()) dx = x2 - globals.getBoxResizeWidthMin();
            tempBox.x = dx;
            tempBox.width = x2 - dx;

            dh = mouseCurrPoint.y - y1;
            if (dh < globals.getBoxResizeHeightMin()) dh = globals.getBoxResizeHeightMin();
            tempBox.height = dh;

            break;

        case SE:
            dw = mouseCurrPoint.x - x1;
            if (dw < globals.getBoxResizeWidthMin()) dw = globals.getBoxResizeWidthMin();
            tempBox.width = dw;

            dh = mouseCurrPoint.y - y1;
            if (dh < globals.getBoxResizeHeightMin()) dh = globals.getBoxResizeHeightMin();
            tempBox.height = dh;

            break;

        case NONE:
            break;

    }


    minX = Math.min(minX, tempBox.x);
    minY = Math.min(minY, tempBox.y);
    maxX = Math.max(maxX, tempBox.x + tempBox.width);
    maxY = Math.max(maxY, tempBox.y + tempBox.height);

    repaint(minX - 3, minY - 3, maxX - minX + 6, maxY - minY + 6);


}

// --------------------------------------------------------

//	public void mouseReleased(MouseEvent e){
public void onPrimaryUp() {
    setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
    finish();
}

private ResizeType getResizeType(Rectangle r, int x, int y) {
    ScanResult hScan;
    ScanResult vScan;

    if (r.contains(x, y)) {
        hScan = ScanResult.INSIDE;
        vScan = ScanResult.INSIDE;
    } else {
        hScan = ScanResult.OUTSIDE;
        vScan = ScanResult.OUTSIDE;
    }

    if ((r.x - Globals.HIT_EPSILON <= x) && (r.x + Globals.HIT_EPSILON >= x)) hScan = ScanResult.FIRSTBORDER;
    if ((r.y - Globals.HIT_EPSILON <= y) && (r.y + Globals.HIT_EPSILON >= y)) vScan = ScanResult.FIRSTBORDER;

    if ((r.x + Globals.HIT_EPSILON <= x) && (r.x + r.width - Globals.HIT_EPSILON >= x)) hScan = ScanResult.INSIDE;
    if ((r.y + Globals.HIT_EPSILON <= y) && (r.y + r.height - Globals.HIT_EPSILON >= y)) vScan = ScanResult.INSIDE;

    if ((r.x + r.width - Globals.HIT_EPSILON <= x) && (r.x + r.width + Globals.HIT_EPSILON >= x))
        hScan = ScanResult.SECONDBORDER;
    if ((r.y + r.height - Globals.HIT_EPSILON <= y) && (r.y + r.height + Globals.HIT_EPSILON >= y))
        vScan = ScanResult.SECONDBORDER;

    switch (hScan) {
        case OUTSIDE:
            return ResizeType.NONE;
        case FIRSTBORDER:
            switch (vScan) {
                case OUTSIDE:
                    return ResizeType.NONE;
                case FIRSTBORDER:
                    return ResizeType.NW;
                case INSIDE:
                    return ResizeType.W;
                case SECONDBORDER:
                    return ResizeType.SW;
            }
        case INSIDE:
            switch (vScan) {
                case OUTSIDE:
                    return ResizeType.NONE;
                case FIRSTBORDER:
                    return ResizeType.N;
                case INSIDE:
                    return ResizeType.MOVE;
                case SECONDBORDER:
                    return ResizeType.S;
            }
        case SECONDBORDER:
            switch (vScan) {
                case OUTSIDE:
                    return ResizeType.NONE;
                case FIRSTBORDER:
                    return ResizeType.NE;
                case INSIDE:
                    return ResizeType.E;
                case SECONDBORDER:
                    return ResizeType.SE;
            }

    }
    return ResizeType.NONE; // unreachable
}

private int getCursorType(ResizeType rt) {
    switch (rt) {
        case MOVE:
            return Cursor.MOVE_CURSOR;
        case E:
            return Cursor.E_RESIZE_CURSOR;
        case W:
            return Cursor.W_RESIZE_CURSOR;
        case N:
            return Cursor.N_RESIZE_CURSOR;
        case S:
            return Cursor.S_RESIZE_CURSOR;
        case NW:
            return Cursor.NW_RESIZE_CURSOR;
        case SW:
            return Cursor.SW_RESIZE_CURSOR;
        case NE:
            return Cursor.NE_RESIZE_CURSOR;
        case SE:
            return Cursor.SE_RESIZE_CURSOR;
        default:
            return Cursor.DEFAULT_CURSOR;
    }
}

protected void finish() {
    if (tempBox.equals(oldBox)) {
        abort();
    }
//		else if ((resizeType == ResizeType.MOVE) && (!oldBox.contains(mouseCurrPoint))) {
    else if (resizeType == ResizeType.MOVE) {
        owner.endMoveBox(tempBox, mouseCurrPoint, shiftPressed, ctrlPressed);
    } else {
        owner.endResizeBox(tempBox);
    }
}

private enum ResizeType {N, NE, E, SE, S, SW, W, NW, MOVE, NONE}

//	private static final int HITEPSILON = 10;
private enum ScanResult {
    OUTSIDE, FIRSTBORDER, INSIDE, SECONDBORDER
}
}
