/*
*
* Copyright (c) 2013,2014 Institute of Mathematics and Computer Science, University of Latvia (IMCS UL). 
*
* This file is part of DEEK (diagram editor engine kit). 
*
* You can redistribute it and/or modify it under the terms of the GNU General Public License 
* as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
*
* You should have received a copy of the GNU General Public License along with DEEK If not, see http://www.gnu.org/licenses/.
*
*/

package lv.lumii.deek.ui.tools;

import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.Rectangle;

import lv.lumii.deek.util.Globals;
import lv.lumii.deek.ui.DViewport;

@SuppressWarnings("serial")
public abstract class DBoxTool extends DElementTool {

protected Rectangle tempBox;

public DBoxTool(DViewport owner) {
    this.owner = owner;
}

@Override
protected void drawElement(Graphics2D g) {

    if (tempBox != null) {

        g.setPaint(Globals.TOOL_FILL_PAINT);
        g.fill(tempBox);

        g.setStroke(Globals.TOOL_BORDER_STROKE);
        g.setColor(Globals.TOOL_BORDER_COLOR);
        g.draw(tempBox);

        String str = String.format("(%d,%d) (%d %d)", tempBox.x, tempBox.y, tempBox.width, tempBox.height);

        FontMetrics fm = g.getFontMetrics(Globals.infoFont);
        java.awt.geom.Rectangle2D rect = fm.getStringBounds(str, g);

        int textHeight = (int) (rect.getHeight());
        int textWidth = (int) (rect.getWidth());

        if ((tempBox.width < textWidth) || (tempBox.height < textHeight)) return;

        // Center text horizontally and vertically
        int x = tempBox.x + (tempBox.width - textWidth) / 2;
        int y = tempBox.y + (tempBox.height - textHeight) / 2 + fm.getAscent();

        g.setFont(Globals.infoFont);

        g.drawString(str, x, y);  // Draw the string.

    }
}

// return toolResult to the caller
protected void finish() {

    owner.endNewBox(tempBox, mouseDownPoint, mouseUpPoint, shiftPressed, ctrlPressed);
}

@Override
protected void abort() {

    owner.endDoingNothing();
}

}
