/*
*
* Copyright (c) 2013,2014 Institute of Mathematics and Computer Science, University of Latvia (IMCS UL). 
*
* This file is part of DEEK (diagram editor engine kit). 
*
* You can redistribute it and/or modify it under the terms of the GNU General Public License 
* as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
*
* You should have received a copy of the GNU General Public License along with DEEK If not, see http://www.gnu.org/licenses/.
*
*/

package lv.lumii.deek.ui.tools;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

import javax.swing.JComponent;

import lv.lumii.deek.ui.DViewport;
import lv.lumii.deek.util.Globals;

@SuppressWarnings("serial")
public abstract class DElementTool
        extends JComponent
        implements MouseListener, MouseMotionListener, KeyListener {

protected Point mouseDownPoint = new Point();
protected Point mousePrevPoint = new Point();
protected Point mouseCurrPoint = new Point();
protected Point mouseUpPoint = new Point();
protected Point mouseExtraPoint = new Point();
protected Dimension dragVector = new Dimension();
protected boolean mouseHasBeenDragged;
protected boolean shiftPressed;
protected boolean ctrlPressed;
protected ToolState toolState;
protected DViewport owner;
protected Globals globals = Globals.getInstance();
protected ToolEvent onFinish;


public DElementTool() {
    initTool();
}

public DElementTool(DViewport owner) {
    this();
    this.owner = owner;
}

private void initTool() {


    addComponentListener(new ComponentAdapter() {
        public void componentShown(ComponentEvent evt) {
            requestFocusInWindow();
        }
    });
    setFocusTraversalKeysEnabled(false);

    toolState = ToolState.BEFORE;

}

// finish() should return tool result to the caller
protected abstract void finish();

protected abstract void abort();

protected abstract void drawElement(Graphics2D g);

protected void paintComponent(Graphics g) {
    Graphics2D g2 = (Graphics2D) g.create();
    drawElement(g2);
    g2.dispose();
}

@Override
public void keyTyped(KeyEvent e) {
    if (e.getKeyChar() == KeyEvent.VK_ESCAPE) {
        // abort the insertion
        // FIXME
        Toolkit.getDefaultToolkit().beep();
        abort();
    }
}

public void onPrimaryDown() {/*System.out.println("*unprocessed down*");*/}

public void onPrimaryUp() {/*System.out.println("*unprocessed up*");*/}

public void onMove() {/*System.out.println("*unprocessed move*");*/}

public void onDrag() {/*System.out.println("*unprocessed drag*");*/}

public void onSecondaryDown() {/*System.out.println("*unprocessed right down*");*/}

public void onClose() {
}

public void onDestroy() {
}

protected void setToolPosition(int x, int y) {
    mouseCurrPoint.x = mouseDownPoint.x = mousePrevPoint.x = x;
    mouseCurrPoint.y = mouseDownPoint.y = mousePrevPoint.y = y;
}

protected void setToolPosition(Point p) {
    setToolPosition(p.x, p.y);
}

@Override
public void mousePressed(MouseEvent e) {

    if (e.getButton() == MouseEvent.BUTTON1) {

        setToolPosition(e.getPoint());

        shiftPressed = e.isShiftDown();
        ctrlPressed = e.isControlDown();

        mouseHasBeenDragged = false;

        onPrimaryDown();

    } else if (e.getButton() == MouseEvent.BUTTON3) {
        mouseExtraPoint.setLocation(e.getPoint());
        onSecondaryDown(); // TODO
    }

}

@Override
public void mouseDragged(MouseEvent e) {


    dragVector.setSize(mouseCurrPoint.x - mousePrevPoint.x,
            mouseCurrPoint.y - mousePrevPoint.y);

    mouseHasBeenDragged = true;

    mousePrevPoint.setLocation(mouseCurrPoint);
    mouseCurrPoint.setLocation(e.getPoint());

    shiftPressed = e.isShiftDown();
    ctrlPressed = e.isControlDown();

    onDrag();
//			System.out.println("on drag dispatched");
//		}
}

@Override
public void mouseReleased(MouseEvent e) {
    if (e.getButton() == MouseEvent.BUTTON1) {

        mouseUpPoint.setLocation(e.getPoint());

        shiftPressed = e.isShiftDown();
        ctrlPressed = e.isControlDown();

        onPrimaryUp();
    }
}

@Override
public void mouseMoved(MouseEvent e) {
    onMove();
}

@Override
public void mouseClicked(MouseEvent e) {
}

@Override
public void keyPressed(KeyEvent e) {
}

@Override
public void keyReleased(KeyEvent e) {
}

@Override
public void mouseEntered(MouseEvent e) {
}

@Override
public void mouseExited(MouseEvent e) {
}

protected void repaint3(Point p1, Point p2, Point p3) {
    int minX = Math.min(p1.x, Math.min(p2.x, p3.x));
    int minY = Math.min(p1.y, Math.min(p2.y, p3.y));
    int maxX = Math.max(p1.x, Math.max(p2.x, p3.x));
    int maxY = Math.max(p1.y, Math.max(p2.y, p3.y));

//		repaint(minX - 2, minY - 2, maxX - minX + 4, maxY - minY + 4);
    repaint();
}

protected void repaint2(Point p1, Point p2) {
    int minX = Math.min(p1.x, p2.x);
    int minY = Math.min(p1.y, p2.y);
    int maxX = Math.max(p1.x, p2.x);
    int maxY = Math.max(p1.y, p2.y);

    repaint(minX - 2, minY - 2, maxX - minX + 4, maxY - minY + 4);

}

protected enum ToolState {BEFORE, DOING, FINISHING}

public interface ToolCallback {
    public void onEvent(ToolEvent e);
}

public class ToolEvent {
    private boolean shiftState;
    private boolean ctrlState;

}

}
