/*
*
* Copyright (c) 2013,2014 Institute of Mathematics and Computer Science, University of Latvia (IMCS UL). 
*
* This file is part of DEEK (diagram editor engine kit). 
*
* You can redistribute it and/or modify it under the terms of the GNU General Public License 
* as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
*
* You should have received a copy of the GNU General Public License along with DEEK If not, see http://www.gnu.org/licenses/.
*
*/

package lv.lumii.deek.ui.tools;

import java.awt.Point;
import java.awt.Rectangle;

import lv.lumii.deek.ui.DViewport;

@SuppressWarnings("serial")
public class DNewBoxTool extends DBoxTool {

private boolean allowReverseRectangle;

public DNewBoxTool(DViewport owner) {
    super(owner);

    allowReverseRectangle = true;
}

public DNewBoxTool(DViewport owner, Point startPoint) {
    this(owner);

    setToolPosition(startPoint);

    toolState = ToolState.DOING;

    tempBox = new Rectangle(startPoint);
}

@Override
public void onPrimaryDown() {
    toolState = ToolState.DOING;
}

@Override
public void onDrag() {

    if (allowReverseRectangle) {

        // ar reversu taisnsturi

        tempBox.width = Math.abs(mouseCurrPoint.x - mouseDownPoint.x);
        tempBox.height = Math.abs(mouseCurrPoint.y - mouseDownPoint.y);

        tempBox.x = Math.min(mouseCurrPoint.x, mouseDownPoint.x);
        tempBox.y = Math.min(mouseCurrPoint.y, mouseDownPoint.y);

        int minX = Math.min(mouseDownPoint.x, Math.min(mouseCurrPoint.x, mousePrevPoint.x));
        int minY = Math.min(mouseDownPoint.y, Math.min(mouseCurrPoint.y, mousePrevPoint.y));
        int maxX = Math.max(mouseDownPoint.x, Math.max(mouseCurrPoint.x, mousePrevPoint.x));
        int maxY = Math.max(mouseDownPoint.y, Math.max(mouseCurrPoint.y, mousePrevPoint.y));

        repaint(minX - 2, minY - 2, maxX - minX + 4, maxY - minY + 4);

    } else {

        tempBox.width = Math.max(globals.getBoxResizeWidthMin(), mouseCurrPoint.x - mouseDownPoint.x);
        tempBox.height = Math.max(globals.getBoxResizeHeightMin(), mouseCurrPoint.y - mouseDownPoint.y);

        repaint(tempBox.x - 2, tempBox.y - 2,
                Math.max(mouseCurrPoint.x, mousePrevPoint.x) - mouseDownPoint.x + 4,
                Math.max(mouseCurrPoint.y, mousePrevPoint.y) - mouseDownPoint.y + 4);

    }

}

@Override
public void onPrimaryUp() {

    toolState = ToolState.FINISHING;

    if (mouseHasBeenDragged) {
        tempBox.width = Math.max(tempBox.width, globals.getBoxResizeWidthMin());
        tempBox.height = Math.max(tempBox.height, globals.getBoxResizeHeightMin());
    } else {
        // kaste 0 x 0
    }


    repaint(tempBox.x - 2, tempBox.y - 2, tempBox.width + 4, tempBox.height + 4);

    finish();
}

@Override
public void onMove() {
}


}
