/*
*
* Copyright (c) 2013,2014 Institute of Mathematics and Computer Science, University of Latvia (IMCS UL). 
*
* This file is part of DEEK (diagram editor engine kit). 
*
* You can redistribute it and/or modify it under the terms of the GNU General Public License 
* as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
*
* You should have received a copy of the GNU General Public License along with DEEK If not, see http://www.gnu.org/licenses/.
*
*/

package lv.lumii.deek.ui.tools;

import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.event.MouseEvent;

import lv.lumii.deek.style.LineDecorationType;
import lv.lumii.deek.ui.DViewport;
import lv.lumii.deek.util.Globals;
import lv.lumii.deek.util.Drawer;

@SuppressWarnings("serial")
public class DNewLineTool
        extends DLineTool {

public DNewLineTool(DViewport owner) {
    this.owner = owner;

    initTool();
}

public DNewLineTool(DViewport owner, MouseEvent e) {
    this(owner);

    mousePressed(e);
}

private void initTool() {
    toolState = ToolState.DOING;
}

@Override
public void onPrimaryDown() {
    tempLine.add(new Point(mouseDownPoint));
}

@Override
public void onDrag() {
    if (toolState == ToolState.DOING) {
        repaint3(mouseDownPoint, mouseCurrPoint, mousePrevPoint);
    }
}

@Override
public void onPrimaryUp() {

    toolState = ToolState.FINISHING;

    tempLine.add(new Point(mouseUpPoint));

    repaint2(mouseDownPoint, mouseUpPoint);

    finish();

}

@Override
protected void finish() {
    owner.endNewLine(tempLine);
}

@Override
protected void abort() {
    owner.endNewLine(null);
}

@Override
protected void drawElement(Graphics2D g) {
    super.drawElement(g);
    drawTempSegment(g);
}

protected void drawTempSegment(Graphics2D g) {

    g.setColor(Globals.TOOL_BORDER_HILITE_COLOR);
    g.drawLine(mouseDownPoint.x, mouseDownPoint.y, mouseCurrPoint.x, mouseCurrPoint.y);

    if (drawEndArrow) {
        g.setPaint(Globals.TOOL_BORDER_HILITE_COLOR);
        Drawer.drawLineEndDecoration(g,
                mouseDownPoint.x, mouseDownPoint.y,
                mouseCurrPoint.x, mouseCurrPoint.y,
                LineDecorationType.FILLED_ARROW);

    }

}

}
