/*
*
* Copyright (c) 2013,2014 Institute of Mathematics and Computer Science, University of Latvia (IMCS UL). 
*
* This file is part of DEEK (diagram editor engine kit). 
*
* You can redistribute it and/or modify it under the terms of the GNU General Public License 
* as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
*
* You should have received a copy of the GNU General Public License along with DEEK If not, see http://www.gnu.org/licenses/.
*
*/

package lv.lumii.deek.ui.tools;

import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.event.MouseEvent;

import lv.lumii.deek.style.LineDecorationType;
import lv.lumii.deek.util.Globals;
import lv.lumii.deek.util.Drawer;
import lv.lumii.deek.ui.DViewport;

@SuppressWarnings("serial")
public class DNewOrthoLineTool extends DNewPolyLineTool {

private int tempCornerX, tempCornerY;
private HVDirection lastDirection;

public DNewOrthoLineTool(DViewport owner) {
    super(owner);
    initTool();
}

public DNewOrthoLineTool(DViewport owner, MouseEvent e) {
    this(owner);
    mousePressed(e);
}

private void initTool() {
    lastDirection = HVDirection.NONE;
}

@Override
public void onPrimaryDown() {
    calculateTempCorner();
    super.onPrimaryDown();
}

@Override
public void onDrag() {
    if (toolState == ToolState.DOING) {
        calculateTempCorner();
        repaint3(prevCornerPoint, mouseCurrPoint, mousePrevPoint);
    }
}

@Override
public void onSecondaryDown() {
    calculateTempCorner();

    if (tempCornerX == prevCornerPoint.x) {
        lastDirection = HVDirection.V;
    } else {
        lastDirection = HVDirection.H;
    }

    prevCornerPoint.setLocation(tempCornerX, tempCornerY);
    tempLine.add(new Point(tempCornerX, tempCornerY));
}

@Override
public void onPrimaryUp() {
    toolState = ToolState.FINISHING;

    calculateTempCorner();

    if (!isAutoTraceActive()) {
        tempLine.add(new Point(tempCornerX, tempCornerY));
    }
    tempLine.add(new Point(mouseUpPoint));


    repaint2(prevCornerPoint, mouseUpPoint);
    finish();
}

protected void calculateTempCorner() {

    // calculate temp points from prevCornerPoint and mouseCurrPoint
    switch (lastDirection) {
        case NONE:
            if (Math.abs(prevCornerPoint.x - mouseCurrPoint.x) > Math.abs(prevCornerPoint.y - mouseCurrPoint.y)) {
                tempCornerX = mouseCurrPoint.x;
                tempCornerY = prevCornerPoint.y;
            } else {
                tempCornerX = prevCornerPoint.x;
                tempCornerY = mouseCurrPoint.y;
            }
            break;

        case H:
            tempCornerX = prevCornerPoint.x;
            tempCornerY = mouseCurrPoint.y;
//			System.out.println("H");
            break;

        case V:
            tempCornerX = mouseCurrPoint.x;
            tempCornerY = prevCornerPoint.y;
            break;

        default:
            tempCornerX = prevCornerPoint.x;
            tempCornerY = mouseCurrPoint.y;
    }

}

private boolean isAutoTraceActive() {
    if ((lastDirection == HVDirection.NONE) && (shiftPressed))
        return true;
    return false;
}

protected void drawTempSegment(Graphics2D g) {

    g.setColor(Globals.TOOL_BORDER_HILITE_COLOR);

    if (isAutoTraceActive()) {

        g.drawLine(prevCornerPoint.x, prevCornerPoint.y, mouseCurrPoint.x, mouseCurrPoint.y);

        if (drawEndArrow) {
            g.setPaint(Globals.TOOL_BORDER_HILITE_COLOR);
            Drawer.drawLineEndDecoration(g,
                    prevCornerPoint.x, prevCornerPoint.y,
                    mouseCurrPoint.x, mouseCurrPoint.y,
                    LineDecorationType.FILLED_ARROW);
        }

    } else {

        g.drawLine(prevCornerPoint.x, prevCornerPoint.y, tempCornerX, tempCornerY);
        g.drawLine(tempCornerX, tempCornerY, mouseCurrPoint.x, mouseCurrPoint.y);

        if (drawEndArrow) {
            g.setPaint(Globals.TOOL_BORDER_HILITE_COLOR);
            Drawer.drawLineEndDecoration(g,
                    tempCornerX, tempCornerY,
                    mouseCurrPoint.x, mouseCurrPoint.y,
                    LineDecorationType.FILLED_ARROW);
        }

    }

}

@Override
protected void finish() {
    owner.endNewOrthoLine(tempLine);
}

@Override
protected void abort() {
    owner.endNewOrthoLine(null);
}

private enum HVDirection {H, V, NONE}


}
