/*
*
* Copyright (c) 2013,2014 Institute of Mathematics and Computer Science, University of Latvia (IMCS UL). 
*
* This file is part of DEEK (diagram editor engine kit). 
*
* You can redistribute it and/or modify it under the terms of the GNU General Public License 
* as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
*
* You should have received a copy of the GNU General Public License along with DEEK If not, see http://www.gnu.org/licenses/.
*
*/

package lv.lumii.deek.ui.tools;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Point;
import java.util.ArrayList;

public class DOrthoLine {
private ArrayList<Point> points;
private Point prevAnchor;
private Point nextAnchor;

public DOrthoLine() {
    points = new ArrayList<>();
}

public DOrthoLine(Point startPoint) {
    this();
    prevAnchor = startPoint;
    nextAnchor = startPoint;
}

public ArrayList<Point> moveNextAnchor(Point newPosition) {
    nextAnchor = newPosition;

    if (Math.abs(prevAnchor.x - nextAnchor.x) > Math.abs(prevAnchor.y - nextAnchor.y)) {
        nextAnchor.y = prevAnchor.y;
    } else {
        nextAnchor.x = prevAnchor.x;
    }
    System.out.println("anchor set to: " + nextAnchor.toString());

    return points;
}

public void confirmNextAnchor(Point nextPoint) {
    nextAnchor = nextPoint;
    confirmNextAnchor();
}

public void confirmNextAnchor() {
    if (points.size() == 0) {
        points.add(prevAnchor);
        points.add(nextAnchor);
        System.out.println("first points added: " + prevAnchor.toString() + nextAnchor.toString());
    } else {
        boolean isLastHorizontal = (points.get(points.size() - 1).y == points.get(points.size() - 2).y);
        boolean isNextHorizontal = (prevAnchor.y == nextAnchor.y);

        if (isLastHorizontal && isNextHorizontal) {
            points.get(points.size() - 1).x = nextAnchor.x;
            System.out.println("hor segment adapted");
        } else if (!isLastHorizontal && !isNextHorizontal) {
            points.get(points.size() - 1).y = nextAnchor.y;
            System.out.println("ver segment adapted");
        } else {
            points.add(nextAnchor);
            System.out.println("point added: " + nextAnchor.toString());
        }

        prevAnchor = nextAnchor;

        // TODO - collapse last 2 points if points(n-2) == points(n)
        // TODO - prevent tiny segments

    }
}

public void draw(Graphics2D g2) {
    this.draw(g2, Color.BLACK);
}

public void draw(Graphics2D g2, Color c) {
    if (points.size() > 1) {
        g2.setColor(c);

        Point prevPoint = points.get(0);
        for (int i = 1; i <= points.size() - 1; i++) {
            Point nextPoint = points.get(i);

            // draw segment from prevPoint to nextPoint
            g2.drawLine((int) prevPoint.x, (int) prevPoint.y, (int) nextPoint.x, (int) nextPoint.y);

            prevPoint = nextPoint;
        }
    }
    // draw pseudo-segment from prevAnchor to nextAnchor
    g2.setColor(Color.RED);
    g2.drawLine((int) prevAnchor.x, (int) prevAnchor.y, (int) nextAnchor.x, (int) nextAnchor.y);
}

public ArrayList<Point> getPoints() {
    return points;
}
}
