/*
*
* Copyright (c) 2013,2014 Institute of Mathematics and Computer Science, University of Latvia (IMCS UL). 
*
* This file is part of DEEK (diagram editor engine kit). 
*
* You can redistribute it and/or modify it under the terms of the GNU General Public License 
* as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
*
* You should have received a copy of the GNU General Public License along with DEEK If not, see http://www.gnu.org/licenses/.
*
*/

package lv.lumii.deek.util;

import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.List;

import lv.lumii.deek.model.DLine;
import lv.lumii.layoutengine.Line;
import lv.lumii.deek.model.DBox;
import lv.lumii.deek.model.DContainer;
import lv.lumii.deek.model.DElement;


/**
 * Helper class for logging operations.
 */
public class DLogger {

private boolean localLoggingOn = false;

public boolean isLoggingOn() {
    return Globals.debugOutputIsOn;
}

public void setLoggingOn(boolean debug) {
    Globals.debugOutputIsOn = debug;
}

public void outputToLog(String s) {
    if (isLoggingOn() || isLocalLoggingOn()) {
        Globals.outputToLog(s);
    }
}

public void appendToLog(String s) {
    if (isLoggingOn() || isLocalLoggingOn()) {
        Globals.appendToLog(s);
    }
}

public boolean isLocalLoggingOn() {
    return localLoggingOn;
}

public void setLocalLoggingOn(boolean debug) {
    localLoggingOn = debug;
}

public String dumpLinePoints(List<Point2D.Double> points) {
    if (points == null)
        return "(line has no points yet)";

    StringBuilder s = new StringBuilder("[");

    for (Point2D.Double p : points) {
        s.append(String.format("(%.1f %.1f)", p.getX(), p.getY()));
    }
    s.append("]");
    return s.toString();
}

public String dumpLine(DLine l) {
    return dumpLinePoints(l.getPoints());
}

public String dumpLine(Line l) {
    return dumpLinePoints(l.getPoints());
}

public String dumpContainer(DContainer b) {
    if (b == null)
        return "[null]";
    return dumpRectangle(b.getBounds());
}

public String dumpRectangle(Rectangle2D r) {
    if (r == null)
        return "[null]";

    return String.format("[x:%.1f y:%.1f width:%.1f height:%.1f]",
            r.getX(), r.getY(),
            r.getWidth(), r.getHeight());
}

public String dumpElement(DElement e) {
    if (e == null)
        return "[null]";
    if (e instanceof DBox) {
        return dumpContainer((DBox) e);
    } else if (e instanceof DLine) {
        return dumpLine((DLine) e);
    }
    return "[none]";
}

public String dumpPoint(Point2D p) {
    if (p == null)
        return "[null]";
    return String.format("[x:%.1f y:%.1f]",
            p.getX(), p.getY());
}

}
