/*
*
* Copyright (c) 2013,2014 Institute of Mathematics and Computer Science, University of Latvia (IMCS UL). 
*
* This file is part of DEEK (diagram editor engine kit). 
*
* You can redistribute it and/or modify it under the terms of the GNU General Public License 
* as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
*
* You should have received a copy of the GNU General Public License along with DEEK If not, see http://www.gnu.org/licenses/.
*
*/

package lv.lumii.deek.util;

import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.font.FontRenderContext;
import java.awt.geom.AffineTransform;
import java.util.ArrayList;

import lv.lumii.deek.util.Globals;

/**
 * Helper class for some convenience operations.
 */
public abstract class DUtils {

private static ArrayList<String> phraseList = new ArrayList<>();

static {
    phraseList.add("Nulla massa eros, feugiat vitae gravida et");
    phraseList.add("vehicula laoreet neque");
    phraseList.add("Cras ipsum felis, lacinia ut mollis ac");
    phraseList.add("accumsan eget sapien");
    phraseList.add("Aliquam fringilla mauris id lorem vulputate");
    phraseList.add("vel iaculis lectus consectetur");
    phraseList.add("Class aptent taciti sociosqu ad litora");
    phraseList.add("torquent per conubia nostra");
    phraseList.add("per inceptos himenaeos");
    phraseList.add("Vivamus posuere, justo vel tempor gravida");
    phraseList.add("mauris massa commodo nisi");
    phraseList.add("nec mattis metus dui nec justo");
    phraseList.add("Donec tristique pulvinar vulputate");
    phraseList.add("Donec dignissim, ligula quis fringilla aliquam");
    phraseList.add("nunc arcu vehicula eros, eget ullamcorper libero");
    phraseList.add("orci faucibus turpis");
}

private static ArrayList<String> wordList = new ArrayList<>();

static {
    wordList.add("red");
    wordList.add("blue");
    wordList.add("yellow");
    wordList.add("green");
    wordList.add("white");
    wordList.add("black");
    wordList.add("orange");
    wordList.add("cyan");
    wordList.add("magenta");
    wordList.add("pink");
    wordList.add("pale");
    wordList.add("beige");
    wordList.add("chocolate");
    wordList.add("champagne");
    wordList.add("brown");
    wordList.add("azure");
    wordList.add("gray");
    wordList.add("gold");
    wordList.add("silver");
    wordList.add("lime");
    wordList.add("navy");
}

/**
 * Calculate the dimensions of the given text
 *
 * @param g    graphics context
 * @param text text
 * @return
 */
public static BlockSize measureText(Graphics2D g, String text) {

    Font font = g.getFont();
    FontMetrics metrics = g.getFontMetrics(font);

    int h = metrics.getHeight();
    int adv = metrics.stringWidth(text);

    return new BlockSize(adv + 2, h + 2);
}

/**
 * Calculate the dimensions of the given text
 *
 * @param font font to be used
 * @param text text
 * @return
 */
public static BlockSize measureText2(Font font, String text) {
    AffineTransform affineTransform = new AffineTransform();
    FontRenderContext frc = new FontRenderContext(affineTransform, true, true);
    double textWidth = font.getStringBounds(text, frc).getWidth();
    double textHeight = font.getStringBounds(text, frc).getHeight();
    return new BlockSize(textWidth, textHeight);
}

/**
 * Returns a random integer between min and max.
 *
 * @param min
 * @param max
 * @return
 */
public static int getRandomIndex(int min, int max) {
    // return min + (int)(Math.random() * (max - min + 1));
    return min + Globals.randomGenerator.nextInt(max - min) + 1;
}

/**
 * Returns short random phrase from the internal phrase collection.
 *
 * @return phrase
 */
public static String getRandomPhrase() {
    return phraseList.get(Globals.randomGenerator.nextInt(phraseList.size()));
}

/**
 * Returns random word from the internal word collection.
 *
 * @return word
 */
public static String getRandomWord() {
    return wordList.get(Globals.randomGenerator.nextInt(wordList.size()));
}


}
