/*
*
* Copyright (c) 2013,2014 Institute of Mathematics and Computer Science, University of Latvia (IMCS UL). 
*
* This file is part of DEEK (diagram editor engine kit). 
*
* You can redistribute it and/or modify it under the terms of the GNU General Public License 
* as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
*
* You should have received a copy of the GNU General Public License along with DEEK If not, see http://www.gnu.org/licenses/.
*
*/

package lv.lumii.deek.util;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.Paint;
import java.awt.Stroke;
import java.awt.geom.Rectangle2D;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Random;
import java.util.logging.Logger;
import java.util.prefs.Preferences;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

public class Globals {

// tolerance for hitting box borders (and line segments)
public static final double HIT_EPSILON = 5.0;

//-------------------------------
public static final int SCREEN_HIT_EPSILON = 5;
// styling
public static final Color NORMAL_FILL_COLOR = Color.YELLOW;
public static final Color NORMAL_TEXT_COLOR = Color.GRAY;
public static final Color AS_FLOW_FILL_COLOR = new Color(1.0f, 1.0f, 0.7f);
public static final Color AS_SPRING_FILL_COLOR = new Color(1.0f, 0.8f, 1.0f);
public static final Color AS_UNIVERSAL_FILL_COLOR = new Color(0.7f, 1.0f, 1.0f);
public static final Color AS_INHERITED_FILL_COLOR = new Color(0.8f, 0.8f, 0.8f);
public static final Color AS_NONE_FILL_COLOR = new Color(1.0f, 1.0f, 1.0f);
public static final Color BORDER_COLOR = Color.BLACK;
public final static float BOX_SHADOW_ALPHA = 0.4f;
public static final Color SHADOW_COLOR = new Color(0f, 0f, 0f, BOX_SHADOW_ALPHA);
public static final Color HOVER_COLOR = Color.PINK;
public static final Color CURRENT_COLOR = Color.RED;
//	public static final Color LINE_COLOR = Color.GREEN;
public static final Color LINE_COLOR = new Color(0.0f, 0.2f, 0.0f);
public static final Color LINE_COLOR_ORTHO = new Color(0.2f, 0.0f, 0.2f);
public static final Color LINE_COLOR_STRAIGHT = new Color(0.0f, 0.2f, 0.2f);
public static final Color LINE_COLOR_POLY = new Color(0.2f, 0.2f, 0.0f);
public static final Color LINE_COLOR_PANTOM = new Color(0.5f, 0.5f, 0.5f);
public static final Stroke LINE_STROKE_POLY = new BasicStroke(
        1.0f,
//			BasicStroke.CAP_BUTT,
        BasicStroke.CAP_ROUND,
//			BasicStroke.JOIN_MITER,
        BasicStroke.JOIN_ROUND,
        2.0f);
public static final Stroke LINE_STROKE_ORTHO = LINE_STROKE_POLY;
public static final Stroke LINE_STROKE_STRAIGHT = LINE_STROKE_POLY;
public static final Stroke LINE_STROKE_PHANTOM = new BasicStroke(
        1.0f, BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER,
        2.0f, new float[]{2.0f, 5.0f}, 0.0f);
public static final Paint TOOL_FILL_PAINT = new Color(0.7f, 1.0f, 0.7f, 0.3f);
public static final Color TOOL_BORDER_COLOR = new Color(0.5f, 0.8f, 0.5f, 0.5f);
public static final Stroke TOOL_BORDER_STROKE = new BasicStroke(3f);
public static final Color TOOL_BORDER_HILITE_COLOR = new Color(0.8f, 0.5f, 0.5f, 0.5f);
public static final Color GRID_BORDER_COLOR = new Color(0.8f, 0.2f, 0.8f);
//	public void setBoxAutosizeDefaultWidth(double boxAutosizeDefaultWidth) { this.boxAutosizeDefaultWidth = boxAutosizeDefaultWidth; }
public static final Stroke GRID_BORDER_STROKE = new BasicStroke(
        1.0f, BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER,
        2.0f, new float[]{5.0f}, 0.0f);
//	public void setBoxAutosizeDefaultHeight(double boxAutosizeDefaultHeight) { this.boxAutosizeDefaultHeight = boxAutosizeDefaultHeight; }
public static final Color LABEL_COLOR = new Color(1.0f, 1.0f, 1.0f);
public static final Color LABEL_BORDER_COLOR = new Color(0.0f, 0.0f, 0.0f);
public static final Color LABEL_BORDER_COLOR_PHANTOM = new Color(0.7f, 0.7f, 0.7f);
public static final double WORLD_PADDING = 20.0;
// ui
public static final int OVERVIEW_PANE_SIZE = 150;
// other
public static final int NUM_GENERATED_BOXES = 100;
// randomizer
public static final long DEFAULT_RANDOM_SEED = 42L;
public static final boolean RANDOMIZE_AT_START = false; // false gives repeatable runs
public static final int DEFAULT_ANIMATION_STEPS = 100;
public static final int ANIMATION_STEP_DELAY = 150;
public static final int DEFAULT_BOX_GENERATION_COUNT = 50;
public static final int DEFAULT_LINE_GENERATION_COUNT = 50;
public static final int DEFAULT_LABEL_GENERATION_COUNT = 25;
public static final Rectangle2D.Double defaultGeneratorArea = new Rectangle2D.Double(0.0, 0.0, 1024.0, 768.0);
public static final Font infoFont = new Font("SansSerif", Font.PLAIN, 10);
public static final int textNormalFontSize = 12;
public static final Font textNormalFont = new Font("Dialog", Font.PLAIN, textNormalFontSize);
public static final Font textNormalFontBold = new Font("Dialog", Font.BOLD, textNormalFontSize);
public static final int textSmallFontSize = 10;
public static final Font textSmallFont = new Font("Dialog", Font.PLAIN, textSmallFontSize);
public final static int BOX_ROUNDING = 18;
public final static int BOX_SHADOW_SHIFT = 7;
private static final double LABEL_AUTOSIZE_WIDTH = 25;
private static final double LABEL_AUTOSIZE_HEIGHT = 10;
private static final double DEFAULTBOXCONTENTWIDTHMIN = 20.0;
private static final double DEFAULTBOXCONTENTWIDTHDELTA = 50.0;
private static final double DEFAULTBOXCONTENTHEIGHTMIN = 15.0;
private static final double DEFAULTBOXCONTENTHEIGHTDELTA = 40.0;
private static final double RESIZE_BOX_STEP = 30.0;
private static final double RESIZE_BOX_RATIO = 1.25;
private static final double INITIAL_BOX_SPACING = 10.0;
private static final double INITIAL_LINE_SPACING = 5.0;
private static final double SPACING_STEP = 7.0;
private static final double SPACING_MIN_VALUE = 1.0;
private static final double LINE_CORNER_ROUNDING = 2.0;
private static final String PREFS_ROOT = "lv.lumii.deek";
public static boolean useIntegerDraw = false;
public static double resizeBoxStep = RESIZE_BOX_STEP;
public static double resizeBoxRatio = RESIZE_BOX_RATIO;
public static double spacingStep = SPACING_STEP;
public static double spacingMin = SPACING_MIN_VALUE;
public static double lineCornerRounding = LINE_CORNER_ROUNDING;
public static long randomSeed = RANDOMIZE_AT_START ? new Random().nextLong() : DEFAULT_RANDOM_SEED;
public static Random randomGenerator = new Random(randomSeed);
public static Logger logger = Logger.getGlobal();
public static boolean debugOutputIsOn = false;
private static boolean f_devMode = true;
private static double currentBoxSpacing = INITIAL_BOX_SPACING;
private static double currentLineSpacing = INITIAL_LINE_SPACING;
private static long f_lastUuid = 0;
// -- logging --
private static int logCounter = 0;
private static Globals thisGlobals = new Globals();
private static Preferences prefs;
private static Map<String, Object> defaultValues;

static {
    defaultValues = new LinkedHashMap<>();
    prefs = Preferences.userRoot().node(PREFS_ROOT);
    setupDefaults();
}

public double labelAutosizeWidth = LABEL_AUTOSIZE_WIDTH;
public double labelAutosizeHeight = LABEL_AUTOSIZE_HEIGHT;
public double defaultBoxContentWidthMin = DEFAULTBOXCONTENTWIDTHMIN;
public double defaultBoxContentWidthDelta = DEFAULTBOXCONTENTWIDTHDELTA;
public double defaultBoxContentHeightMin = DEFAULTBOXCONTENTHEIGHTMIN;
public double defaultBoxContentHeightDelta = DEFAULTBOXCONTENTHEIGHTDELTA;
public int animationSteps = DEFAULT_ANIMATION_STEPS;
public int boxGenerationCount = DEFAULT_BOX_GENERATION_COUNT;
public int lineGenerationCount = DEFAULT_LINE_GENERATION_COUNT;
public int labelGenerationCount = DEFAULT_LABEL_GENERATION_COUNT;
public int animationStepsLeft = 0;
public boolean generateTreeFromRoot = false;
// geometry
private int boxResizeWidthMin = 20;
private int boxResizeHeightMin = 20;
private double boxAutosizeWidthMin = 40.0;
private double boxAutosizeWidthDelta = 60.0;
private double boxAutosizeHeightMin = 30.0;
private double boxAutosizeHeightDelta = 50.0;

private Globals() {
}

private static void setupDefaults() {
    defaultValues.put("aaa", 4);
}

public static boolean isDeveloperMode() {
    return f_devMode;
}

public static void setDeveloperMode(boolean isMode) {
    f_devMode = isMode;
}

public static double getCurrentBoxSpacing() {
    return currentBoxSpacing;
}

public static void setCurrentBoxSpacing(double currentBoxSpacing) {
    Globals.currentBoxSpacing = currentBoxSpacing;
}

public static double getCurrentLineSpacing() {
    return currentLineSpacing;
}

public static void setCurrentLineSpacing(double currentLineSpacing) {
    Globals.currentLineSpacing = currentLineSpacing;
}

public static void resetRandom() {
    randomGenerator.setSeed(randomSeed);
}

public static void randomize() {
    Random r = new Random();
    randomSeed = r.nextLong();

    randomGenerator.setSeed(randomSeed);
}

public static long getSeed() {
    return SeedGetter.getSeed(randomGenerator);
}

public static long nextUuid() {
    return ++f_lastUuid;
}

public static void outputToLog(String s) {
    ++logCounter;
    System.out.println(String.format("%8d # %s", logCounter, s));
}

public static void outputToLog(Class c, String s) {
//		Logger logger = LoggerFactory.getLogger(c);
    ++logCounter;
    System.out.println(String.format("%8d # %s", logCounter, s));
}

public static void appendToLog(String s) {
    System.out.println(String.format("         # %s", s));
}
//	public static boolean debugOutputIsOn = true;

public static void appendToLog(Class c, String s) {
    System.out.println(String.format("         # %s", s));
}

public static void resetLogCounter() {
    logCounter = 0;
}

public static Globals getInstance() {
    return thisGlobals;
}

public int getBoxResizeWidthMin() {
    return boxResizeWidthMin;
}

public void setBoxResizeWidthMin(int boxResizeWidthMin) {
    this.boxResizeWidthMin = boxResizeWidthMin;
}

public int getBoxResizeHeightMin() {
    return boxResizeHeightMin;
}

public void setBoxResizeHeightMin(int boxResizeHeightMin) {
    this.boxResizeHeightMin = boxResizeHeightMin;
}

public double getBoxAutosizeWidthMin() {
    return boxAutosizeWidthMin;
}

public void setBoxAutosizeWidthMin(double boxAutosizeWidthMin) {
    this.boxAutosizeWidthMin = Math.max(10.0, boxAutosizeWidthMin);
}

public double getBoxAutosizeWidthDelta() {
    return this.boxAutosizeWidthDelta;
}

public void setBoxAutosizeWidthDelta(double boxAutosizeWidthDelta) {
    this.boxAutosizeWidthDelta = Math.max(0.0, boxAutosizeWidthDelta);
}

public double getBoxAutosizeWidthMax() {
    return this.boxAutosizeWidthMin + this.boxAutosizeWidthDelta;
}

public void setBoxAutosizeWidthMax(double boxAutosizeWidthMax) {
    this.boxAutosizeWidthDelta = (boxAutosizeWidthMax > this.boxAutosizeWidthMin) ?
            boxAutosizeWidthMax - this.boxAutosizeWidthMin : 0;
}

public double getBoxAutosizeHeightMin() {
    return boxAutosizeHeightMin;
}

public void setBoxAutosizeHeightMin(double boxAutosizeHeightMin) {
    this.boxAutosizeHeightMin = Math.max(10.0, boxAutosizeHeightMin);
}

public double getBoxAutosizeHeightDelta() {
    return this.boxAutosizeHeightDelta;
}

public void setBoxAutosizeHeightDelta(double boxAutosizeHeightDelta) {
    this.boxAutosizeHeightDelta = Math.max(0.0, boxAutosizeHeightDelta);
}

public double getBoxAutosizeHeightMax() {
    return this.boxAutosizeHeightMin + this.boxAutosizeHeightDelta;
}

public void setBoxAutosizeHeightMax(double boxAutosizeHeightMax) {
    this.boxAutosizeHeightDelta = (boxAutosizeHeightMax > this.boxAutosizeHeightMin) ?
            boxAutosizeHeightMax - this.boxAutosizeHeightMin : 0;
}

//	private static final double BOX_AUTOSIZE_DEFAULT_WIDTH = boxAutosizeMinWidth + 0.5 * boxAutosizeWidthDelta;
//	private double boxAutosizeDefaultWidth = boxAutosizeMinWidth + 0.5 * boxAutosizeWidthDelta;
public double getBoxAutosizeDefaultWidth() {
    return boxAutosizeWidthMin + 0.5 * boxAutosizeWidthDelta;
}

//	private static final double BOX_AUTOSIZE_DEFAULT_HEIGHT = BOX_AUTOSIZE_MIN_HEIGHT + 0.5 * BOX_AUTOSIZE_HEIGHT_DELTA;
//	private double boxAutosizeDefaultHeight = boxAutosizeMinHeight + 0.5 * boxAutosizeHeightDelta;
public double getBoxAutosizeDefaultHeight() {
    return boxAutosizeHeightMin + 0.5 * boxAutosizeHeightDelta;
}

public void showDialog() {

    JTextField f_minw = new JTextField(5);
    f_minw.setText(Double.toString(getBoxAutosizeWidthMin()));
    JTextField f_maxw = new JTextField(5);
    f_maxw.setText(Double.toString(getBoxAutosizeWidthMax()));
    JTextField f_minh = new JTextField(5);
    f_minh.setText(Double.toString(getBoxAutosizeHeightMin()));
    JTextField f_maxh = new JTextField(5);
    f_maxh.setText(Double.toString(getBoxAutosizeHeightMax()));


    JPanel fields = new JPanel();
    fields.setLayout(new BoxLayout(fields, BoxLayout.Y_AXIS));

    fields.add(new JLabel("min platums:"));
    fields.add(f_minw);
    fields.add(Box.createHorizontalStrut(5));

    fields.add(new JLabel("max platums:"));
    fields.add(f_maxw);
    fields.add(Box.createHorizontalStrut(5));

    fields.add(new JLabel("min augstums:"));
    fields.add(f_minh);
    fields.add(Box.createHorizontalStrut(5));

    fields.add(new JLabel("max augstums:"));
    fields.add(f_maxh);

    int result = JOptionPane.showConfirmDialog(null, fields, "Options", JOptionPane.OK_CANCEL_OPTION);

    if (result == JOptionPane.OK_OPTION) {
        setBoxAutosizeWidthMin(Double.parseDouble(f_minw.getText()));
        setBoxAutosizeWidthMax(Double.parseDouble(f_maxw.getText()));

        setBoxAutosizeHeightMin(Double.parseDouble(f_minh.getText()));
        setBoxAutosizeHeightMax(Double.parseDouble(f_maxh.getText()));
    }
}

// -----------------------------------------------
public int getInt(String key) {
    return prefs.getInt(key, getDefaultInt(key));
}

public boolean getBoolean(String key) {
    return prefs.getBoolean(key, getDefaultBoolean(key));
}

public double getDouble(String key) {
    return prefs.getDouble(key, getDefaultDouble(key));
}

public String get(String key) {
    return prefs.get(key, "");
}

public void setInt(String key, int value) {
    prefs.putInt(key, value);
}

public void setBoolean(String key, boolean value) {
    prefs.putBoolean(key, value);
}

public void setDouble(String key, double value) {
    prefs.putDouble(key, value);
}

public void set(String key, String value) {
    prefs.put(key, value);
}

private boolean getDefaultBoolean(String key) {
    Object value = defaultValues.get(key);
    if (value instanceof Boolean)
        return (Boolean) value;

    return true;
}

private int getDefaultInt(String key) {
    Object value = defaultValues.get(key);
    if (value instanceof Integer)
        return (Integer) value;

    return 0;
}

private double getDefaultDouble(String key) {
    Object value = defaultValues.get(key);
    if (value instanceof Double)
        return (Double) value;

    return 1.0;
}

}

// ----------------- don't look below this line -----------------

class SeedGetter {
static long getSeed(Random random) {
    byte[] ba0, ba1, bar;
    try {
        ByteArrayOutputStream baos = new ByteArrayOutputStream(128);
        ObjectOutputStream oos = new ObjectOutputStream(baos);
        oos.writeObject(new Random(0));
        ba0 = baos.toByteArray();
        baos = new ByteArrayOutputStream(128);
        oos = new ObjectOutputStream(baos);
        oos.writeObject(new Random(-1));
        ba1 = baos.toByteArray();
        baos = new ByteArrayOutputStream(128);
        oos = new ObjectOutputStream(baos);
        oos.writeObject(random);
        bar = baos.toByteArray();
    } catch (IOException e) {
        throw new RuntimeException("IOException: " + e);
    }
    if (ba0.length != ba1.length || ba0.length != bar.length)
        throw new RuntimeException("bad serialized length");
    int i = 0;
    while (i < ba0.length && ba0[i] == ba1[i]) {
        i++;
    }
    int j = ba0.length;
    while (j > 0 && ba0[j - 1] == ba1[j - 1]) {
        j--;
    }
    if (j - i != 6)
        throw new RuntimeException("6 differing bytes not found");
    // The constant 0x5DEECE66DL is from
    // http://download.oracle.com/javase/6/docs/api/java/util/Random.html .
    return ((bar[i] & 255L) << 40 | (bar[i + 1] & 255L) << 32 |
            (bar[i + 2] & 255L) << 24 | (bar[i + 3] & 255L) << 16 |
            (bar[i + 4] & 255L) << 8 | (bar[i + 5] & 255L)) ^ 0x5DEECE66DL;
}
}