/*
*
* Copyright (c) 2013,2014 Institute of Mathematics and Computer Science, University of Latvia (IMCS UL). 
*
* This file is part of DEEK (diagram editor engine kit). 
*
* You can redistribute it and/or modify it under the terms of the GNU General Public License 
* as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
*
* You should have received a copy of the GNU General Public License along with DEEK If not, see http://www.gnu.org/licenses/.
*
*/

package lv.lumii.test;

import java.awt.*;
import java.awt.geom.Rectangle2D;
import java.util.Locale;

import javax.swing.*;

import lv.lumii.deek.ui.*;
import lv.lumii.deek.ui.OpusMagnifier;

@SuppressWarnings("serial")
public class OpusFrame
        extends JFrame
        implements OpusHost {

OpusStatusBar statusbar;
OpusToolbar toolbar;
JMenuBar menubar;
private DController controller;
private DViewport viewPort;
private DDrawing drawing;
private OpusOverview overviewPane;
private OpusMagnifier magnifierPane;

public OpusFrame() {
    initComponents();
    recalculateLayout();

}

private void initComponents() {
    setTitle("OpusApp");

    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
    d.height -= 200;
    d.width -= 100;
    setSize(d);
//		setSize(800, 600);

    setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);

    Locale.setDefault(Locale.US);

    statusbar = new OpusStatusBar(this);
    add(statusbar, BorderLayout.SOUTH);

    drawing = new DDrawing();
    viewPort = new DViewport(this, drawing);
    controller = new DController(this, drawing, viewPort);
    drawing.setController(controller);

    // for the toolbar placement
    JPanel inner = new JPanel(new BorderLayout());

    toolbar = new OpusToolbar(this);
    controller.addToolbarButtons(toolbar);
    inner.add(toolbar, BorderLayout.SOUTH);

    inner.add(viewPort, BorderLayout.CENTER);
//		inner.add(new JScrollPane(viewPort), BorderLayout.CENTER);

    add(inner, BorderLayout.CENTER);

    menubar = controller.createMainMenu();
//		setJMenuBar(menubar);

    overviewPane = new OpusOverview(drawing, this);
//		overviewPane.setVisible(true);

    magnifierPane = new OpusMagnifier(drawing, this);
//		magnifierPane.setVisible(true);

}

public DViewport getViewport() {
    return viewPort;
}

public void setStatusText(String s) {
    if (statusbar != null)
        statusbar.setText(s);
    else
        setTitle(s);
}

public void toggleMenuBar() {
    if (getJMenuBar() != null)
        setJMenuBar(null);
    else
        setJMenuBar(menubar);

    this.repaint();
}

@Override
public void toggleToolBar() {
}

@Override
public void toggleStatusBar() {
}

@Override
public void toggleOverview() {
    if (overviewPane.isVisible()) {
        overviewPane.setVisible(false);
    } else {
        overviewPane.setVisible(true);
        toFront();
    }
}

@Override
public void toggleMagnifier() {
    if (magnifierPane.isVisible()) {
        magnifierPane.setVisible(false);
    } else {
        magnifierPane.setVisible(true);
        toFront();
    }
}

@Override
public OpusOverview getOverview() {
    return overviewPane;
}

@Override
public OpusMagnifier getMagnifier() {
    return magnifierPane;
}

private void recalculateLayout() {
    // to be called on each frame resize etc.

}
}

class DOverviewMock extends JPanel {
@Override
public void paintComponent(Graphics g) {
    Rectangle2D r = this.getBounds();
    g.drawLine((int) r.getMinX(), (int) r.getMinY(), (int) r.getMaxX(), (int) r.getMaxY());
    g.drawLine((int) r.getMaxX(), (int) r.getMaxY(), (int) r.getMinX(), (int) r.getMinY());
}
}
